@@ -59,6 +59,7 @@
 #include "src/common/xsignal.h"
 #include "src/common/xstring.h"
 
+#include "src/interfaces/conn.h"
 #include "src/interfaces/cred.h"
 
 #define STDIO_MAX_FREE_BUF 1024
@@ -79,7 +80,8 @@ static struct io_buf *_alloc_io_buf(void);
 static void	_init_stdio_eio_objs(slurm_step_io_fds_t fds,
 				     client_io_t *cio);
 static void	_handle_io_init_msg(int fd, client_io_t *cio);
-static int _read_io_init_msg(int fd, client_io_t *cio, slurm_addr_t *host);
+static int _read_io_init_msg(int fd, void *conn, client_io_t *cio,
+			     slurm_addr_t *host);
 static int      _wid(int n);
 static bool     _incoming_buf_free(client_io_t *cio);
 static bool     _outgoing_buf_free(client_io_t *cio);
@@ -217,9 +219,9 @@ _set_listensocks_nonblocking(client_io_t *cio)
 /**********************************************************************
  * IO server socket functions
  **********************************************************************/
-static eio_obj_t *
-_create_server_eio_obj(int fd, client_io_t *cio, int nodeid,
-		       int stdout_objs, int stderr_objs)
+static eio_obj_t *_create_server_eio_obj(int fd, void *conn, client_io_t *cio,
+					 int nodeid, int stdout_objs,
+					 int stderr_objs)
 {
 	eio_obj_t *eio = NULL;
 	struct server_io_info *info = xmalloc(sizeof(*info));
@@ -239,6 +241,7 @@ _create_server_eio_obj(int fd, client_io_t *cio, int nodeid,
 
 	net_set_keep_alive(fd);
 	eio = eio_obj_create(fd, &server_ops, (void *)info);
+	eio->conn = conn;
 
 	return eio;
 }
@@ -298,7 +301,7 @@ static int _server_read(eio_obj_t *obj, list_t *objs)
 			return SLURM_ERROR;
 		}
 
-		n = io_hdr_read_fd(obj->fd, &s->header);
+		n = io_hdr_read_fd(obj->fd, obj->conn, &s->header);
 		if (n <= 0) { /* got eof or error on socket read */
 			if (n < 0) {	/* Error */
 				if (obj->shutdown) {
@@ -367,7 +370,12 @@ static int _server_read(eio_obj_t *obj, list_t *objs)
 	if (s->header.length != 0) {
 		buf = s->in_msg->data + (s->in_msg->length - s->in_remaining);
 	again:
-		if ((n = read(obj->fd, buf, s->in_remaining)) < 0) {
+		if (obj->conn) {
+			n = conn_g_recv(obj->conn, buf, s->in_remaining);
+		} else {
+			n = read(obj->fd, buf, s->in_remaining);
+		}
+		if (n < 0) {
 			if (errno == EINTR)
 				goto again;
 			if ((errno == EAGAIN) || (errno == EWOULDBLOCK))
@@ -491,7 +499,12 @@ static int _server_write(eio_obj_t *obj, list_t *objs)
 	 */
 	buf = s->out_msg->data + (s->out_msg->length - s->out_remaining);
 again:
-	if ((n = write(obj->fd, buf, s->out_remaining)) < 0) {
+	if (obj->conn) {
+		n = conn_g_send(obj->conn, buf, s->out_remaining);
+	} else {
+		n = write(obj->fd, buf, s->out_remaining);
+	}
+	if (n < 0) {
 		if (errno == EINTR) {
 			goto again;
 		} else if ((errno == EAGAIN) || (errno == EWOULDBLOCK)) {
@@ -787,7 +800,7 @@ again:
 
 
 /**********************************************************************
- * General fuctions
+ * General functions
  **********************************************************************/
 
 static void *
@@ -832,11 +845,12 @@ _create_listensock_eio(int fd, client_io_t *cio)
 	return eio;
 }
 
-static int _read_io_init_msg(int fd, client_io_t *cio, slurm_addr_t *host)
+static int _read_io_init_msg(int fd, void *conn, client_io_t *cio,
+			     slurm_addr_t *host)
 {
 	io_init_msg_t msg = { 0 };
 
-	if (io_init_msg_read_from_fd(fd, &msg) != SLURM_SUCCESS)
+	if (io_init_msg_read_from_fd(fd, conn, &msg) != SLURM_SUCCESS)
 		goto fail;
 
 	if (io_init_msg_validate(&msg, cio->io_key) < 0) {
@@ -858,9 +872,10 @@ static int _read_io_init_msg(int fd, client_io_t *cio, slurm_addr_t *host)
 		error("IO: Hey, you told me node %d was down!", msg.nodeid);
 	}
 
-	cio->ioserver[msg.nodeid] = _create_server_eio_obj(fd, cio, msg.nodeid,
-							   msg.stdout_objs,
-							   msg.stderr_objs);
+	cio->ioserver[msg.nodeid] =
+		_create_server_eio_obj(fd, conn, cio, msg.nodeid,
+				       msg.stdout_objs, msg.stderr_objs);
+
 	slurm_mutex_lock(&cio->ioservers_lock);
 	bit_set(cio->ioservers_ready_bits, msg.nodeid);
 	cio->ioservers_ready = bit_set_count(cio->ioservers_ready_bits);
@@ -879,6 +894,7 @@ static int _read_io_init_msg(int fd, client_io_t *cio, slurm_addr_t *host)
 	return SLURM_SUCCESS;
 
     fail:
+	conn_g_destroy(conn, false);
 	xfree(msg.io_key);
 	if (fd > STDERR_FILENO)
 		close(fd);
@@ -909,6 +925,7 @@ _handle_io_init_msg(int fd, client_io_t *cio)
 
 	for (j = 0; j < 15; j++) {
 		int sd;
+		void *conn = NULL;
 		slurm_addr_t addr;
 
 		/*
@@ -917,7 +934,7 @@ _handle_io_init_msg(int fd, client_io_t *cio)
 		if (!_is_fd_ready(fd))
 			return;
 
-		while ((sd = slurm_accept_msg_conn(fd, &addr)) < 0) {
+		while (!(conn = slurm_accept_msg_conn(fd, &addr))) {
 			if (errno == EINTR)
 				continue;
 			if (errno == EAGAIN)	/* No more connections */
@@ -930,6 +947,7 @@ _handle_io_init_msg(int fd, client_io_t *cio)
 			return;
 		}
 
+		sd = conn_g_get_fd(conn);
 		debug3("Accepted IO connection: ip=%pA sd=%d", &addr, sd);
 
 		/*
@@ -944,7 +962,7 @@ _handle_io_init_msg(int fd, client_io_t *cio)
 		/*
 		 * Read IO header and update cio structure appropriately
 		 */
-		if (_read_io_init_msg(sd, cio, &addr) < 0)
+		if (_read_io_init_msg(sd, conn, cio, &addr) < 0)
 			continue;
 
 		fd_set_nonblocking(sd);
