@@ -1,8 +1,7 @@
 /*****************************************************************************\
  *  fetch_config.h - functions for "configless" slurm operation
  *****************************************************************************
- *  Copyright (C) 2020 SchedMD LLC.
- *  Written by Tim Wickberg <tim@schedmd.com>
+ *  Copyright (C) SchedMD LLC.
  *
  *  This file is part of Slurm, a resource management program.
  *  For details, see <https://slurm.schedmd.com/>.
@@ -41,14 +40,16 @@
 
 typedef struct {
 	char *conf_file;
-	List include_list;
+	list_t *include_list;
 } conf_includes_map_t;
 
-extern List conf_includes_list;
+extern list_t *conf_includes_list;
 
-extern config_response_msg_t *fetch_config(char *conf_server, uint32_t flags);
+extern config_response_msg_t *fetch_config(char *conf_server, uint32_t flags,
+					   uint16_t sackd_port, char *ca_cert);
 
-extern config_response_msg_t *fetch_config_from_controller(uint32_t flags);
+extern config_response_msg_t *fetch_config_from_controller(uint32_t flags,
+							   uint16_t port);
 
 extern int dump_to_memfd(char *type, char *config, char **filename);
 
@@ -61,9 +62,9 @@ extern int find_map_conf_file(void *x, void *key);
 extern int write_configs_to_conf_cache(config_response_msg_t *msg,
 				       char *dir);
 
-extern void load_config_response_list(config_response_msg_t *msg,
-				      char *files[]);
+extern config_response_msg_t *new_config_response(bool to_slurmd);
 
 extern void destroy_config_file(void *object);
 
+extern void grab_include_directives(void);
 #endif
