@@ -47,29 +47,37 @@
 #include "slurm/slurm.h"
 
 #include "src/common/forward.h"
+#include "src/common/hostlist.h"
 #include "src/common/macros.h"
-#include "src/interfaces/auth.h"
-#include "src/interfaces/route.h"
 #include "src/common/read_config.h"
-#include "src/common/slurm_protocol_interface.h"
+#include "src/common/slurm_protocol_defs.h"
 #include "src/common/slurm_protocol_pack.h"
+#include "src/common/slurm_protocol_socket.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
 
+#include "src/interfaces/auth.h"
+#include "src/interfaces/conn.h"
+#include "src/interfaces/topology.h"
+
+static slurm_node_alias_addrs_t *last_alias_addrs = NULL;
+static pthread_mutex_t alias_addrs_mutex = PTHREAD_MUTEX_INITIALIZER;
+
 typedef struct {
 	pthread_cond_t *notify;
 	int            *p_thr_count;
 	slurm_msg_t *orig_msg;
-	List ret_list;
+	list_t *ret_list;
 	int timeout;
-	hostlist_t tree_hl;
+	int tree_depth;
+	hostlist_t *tree_hl;
 	pthread_mutex_t *tree_mutex;
 } fwd_tree_t;
 
-static void _start_msg_tree_internal(hostlist_t hl, hostlist_t* sp_hl,
+static void _start_msg_tree_internal(hostlist_t *hl, hostlist_t **sp_hl,
 				     fwd_tree_t *fwd_tree_in,
 				     int hl_count);
-static void _forward_msg_internal(hostlist_t hl, hostlist_t* sp_hl,
+static void _forward_msg_internal(hostlist_t *hl, hostlist_t **sp_hl,
 				  forward_struct_t *fwd_struct,
 				  header_t *header, int timeout,
 				  int hl_count);
@@ -77,8 +85,7 @@ static void _forward_msg_internal(hostlist_t hl, hostlist_t* sp_hl,
 void _destroy_tree_fwd(fwd_tree_t *fwd_tree)
 {
 	if (fwd_tree) {
-		if (fwd_tree->tree_hl)
-			hostlist_destroy(fwd_tree->tree_hl);
+		FREE_NULL_HOSTLIST(fwd_tree->tree_hl);
 
 		/*
 		 * Lock and decrease thread counter, start_msg_tree is waiting
@@ -93,27 +100,40 @@ void _destroy_tree_fwd(fwd_tree_t *fwd_tree)
 	}
 }
 
-void *_forward_thread(void *arg)
+static int _forward_get_addr(forward_struct_t *fwd_struct, char *name,
+			     slurm_addr_t *address)
 {
-	forward_msg_t *fwd_msg = (forward_msg_t *)arg;
+	hostlist_t *hl = hostlist_create(fwd_struct->alias_addrs->node_list);
+	int n = hostlist_find(hl, name);
+	hostlist_destroy(hl);
+	if (n < 0)
+		return SLURM_ERROR;
+	*address = fwd_struct->alias_addrs->node_addrs[n];
+
+	return SLURM_SUCCESS;
+}
+
+static void *_forward_thread(void *arg)
+{
+	forward_msg_t *fwd_msg = arg;
 	forward_struct_t *fwd_struct = fwd_msg->fwd_struct;
+	forward_t *fwd_ptr = &fwd_msg->header.forward;
 	buf_t *buffer = init_buf(BUF_SIZE);	/* probably enough for header */
-	List ret_list = NULL;
-	int fd = -1;
+	list_t *ret_list = NULL;
+	void *tls_conn = NULL;
 	ret_data_info_t *ret_data_info = NULL;
 	char *name = NULL;
-	hostlist_t hl = hostlist_create(fwd_msg->header.forward.nodelist);
+	hostlist_t *hl = hostlist_create(fwd_ptr->nodelist);
 	slurm_addr_t addr;
 	char *buf = NULL;
-	int steps = 0;
-	int start_timeout = fwd_msg->timeout;
 
 	/* repeat until we are sure the message was sent */
 	while ((name = hostlist_shift(hl))) {
-		if (slurm_conf_get_addr(name, &addr, fwd_msg->header.flags)
-		    == SLURM_ERROR) {
-			error("forward_thread: can't find address for host "
-			      "%s, check slurm.conf", name);
+		if ((!(fwd_msg->header.flags & SLURM_PACK_ADDRS) ||
+		     _forward_get_addr(fwd_struct, name, &addr)) &&
+		    slurm_conf_get_addr(name, &addr, fwd_msg->header.flags)) {
+			error("%s: can't find address for host %s, check slurm.conf",
+			      __func__, name);
 			slurm_mutex_lock(&fwd_struct->forward_mutex);
 			mark_as_failed_forward(&fwd_struct->ret_list, name,
 					       SLURM_UNKNOWN_FORWARD_ADDR);
@@ -124,8 +144,10 @@ void *_forward_thread(void *arg)
 			}
 			goto cleanup;
 		}
-		if ((fd = slurm_open_msg_conn(&addr)) < 0) {
-			error("forward_thread to %s (%pA): %m", name, &addr);
+
+		if (!(tls_conn = slurm_open_msg_conn(&addr, NULL))) {
+			error("%s: failed to %s (%pA): %m",
+			      __func__, name, &addr);
 
 			slurm_mutex_lock(&fwd_struct->forward_mutex);
 			mark_as_failed_forward(
@@ -146,19 +168,23 @@ void *_forward_thread(void *arg)
 			}
 			goto cleanup;
 		}
+
 		buf = hostlist_ranged_string_xmalloc(hl);
 
-		xfree(fwd_msg->header.forward.nodelist);
-		fwd_msg->header.forward.nodelist = buf;
-		fwd_msg->header.forward.cnt = hostlist_count(hl);
+		xfree(fwd_ptr->nodelist);
+		fwd_ptr->nodelist = buf;
+		fwd_ptr->cnt = hostlist_count(hl);
+
+		if (fwd_msg->header.flags & SLURM_PACK_ADDRS)
+			fwd_ptr->alias_addrs = *(fwd_struct->alias_addrs);
+
 #if 0
 		info("sending %d forwards (%s) to %s",
-		     fwd_msg->header.forward.cnt,
-		     fwd_msg->header.forward.nodelist, name);
+		     fwd_ptr->cnt, fwd_ptr->nodelist, name);
 #endif
-		if (fwd_msg->header.forward.nodelist[0]) {
+		if (fwd_ptr->nodelist[0]) {
 			debug3("forward: send to %s along with %s",
-			       name, fwd_msg->header.forward.nodelist);
+			       name, fwd_ptr->nodelist);
 		} else
 			debug3("forward: send to %s ", name);
 
@@ -180,10 +206,9 @@ void *_forward_thread(void *arg)
 		/*
 		 * forward message
 		 */
-		if (slurm_msg_sendto(fd,
-				     get_buf_data(buffer),
+		if (slurm_msg_sendto(tls_conn, get_buf_data(buffer),
 				     get_buf_offset(buffer)) < 0) {
-			error("forward_thread: slurm_msg_sendto: %m");
+			error("%s: slurm_msg_sendto: %m", __func__);
 
 			slurm_mutex_lock(&fwd_struct->forward_mutex);
 			mark_as_failed_forward(&fwd_struct->ret_list, name,
@@ -193,8 +218,8 @@ void *_forward_thread(void *arg)
 				FREE_NULL_BUFFER(buffer);
 				buffer = init_buf(fwd_struct->buf_len);
 				slurm_mutex_unlock(&fwd_struct->forward_mutex);
-				close(fd);
-				fd = -1;
+				conn_g_destroy(tls_conn, true);
+				tls_conn = NULL;
 				/* Abandon tree. This way if all the
 				 * nodes in the branch are down we
 				 * don't have to time out for each
@@ -230,26 +255,13 @@ void *_forward_thread(void *arg)
 			goto cleanup;
 		}
 
-		if (fwd_msg->header.forward.cnt > 0) {
-			if (!fwd_msg->header.forward.tree_width)
-				fwd_msg->header.forward.tree_width =
-					slurm_conf.tree_width;
-			steps = (fwd_msg->header.forward.cnt+1) /
-					fwd_msg->header.forward.tree_width;
-			fwd_msg->timeout =
-				slurm_conf.msg_timeout * 1000 * steps;
-			steps++;
-			fwd_msg->timeout += (start_timeout*steps);
-			/* info("now  + %d*%d = %d", start_timeout, */
-			/*      steps, fwd_msg->timeout); */
-		}
-
-		ret_list = slurm_receive_resp_msgs(fd, steps, fwd_msg->timeout);
+		ret_list =
+			slurm_receive_resp_msgs(tls_conn, fwd_ptr->tree_depth,
+						fwd_ptr->timeout);
 		/* info("sent %d forwards got %d back", */
-		/*      fwd_msg->header.forward.cnt, list_count(ret_list)); */
+		/*      fwd_ptr->cnt, list_count(ret_list)); */
 
-		if (!ret_list || (fwd_msg->header.forward.cnt != 0
-				  && list_count(ret_list) <= 1)) {
+		if (!ret_list || (fwd_ptr->cnt && list_count(ret_list) <= 1)) {
 			slurm_mutex_lock(&fwd_struct->forward_mutex);
 			mark_as_failed_forward(&fwd_struct->ret_list, name,
 					       errno);
@@ -259,27 +271,24 @@ void *_forward_thread(void *arg)
 				FREE_NULL_BUFFER(buffer);
 				buffer = init_buf(fwd_struct->buf_len);
 				slurm_mutex_unlock(&fwd_struct->forward_mutex);
-				close(fd);
-				fd = -1;
+				conn_g_destroy(tls_conn, true);
+				tls_conn = NULL;
 				continue;
 			}
 			goto cleanup;
-		} else if ((fwd_msg->header.forward.cnt+1)
-			  != list_count(ret_list)) {
+		} else if ((fwd_ptr->cnt + 1) != list_count(ret_list)) {
 			/* this should never be called since the above
 			   should catch the failed forwards and pipe
 			   them back down, but this is here so we
 			   never have to worry about a locked
 			   mutex */
-			ListIterator itr = NULL;
+			list_itr_t *itr = NULL;
 			char *tmp = NULL;
 			int first_node_found = 0;
-			hostlist_iterator_t host_itr
+			hostlist_iterator_t *host_itr
 				= hostlist_iterator_create(hl);
-			error("We shouldn't be here.  We forwarded to %d "
-			      "but only got %d back",
-			      (fwd_msg->header.forward.cnt+1),
-			      list_count(ret_list));
+			error("We shouldn't be here.  We forwarded to %d but only got %d back",
+			      (fwd_ptr->cnt + 1), list_count(ret_list));
 			while ((tmp = hostlist_next(host_itr))) {
 				int node_found = 0;
 				itr = list_iterator_create(ret_list);
@@ -332,10 +341,12 @@ void *_forward_thread(void *arg)
 	}
 	free(name);
 cleanup:
-	if ((fd >= 0) && close(fd) < 0)
-		error ("close(%d): %m", fd);
+	conn_g_destroy(tls_conn, true);
 	hostlist_destroy(hl);
-	destroy_forward(&fwd_msg->header.forward);
+	fwd_ptr->alias_addrs.net_cred = NULL;
+	fwd_ptr->alias_addrs.node_addrs = NULL;
+	fwd_ptr->alias_addrs.node_list = NULL;
+	destroy_forward(fwd_ptr);
 	FREE_NULL_BUFFER(buffer);
 	slurm_cond_signal(&fwd_struct->notify);
 	slurm_mutex_unlock(&fwd_struct->forward_mutex);
@@ -344,10 +355,40 @@ cleanup:
 	return (NULL);
 }
 
-void *_fwd_tree_thread(void *arg)
+static int _fwd_tree_get_addr(fwd_tree_t *fwd_tree, char *name,
+			      slurm_addr_t *address)
 {
-	fwd_tree_t *fwd_tree = (fwd_tree_t *)arg;
-	List ret_list = NULL;
+	if ((fwd_tree->orig_msg->flags & SLURM_PACK_ADDRS) &&
+	    fwd_tree->orig_msg->forward.alias_addrs.node_addrs) {
+		hostlist_t *hl =
+			hostlist_create(fwd_tree->orig_msg->forward.alias_addrs.node_list);
+		int n = hostlist_find(hl, name);
+		hostlist_destroy(hl);
+		if (n < 0)
+			return SLURM_ERROR;
+		*address =
+			fwd_tree->orig_msg->forward.alias_addrs.node_addrs[n];
+	} else if (slurm_conf_get_addr(name, address,
+				       fwd_tree->orig_msg->flags) ==
+		   SLURM_ERROR) {
+		error("%s: can't find address for host %s, check slurm.conf",
+		      __func__, name);
+		slurm_mutex_lock(fwd_tree->tree_mutex);
+		mark_as_failed_forward(&fwd_tree->ret_list, name,
+				       SLURM_UNKNOWN_FORWARD_ADDR);
+		slurm_cond_signal(fwd_tree->notify);
+		slurm_mutex_unlock(fwd_tree->tree_mutex);
+
+		return SLURM_ERROR;
+	}
+
+	return SLURM_SUCCESS;
+}
+
+static void *_fwd_tree_thread(void *arg)
+{
+	fwd_tree_t *fwd_tree = arg;
+	list_t *ret_list = NULL;
 	char *name = NULL;
 	char *buf = NULL;
 	slurm_msg_t send_msg;
@@ -363,25 +404,29 @@ void *_fwd_tree_thread(void *arg)
 
 	/* repeat until we are sure the message was sent */
 	while ((name = hostlist_shift(fwd_tree->tree_hl))) {
-		if (slurm_conf_get_addr(name, &send_msg.address, send_msg.flags)
-		    == SLURM_ERROR) {
-			error("fwd_tree_thread: can't find address for host "
-			      "%s, check slurm.conf", name);
-			slurm_mutex_lock(fwd_tree->tree_mutex);
-			mark_as_failed_forward(&fwd_tree->ret_list, name,
-					       SLURM_UNKNOWN_FORWARD_ADDR);
- 			slurm_cond_signal(fwd_tree->notify);
-			slurm_mutex_unlock(fwd_tree->tree_mutex);
+		if (_fwd_tree_get_addr(fwd_tree, name, &send_msg.address)) {
 			free(name);
 
 			continue;
 		}
 
+		/*
+		 * Tell additional message forwarding to use the same
+		 * tree_width; without this, additional message forwarding
+		 * defaults to slurm_conf.tree_width (see _forward_thread).
+		 */
+		send_msg.forward.tree_width =
+			fwd_tree->orig_msg->forward.tree_width;
+		send_msg.forward.tree_depth = fwd_tree->tree_depth;
 		send_msg.forward.timeout = fwd_tree->timeout;
 		if ((send_msg.forward.cnt = hostlist_count(fwd_tree->tree_hl))){
 			buf = hostlist_ranged_string_xmalloc(
 					fwd_tree->tree_hl);
 			send_msg.forward.nodelist = buf;
+			if (send_msg.flags & SLURM_PACK_ADDRS) {
+				send_msg.forward.alias_addrs =
+					fwd_tree->orig_msg->forward.alias_addrs;
+			}
 		} else
 			send_msg.forward.nodelist = NULL;
 
@@ -404,13 +449,12 @@ void *_fwd_tree_thread(void *arg)
 			*/
 			if ((ret_cnt <= send_msg.forward.cnt) &&
 			    (errno != SLURM_COMMUNICATIONS_CONNECTION_ERROR)) {
-				error("fwd_tree_thread: %s failed to forward "
-				      "the message, expecting %d ret got only "
-				      "%d",
-				      name, send_msg.forward.cnt + 1, ret_cnt);
+				error("%s: %s failed to forward the message, expecting %d ret got only %d",
+				      __func__, name, send_msg.forward.cnt + 1,
+				      ret_cnt);
 				if (ret_cnt > 1) { /* not likely */
 					ret_data_info_t *ret_data_info = NULL;
-					ListIterator itr =
+					list_itr_t *itr =
 						list_iterator_create(ret_list);
 					while ((ret_data_info =
 						list_next(itr))) {
@@ -449,9 +493,8 @@ void *_fwd_tree_thread(void *arg)
 			/* This should never happen (when this was
 			 * written slurm_send_addr_recv_msgs always
 			 * returned a list */
-			error("fwd_tree_thread: no return list given from "
-			      "slurm_send_addr_recv_msgs spawned for %s",
-			      name);
+			error("%s: no return list given from slurm_send_addr_recv_msgs spawned for %s",
+			      __func__, name);
 			slurm_mutex_lock(fwd_tree->tree_mutex);
 			mark_as_failed_forward(
 				&fwd_tree->ret_list, name,
@@ -477,7 +520,7 @@ void *_fwd_tree_thread(void *arg)
 	return NULL;
 }
 
-static void _start_msg_tree_internal(hostlist_t hl, hostlist_t* sp_hl,
+static void _start_msg_tree_internal(hostlist_t *hl, hostlist_t **sp_hl,
 				     fwd_tree_t *fwd_tree_in,
 				     int hl_count)
 {
@@ -522,11 +565,11 @@ static void _start_msg_tree_internal(hostlist_t hl, hostlist_t* sp_hl,
 		(*fwd_tree->p_thr_count)++;
 		slurm_mutex_unlock(fwd_tree->tree_mutex);
 
-		slurm_thread_create_detached(NULL, _fwd_tree_thread, fwd_tree);
+		slurm_thread_create_detached(_fwd_tree_thread, fwd_tree);
 	}
 }
 
-static void _forward_msg_internal(hostlist_t hl, hostlist_t* sp_hl,
+static void _forward_msg_internal(hostlist_t *hl, hostlist_t **sp_hl,
 				  forward_struct_t *fwd_struct,
 				  header_t *header, int timeout,
 				  int hl_count)
@@ -568,7 +611,10 @@ static void _forward_msg_internal(hostlist_t hl, hostlist_t* sp_hl,
 
 		forward_init(&fwd_msg->header.forward);
 		fwd_msg->header.forward.nodelist = buf;
-		slurm_thread_create_detached(NULL, _forward_thread, fwd_msg);
+		fwd_msg->header.forward.tree_width = header->forward.tree_width;
+		fwd_msg->header.forward.tree_depth = header->forward.tree_depth;
+		fwd_msg->header.forward.timeout = header->forward.timeout;
+		slurm_thread_create_detached(_forward_thread, fwd_msg);
 	}
 }
 
@@ -579,8 +625,7 @@ static void _forward_msg_internal(hostlist_t hl, hostlist_t* sp_hl,
  */
 extern void forward_init(forward_t *forward)
 {
-	memset(forward, 0, sizeof(forward_t));
-	forward->init = FORWARD_INIT;
+	*forward = (forward_t) FORWARD_INITIALIZER;
 }
 
 /*
@@ -597,24 +642,58 @@ extern void forward_init(forward_t *forward)
  */
 extern int forward_msg(forward_struct_t *forward_struct, header_t *header)
 {
-	hostlist_t hl = NULL;
-	hostlist_t* sp_hl;
-	int hl_count = 0;
+	hostlist_t *hl = NULL;
+	hostlist_t **sp_hl;
+	int hl_count = 0, depth;
 
 	if (!forward_struct->ret_list) {
 		error("didn't get a ret_list from forward_struct");
 		return SLURM_ERROR;
 	}
 	hl = hostlist_create(header->forward.nodelist);
+	if (header->flags & SLURM_PACK_ADDRS) {
+		forward_struct->alias_addrs = extract_net_cred(
+			header->forward.alias_addrs.net_cred, header->version);
+		if (!forward_struct->alias_addrs) {
+			error("unable to extract net_cred");
+			hostlist_destroy(hl);
+			return SLURM_ERROR;
+		}
+		forward_struct->alias_addrs->net_cred =
+			header->forward.alias_addrs.net_cred;
+		header->forward.alias_addrs.net_cred = NULL;
+	}
+
 	hostlist_uniq(hl);
 
-	if (route_g_split_hostlist(
-		    hl, &sp_hl, &hl_count, header->forward.tree_width)) {
+	if ((depth = topology_g_split_hostlist(hl, &sp_hl, &hl_count,
+					       header->forward.tree_width)) ==
+	    SLURM_ERROR) {
 		error("unable to split forward hostlist");
 		hostlist_destroy(hl);
 		return SLURM_ERROR;
 	}
 
+	/* Calculate the new timeout based on the original timeout */
+	if (header->forward.tree_depth)
+		header->forward.timeout = (header->forward.timeout * depth) /
+					  header->forward.tree_depth;
+	else {
+		/*
+		 * tree_depth not packed - likely using 24.05- protocol version.
+		 * Calculate the timeout based on MessageTimeout instead.
+		 */
+		header->forward.timeout =
+			(2 * depth * slurm_conf.msg_timeout * MSEC_IN_SEC);
+		debug3("%s: original tree_depth was 0 so setting new timeout to %d",
+			__func__, header->forward.timeout);
+	}
+	header->forward.tree_depth = depth;
+	forward_struct->timeout = header->forward.timeout;
+
+	log_flag(NET, "%s: forwarding messages to %u nodes with timeout of %d",
+		 __func__, forward_struct->fwd_cnt, forward_struct->timeout);
+
 	_forward_msg_internal(NULL, sp_hl, forward_struct, header,
 			      forward_struct->timeout, hl_count);
 
@@ -623,6 +702,104 @@ extern int forward_msg(forward_struct_t *forward_struct, header_t *header)
 	return SLURM_SUCCESS;
 }
 
+static void _get_alias_addrs(hostlist_t *hl, slurm_msg_t *msg, int *cnt)
+{
+	hostlist_iterator_t *hi;
+	char *node_name;
+	int addr_index = 0;
+	forward_t *forward = &(msg->forward);
+
+	if (!(msg->flags & SLURM_PACK_ADDRS))
+		return;
+	slurm_free_node_alias_addrs_members(&forward->alias_addrs);
+
+	forward->alias_addrs.node_addrs = xcalloc(*cnt, sizeof(slurm_addr_t));
+
+	hi = hostlist_iterator_create(hl);
+	while ((node_name = hostlist_next(hi))) {
+		slurm_addr_t *addr =
+			&forward->alias_addrs.node_addrs[addr_index];
+		if (!slurm_conf_get_addr(node_name, addr, msg->flags)) {
+			++addr_index;
+		} else {
+			hostlist_remove(hi);
+			forward->cnt--;
+			(*cnt)--;
+		}
+		free(node_name);
+	}
+	hostlist_iterator_destroy(hi);
+
+	forward->alias_addrs.node_list = hostlist_ranged_string_xmalloc(hl);
+	forward->alias_addrs.node_cnt = *cnt;
+
+	forward->alias_addrs.net_cred =
+		create_net_cred(&forward->alias_addrs, msg->protocol_version);
+}
+
+/*
+ * Get dynamic addrs if forwarding to a unknown/unresolvable nodes.
+ */
+static void _get_dynamic_addrs(hostlist_t *hl, slurm_msg_t *msg)
+{
+	char *name;
+	hostlist_iterator_t *itr;
+	bool try_last = false;
+	hostlist_t *hl_last = NULL;
+
+	xassert(hl);
+	xassert(msg);
+
+	if (running_in_daemon())
+		return;
+
+	if (msg->flags & SLURM_PACK_ADDRS)
+		return;
+
+	itr = hostlist_iterator_create(hl);
+	slurm_mutex_lock(&alias_addrs_mutex);
+	if (last_alias_addrs &&
+	    (last_alias_addrs->expiration - time(NULL)) > 10) {
+		try_last = true;
+		hl_last = hostlist_create(last_alias_addrs->node_list);
+	}
+
+	while ((name = hostlist_next(itr))) {
+		slurm_node_alias_addrs_t *alias_addrs = NULL;
+		char *nodelist;
+		bool dynamic;
+
+		if (!slurm_conf_check_addr(name, &dynamic) && !dynamic) {
+			free(name);
+			continue;
+		}
+
+		if (try_last && (hostlist_find(hl_last, name) >= 0)) {
+			msg->flags |= SLURM_PACK_ADDRS;
+			free(name);
+			continue;
+		}
+		try_last = false;
+		nodelist = hostlist_ranged_string_xmalloc(hl);
+		if (!slurm_get_node_alias_addrs(nodelist, &alias_addrs)) {
+			msg->flags |= SLURM_PACK_ADDRS;
+		}
+		slurm_free_node_alias_addrs(last_alias_addrs);
+		last_alias_addrs = alias_addrs;
+		free(name);
+		xfree(nodelist);
+		break;
+	}
+	hostlist_iterator_destroy(itr);
+	hostlist_destroy(hl_last);
+
+	if (last_alias_addrs && (msg->flags & SLURM_PACK_ADDRS)) {
+		slurm_copy_node_alias_addrs_members(&(msg->forward.alias_addrs),
+						    last_alias_addrs);
+	}
+	slurm_mutex_unlock(&alias_addrs_mutex);
+}
+
 /*
  * start_msg_tree  - logic to begin the forward tree and
  *                   accumulate the return codes from processes getting the
@@ -631,30 +808,44 @@ extern int forward_msg(forward_struct_t *forward_struct, header_t *header)
  * IN: hl          - hostlist_t   - list of every node to send message to
  * IN: msg         - slurm_msg_t  - message to send.
  * IN: timeout     - int          - how long to wait in milliseconds.
- * RET List 	   - List containing the responses of the children
- *		     (if any) we forwarded the message to. List
+ * RET list_t *    - list containing the responses of the children
+ *		     (if any) we forwarded the message to. list
  *		     containing type (ret_data_info_t).
  */
-extern List start_msg_tree(hostlist_t hl, slurm_msg_t *msg, int timeout)
+extern list_t *start_msg_tree(hostlist_t *hl, slurm_msg_t *msg, int timeout)
 {
 	fwd_tree_t fwd_tree;
 	pthread_mutex_t tree_mutex;
 	pthread_cond_t notify;
 	int count = 0;
-	List ret_list = NULL;
+	list_t *ret_list = NULL;
 	int thr_count = 0;
 	int host_count = 0;
-	hostlist_t* sp_hl;
-	int hl_count = 0;
+	hostlist_t **sp_hl;
+	int hl_count = 0, depth;
 
 	xassert(hl);
 	xassert(msg);
 
+	if (timeout <= 0) {
+		/* convert secs to msec */
+		timeout = slurm_conf.msg_timeout * MSEC_IN_SEC;
+	}
+
 	hostlist_uniq(hl);
 	host_count = hostlist_count(hl);
 
-	if (route_g_split_hostlist(hl, &sp_hl, &hl_count,
-				   msg->forward.tree_width)) {
+	_get_alias_addrs(hl, msg, &host_count);
+	_get_dynamic_addrs(hl, msg);
+
+	if (running_in_slurmctld())
+		depth = topology_g_split_hostlist(hl, &sp_hl, &hl_count,
+						  msg->forward.tree_width);
+	else
+		depth = hostlist_split_treewidth(hl, &sp_hl, &hl_count,
+						 msg->forward.tree_width);
+
+	if (depth == SLURM_ERROR) {
 		error("unable to split forward hostlist");
 		return NULL;
 	}
@@ -666,7 +857,8 @@ extern List start_msg_tree(hostlist_t hl, slurm_msg_t *msg, int timeout)
 	memset(&fwd_tree, 0, sizeof(fwd_tree));
 	fwd_tree.orig_msg = msg;
 	fwd_tree.ret_list = ret_list;
-	fwd_tree.timeout = timeout;
+	fwd_tree.tree_depth = depth;
+	fwd_tree.timeout = 2 * depth * timeout;
 	fwd_tree.notify = &notify;
 	fwd_tree.p_thr_count = &thr_count;
 	fwd_tree.tree_mutex = &tree_mutex;
@@ -697,12 +889,12 @@ extern List start_msg_tree(hostlist_t hl, slurm_msg_t *msg, int timeout)
 /*
  * mark_as_failed_forward- mark a node as failed and add it to "ret_list"
  *
- * IN: ret_list       - List *   - ret_list to put ret_data_info
+ * IN: ret_list       - list_t ** - ret_list to put ret_data_info
  * IN: node_name      - char *   - node name that failed
  * IN: err            - int      - error message from attempt
  *
  */
-extern void mark_as_failed_forward(List *ret_list, char *node_name, int err)
+extern void mark_as_failed_forward(list_t **ret_list, char *node_name, int err)
 {
 	ret_data_info_t *ret_data_info = NULL;
 
@@ -749,9 +941,23 @@ extern void forward_wait(slurm_msg_t * msg)
 	return;
 }
 
-void destroy_data_info(void *object)
+extern void fwd_set_alias_addrs(slurm_node_alias_addrs_t *alias_addrs)
 {
-	ret_data_info_t *ret_data_info = (ret_data_info_t *)object;
+	if (!alias_addrs)
+		return;
+
+	slurm_mutex_lock(&alias_addrs_mutex);
+
+	if (!last_alias_addrs)
+		last_alias_addrs = xmalloc(sizeof(*last_alias_addrs));
+	slurm_copy_node_alias_addrs_members(last_alias_addrs, alias_addrs);
+
+	slurm_mutex_unlock(&alias_addrs_mutex);
+}
+
+extern void destroy_data_info(void *object)
+{
+	ret_data_info_t *ret_data_info = object;
 	if (ret_data_info) {
 		slurm_free_msg_data(ret_data_info->type,
 				    ret_data_info->data);
@@ -760,22 +966,24 @@ void destroy_data_info(void *object)
 	}
 }
 
-void destroy_forward(forward_t *forward)
+extern void destroy_forward(forward_t *forward)
 {
 	if (forward->init == FORWARD_INIT) {
+		slurm_free_node_alias_addrs_members(&forward->alias_addrs);
 		xfree(forward->nodelist);
 		forward->init = 0;
 	} else {
-		error("destroy_forward: no init");
+		error("%s: no init", __func__);
 	}
 }
 
-void destroy_forward_struct(forward_struct_t *forward_struct)
+extern void destroy_forward_struct(forward_struct_t *forward_struct)
 {
 	if (forward_struct) {
 		xfree(forward_struct->buf);
 		slurm_mutex_destroy(&forward_struct->forward_mutex);
 		slurm_cond_destroy(&forward_struct->notify);
+		slurm_free_node_alias_addrs(forward_struct->alias_addrs);
 		xfree(forward_struct);
 	}
 }
