@@ -65,6 +65,7 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/types.h>
+#include <sys/un.h>
 #include <unistd.h>
 
 #include "slurm/slurm_errno.h"
@@ -138,13 +139,11 @@ typedef struct {
 	uint16_t fmt;            /* Flag for specifying timestamp format */
 }	log_t;
 
-char *slurm_prog_name = NULL;
-
 /* static variables */
 static pthread_mutex_t  log_lock = PTHREAD_MUTEX_INITIALIZER;
 static log_t            *log = NULL;
 static log_t            *sched_log = NULL;
-
+static bool syslog_open = false;
 static volatile log_level_t highest_log_level = LOG_LEVEL_END;
 static volatile log_level_t highest_sched_log_level = LOG_LEVEL_QUIET;
 
@@ -309,6 +308,11 @@ _log_init(char *prog, log_options_t opt, log_facility_t fac, char *logfile )
 		atfork_install_handlers();
 	}
 
+	if (syslog_open) {
+		closelog();
+		syslog_open = false;
+	}
+
 	if (prog) {
 		if (log->argv0)
 			xfree(log->argv0);
@@ -322,10 +326,6 @@ _log_init(char *prog, log_options_t opt, log_facility_t fac, char *logfile )
 		log->argv0 = xstrdup(short_name);
 	}
 
-	/* Only take the first one here.  In some situations it can change. */
-	if (!slurm_prog_name && log->argv0 && (strlen(log->argv0) > 0))
-		slurm_prog_name = xstrdup(log->argv0);
-
 	if (!log->prefix)
 		log->prefix = xstrdup("");
 
@@ -345,8 +345,11 @@ _log_init(char *prog, log_options_t opt, log_facility_t fac, char *logfile )
 		log->fbuf = cbuf_create(128, 8192);
 	}
 
-	if (log->opt.syslog_level > LOG_LEVEL_QUIET)
+	if (log->opt.syslog_level > LOG_LEVEL_QUIET) {
 		log->facility = fac;
+		openlog(log->argv0, LOG_PID, log->facility);
+		syslog_open = true;
+	}
 
 	if (logfile && (log->opt.logfile_level > LOG_LEVEL_QUIET)) {
 		int mode = O_CREAT | O_WRONLY | O_APPEND | O_CLOEXEC;
@@ -519,6 +522,10 @@ void log_fini(void)
 
 	slurm_mutex_lock(&log_lock);
 	_log_flush(log);
+	if (syslog_open) {
+		closelog();
+		syslog_open = false;
+	}
 	xfree(log->argv0);
 	xfree(log->prefix);
 	if (log->buf)
@@ -528,7 +535,6 @@ void log_fini(void)
 	if (log->logfp)
 		fclose(log->logfp);
 	xfree(log);
-	xfree(slurm_prog_name);
 	slurm_mutex_unlock(&log_lock);
 }
 
@@ -572,6 +578,10 @@ void log_set_prefix(char **prefix)
 void log_set_argv0(char *argv0)
 {
 	slurm_mutex_lock(&log_lock);
+	if (syslog_open) {
+		closelog();
+		syslog_open = false;
+	}
 	if (log->argv0)
 		xfree(log->argv0);
 	if (!argv0)
@@ -706,10 +716,6 @@ static void _set_idbuf(char *idbuf, size_t size)
  */
 static char *_addr2fmt(slurm_addr_t *addr_ptr, char *buf, int buf_size)
 {
-	struct sockaddr_in *in = (struct sockaddr_in *) addr_ptr;
-	char addrbuf[INET6_ADDRSTRLEN];
-	uint16_t port = 0;
-
 	/*
 	 * NOTE: You will notice we put a %.0s in front of the string.
 	 * This is to handle the fact that we can't remove the addr_ptr
@@ -720,17 +726,32 @@ static char *_addr2fmt(slurm_addr_t *addr_ptr, char *buf, int buf_size)
 		return "%.0sNULL";
 
 	if (addr_ptr->ss_family == AF_INET6) {
+		char addrbuf[INET6_ADDRSTRLEN];
+		uint16_t port = 0;
 		struct sockaddr_in6 *in6 = (struct sockaddr_in6 *) addr_ptr;
 		inet_ntop(AF_INET6, &in6->sin6_addr, addrbuf, INET6_ADDRSTRLEN);
 		port = ntohs(in6->sin6_port);
 		snprintf(buf, buf_size, "[%%.0s%s]:%d", addrbuf, port);
-		return buf;
+	} else if (addr_ptr->ss_family == AF_INET) {
+		char addrbuf[INET_ADDRSTRLEN];
+		struct sockaddr_in *in = (struct sockaddr_in *) addr_ptr;
+		uint16_t port = 0;
+		inet_ntop(AF_INET, &in->sin_addr, addrbuf, INET_ADDRSTRLEN);
+		port = ntohs(in->sin_port);
+		snprintf(buf, buf_size, "%%.0s%s:%d", addrbuf, port);
+	} else if (addr_ptr->ss_family == AF_UNIX) {
+		struct sockaddr_un *un = (struct sockaddr_un *) addr_ptr;
+		snprintf(buf, buf_size, "%%.0sunix:%s", un->sun_path);
+	} else if (addr_ptr->ss_family == AF_UNSPEC) {
+		/*
+		 * AF_UNSPEC is place holder for unspecified which may be
+		 * nothing or used as a wild card depending on the API call.
+		 */
+		snprintf(buf, buf_size, "%%.0sAF_UNSPEC");
+	} else {
+		snprintf(buf, buf_size, "%%.0sINVALID");
 	}
 
-	inet_ntop(AF_INET, &in->sin_addr, addrbuf, INET_ADDRSTRLEN);
-	port = ntohs(in->sin_port);
-	snprintf(buf, buf_size, "%%.0s%s:%d", addrbuf, port);
-
 	return buf;
 }
 /*
@@ -824,7 +845,7 @@ static char *_print_data_json(const data_t *d, char *buffer, int size)
 
 extern char *vxstrfmt(const char *fmt, va_list ap)
 {
-	char	*intermediate_fmt = NULL;
+	char *intermediate_fmt = NULL, *intermediate_pos = NULL;
 	char	*out_string = NULL;
 	char	*p;
 	bool found_other_formats = false;
@@ -839,7 +860,7 @@ extern char *vxstrfmt(const char *fmt, va_list ap)
 			 * no more format sequences, append the rest of
 			 * fmt and exit the loop:
 			 */
-			xstrcat(intermediate_fmt, fmt);
+			xstrcatat(intermediate_fmt, &intermediate_pos, fmt);
 			break;
 		}
 
@@ -894,7 +915,8 @@ extern char *vxstrfmt(const char *fmt, va_list ap)
 			 * append anything from fmt up to p to the intermediate
 			 * format string:
 			 */
-			xstrncat(intermediate_fmt, fmt, p - fmt);
+			xstrncatat(intermediate_fmt, &intermediate_pos,
+				   fmt, p - fmt);
 			fmt = p + 1;
 
 			/*
@@ -915,7 +937,9 @@ extern char *vxstrfmt(const char *fmt, va_list ap)
 					for (int i = 0; i < cnt; i++ )
 						ptr = va_arg(ap_copy, void *);
 					addr_ptr = ptr;
-					xstrcat(intermediate_fmt,
+					xstrcatat(
+						intermediate_fmt,
+						&intermediate_pos,
 						_addr2fmt(
 							addr_ptr,
 							substitute_on_stack,
@@ -931,7 +955,9 @@ extern char *vxstrfmt(const char *fmt, va_list ap)
 					va_copy(ap_copy, ap);
 					for (int i = 0; i < cnt; i++ )
 						d = va_arg(ap_copy, void *);
-					xstrcat(intermediate_fmt,
+					xstrcatat(
+						intermediate_fmt,
+						&intermediate_pos,
 						_print_data_json(
 							d,
 							substitute_on_stack,
@@ -947,7 +973,9 @@ extern char *vxstrfmt(const char *fmt, va_list ap)
 					va_copy(ap_copy, ap);
 					for (int i = 0; i < cnt; i++ )
 						d = va_arg(ap_copy, void *);
-					xstrcat(intermediate_fmt,
+					xstrcatat(
+						intermediate_fmt,
+						&intermediate_pos,
 						_print_data_t(
 							d,
 							substitute_on_stack,
@@ -966,7 +994,9 @@ extern char *vxstrfmt(const char *fmt, va_list ap)
 					for (i = 0; i < cnt; i++ )
 						ptr = va_arg(ap_copy, void *);
 					job_ptr = ptr;
-					xstrcat(intermediate_fmt,
+					xstrcatat(
+						intermediate_fmt,
+						&intermediate_pos,
 						_jobid2fmt(
 							job_ptr,
 							substitute_on_stack,
@@ -989,7 +1019,9 @@ extern char *vxstrfmt(const char *fmt, va_list ap)
 					for (i = 0; i < cnt; i++ )
 						ptr = va_arg(ap_copy, void *);
 					step_id = ptr;
-					xstrcat(intermediate_fmt,
+					xstrcatat(
+						intermediate_fmt,
+						&intermediate_pos,
 						log_build_step_id_str(
 							step_id,
 							substitute_on_stack,
@@ -1017,12 +1049,16 @@ extern char *vxstrfmt(const char *fmt, va_list ap)
 					if (step_ptr &&
 					    (step_ptr->magic == STEP_MAGIC))
 						job_ptr = step_ptr->job_ptr;
-					xstrcat(intermediate_fmt,
+					xstrcatat(
+						intermediate_fmt,
+						&intermediate_pos,
 						_jobid2fmt(
 							job_ptr,
 							substitute_on_stack,
 							sizeof(substitute_on_stack)));
-					xstrcat(intermediate_fmt,
+					xstrcatat(
+						intermediate_fmt,
+						&intermediate_pos,
 						_stepid2fmt(
 							step_ptr,
 							substitute_on_stack,
@@ -1052,7 +1088,7 @@ extern char *vxstrfmt(const char *fmt, va_list ap)
 					xiso8601timecat(substitute, true);
 					break;
 				}
-				switch (log->fmt & (~LOG_FMT_FORMAT_STDERR)) {
+				switch (log->fmt) {
 				case LOG_FMT_ISO8601_MS:
 					/* "%M" => "yyyy-mm-ddThh:mm:ss.fff"  */
 					xiso8601timecat(substitute, true);
@@ -1107,13 +1143,16 @@ extern char *vxstrfmt(const char *fmt, va_list ap)
 
 				while (*s && (p = (char *)strchr(s, '%'))) {
 					/* append up through the '%' */
-					xstrncat(intermediate_fmt, s, p - s);
-					xstrcat(intermediate_fmt, "%%");
+					xstrncatat(intermediate_fmt,
+						   &intermediate_pos, s, p - s);
+					xstrcatat(intermediate_fmt,
+						  &intermediate_pos, "%%");
 					s = p + 1;
 				}
 				if (*s) {
 					/* append whatever's left of the substitution: */
-					xstrcat(intermediate_fmt, s);
+					xstrcatat(intermediate_fmt,
+						  &intermediate_pos, s);
 				}
 
 				/* deallocate substitute if necessary: */
@@ -1126,7 +1165,7 @@ extern char *vxstrfmt(const char *fmt, va_list ap)
 			 * no more format sequences for us, append the rest of
 			 * fmt and exit the loop:
 			 */
-			xstrcat(intermediate_fmt, fmt);
+			xstrcatat(intermediate_fmt, &intermediate_pos, fmt);
 			break;
 		}
 	}
@@ -1260,6 +1299,13 @@ static void _log_msg(log_level_t level, bool sched, bool spank, bool warn,
 	char *eol = "\n";
 	int priority = LOG_INFO;
 
+	/*
+	 * Construct the message outside the lock as this can be slow.
+	 * The format_print() macro should ensure that we're always going
+	 * to print the resulting message through one or more channels.
+	 */
+	buf = vxstrfmt(fmt, args);
+
 	slurm_mutex_lock(&log_lock);
 
 	if (!LOG_INITIALIZED) {
@@ -1272,7 +1318,6 @@ static void _log_msg(log_level_t level, bool sched, bool spank, bool warn,
 
 	if (SCHED_LOG_INITIALIZED && sched &&
 	    (highest_sched_log_level > LOG_LEVEL_QUIET)) {
-		buf = vxstrfmt(fmt, args);
 		xlogfmtcat(&msgbuf, "[%M] %s%s", sched_log->prefix, pfx);
 		_log_printf(sched_log, sched_log->fbuf, sched_log->logfp,
 			    "sched: %s%s\n", msgbuf, buf);
@@ -1339,36 +1384,19 @@ static void _log_msg(log_level_t level, bool sched, bool spank, bool warn,
 
 	}
 
-	if (!buf) {
-		/* format the basic message,
-		 * if not already done for scheduling log */
-		buf = vxstrfmt(fmt, args);
-	}
-
 	if (level <= log->opt.stderr_level) {
 
 		fflush(stdout);
 		if (spank) {
 			_log_printf(log, log->buf, stderr, "%s%s", buf, eol);
-		} else if (log->fmt == LOG_FMT_THREAD_ID) {
-			/*
-			 * This is for backward compatibility. In versions
-			 * < 23.11 this was the only way to print to stderr.
-			 * Keep this behavior since LogTimeFormat=format_stderr
-			 * results in a little bit different format.
-			 */
-			char tmp[64];
-			_set_idbuf(tmp, sizeof(tmp));
-			_log_printf(log, log->buf, stderr, "%s: %s%s%s",
-			            tmp, pfx, buf, eol);
-		} else if ((log->fmt & LOG_FMT_FORMAT_STDERR)) {
-			xlogfmtcat(&msgbuf, "[%M] %s", pfx);
-			_log_printf(log, log->buf, stderr, "%s%s%s",
-				    msgbuf, buf, eol);
+		} else if (running_in_daemon()) {
+			xlogfmtcat(&msgbuf, "[%M]");
+			_log_printf(log, log->buf, stderr, "%s %s%s%s", msgbuf,
+				    pfx, buf, eol);
 			xfree(msgbuf);
 		} else {
 			_log_printf(log, log->buf, stderr, "%s: %s%s%s",
-			            log->argv0, pfx, buf, eol);
+				    log->argv0, pfx, buf, eol);
 		}
 		fflush(stderr);
 	}
@@ -1405,7 +1433,6 @@ static void _log_msg(log_level_t level, bool sched, bool spank, bool warn,
 
 		xfree(json);
 		fflush(log->logfp);
-		xfree(msgbuf);
 	} else {
 		xassert(log->opt.logfile_fmt == LOG_FILE_FMT_TIMESTAMP);
 		xlogfmtcat(&msgbuf, "[%M] %s%s", log->prefix, pfx);
@@ -1418,14 +1445,9 @@ static void _log_msg(log_level_t level, bool sched, bool spank, bool warn,
 	if (level <=  log->opt.syslog_level) {
 
 		/* Avoid changing errno if syslog fails */
-		int orig_errno = slurm_get_errno();
-		xlogfmtcat(&msgbuf, "%s%s%s", log->prefix, pfx, buf);
-		openlog(log->argv0, LOG_PID, log->facility);
-		syslog(priority, "%.500s", msgbuf);
-		closelog();
-		slurm_seterrno(orig_errno);
-
-		xfree(msgbuf);
+		int orig_errno = errno;
+		syslog(priority, "%s%s%s", log->prefix, pfx, buf);
+		errno = orig_errno;
 	}
 
 	slurm_mutex_unlock(&log_lock);
@@ -1472,6 +1494,9 @@ void fatal(const char *fmt, ...)
 	LOG_MACRO(LOG_LEVEL_FATAL, false, fmt);
 	log_flush();
 
+	if (getenv("ABORT_ON_FATAL"))
+		abort();
+
 	exit(1);
 }
 
