@@ -5,9 +5,9 @@
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  Copyright (C) 2008-2010 Lawrence Livermore National Security.
  *  Portions Copyright (C) 2008 Vijay Ramasubramanian.
- *  Portions Copyright (C) 2010-2016 SchedMD <https://www.schedmd.com>.
+ *  Copyright (C) SchedMD LLC.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
- *  Written by Morris Mette <jette1@llnl.gov>.
+ *  Written by Morris Jette <jette1@llnl.gov>.
  *  CODE-OCEC-09-009. All rights reserved.
  *
  *  This file is part of Slurm, a resource management program.
@@ -54,10 +54,6 @@ extern slurm_conf_t slurm_conf;
 extern char *default_slurm_config_file;
 extern char *default_plugin_path;
 
-#ifndef NDEBUG
-extern uint16_t drop_priv_flag;
-#endif
-
 /*
  * We can't include node_conf.h to get node_record_t because node_conf.h
  * includes read_config.h and creates a circular dependency. We create the
@@ -79,17 +75,16 @@ typedef struct node_record node_record_t;
 #define DEFAULT_ACCOUNTING_TRES  "cpu,mem,energy,node,billing,fs/disk,vmem,pages"
 #define DEFAULT_ACCOUNTING_DB      "slurm_acct_db"
 #define DEFAULT_ACCOUNTING_ENFORCE  0
-#define DEFAULT_ACCOUNTING_STORAGE_TYPE "accounting_storage/none"
 #define DEFAULT_AUTH_TYPE          "auth/munge"
 #define DEFAULT_AUTH_TOKEN_LIFESPAN 1800
 #define DEFAULT_BATCH_START_TIMEOUT 10
 #define DEFAULT_BCAST_EXCLUDE       "/lib,/usr/lib,/lib64,/usr/lib64"
 #define DEFAULT_COMPLETE_WAIT       0
+#define DEFAULT_CERTGEN_TYPE "certgen/script"
 #define DEFAULT_CRED_TYPE           "cred/munge"
 #define DEFAULT_EPILOG_MSG_TIME     2000
-#define DEFAULT_EXT_SENSORS_TYPE    "ext_sensors/none"
 #define DEFAULT_FIRST_JOB_ID        1
-#define DEFAULT_GET_ENV_TIMEOUT     2
+#define DEFAULT_GET_ENV_TIMEOUT     120
 #define DEFAULT_GETNAMEINFO_CACHE_TIMEOUT 60
 #define DEFAULT_GROUP_TIME          600
 #define DEFAULT_GROUP_FORCE         1	/* if set, update group membership
@@ -98,25 +93,14 @@ typedef struct node_record node_record_t;
 /* NOTE: DEFAULT_INACTIVE_LIMIT must be 0 for Blue Gene/L systems */
 #define DEFAULT_INACTIVE_LIMIT      0
 #define DEFAULT_INTERACTIVE_STEP_OPTS "--interactive --preserve-env --pty $SHELL"
-#define DEFAULT_JOB_ACCT_GATHER_TYPE  "jobacct_gather/none"
-#define JOB_ACCT_GATHER_TYPE_NONE "jobacct_gather/none"
 #define DEFAULT_JOB_ACCT_GATHER_FREQ  "30"
-#define DEFAULT_ACCT_GATHER_ENERGY_TYPE "acct_gather_energy/none"
-#define DEFAULT_ACCT_GATHER_PROFILE_TYPE "acct_gather_profile/none"
-#define DEFAULT_ACCT_GATHER_INTERCONNECT_TYPE "acct_gather_interconnect/none"
-#define DEFAULT_ACCT_GATHER_FILESYSTEM_TYPE "acct_gather_filesystem/none"
-#define ACCOUNTING_STORAGE_TYPE_NONE "accounting_storage/none"
-#define DEFAULT_CORE_SPEC_PLUGIN    "core_spec/none"
 #define DEFAULT_ENFORCE_PART_LIMITS 0
-#define DEFAULT_JOB_COMP_TYPE       "jobcomp/none"
-#if defined HAVE_NATIVE_CRAY
-#  define DEFAULT_ALLOW_SPEC_RESOURCE_USAGE 1
-#  define DEFAULT_JOB_CONTAINER_PLUGIN  "job_container/cncu"
-#else
-#  define DEFAULT_ALLOW_SPEC_RESOURCE_USAGE 0
-#  define DEFAULT_JOB_CONTAINER_PLUGIN "job_container/none"
-#endif
+#define DEFAULT_ALLOW_SPEC_RESOURCE_USAGE 0
+#define DEFAULT_HASH_PLUGIN "hash/k12"
+#define DEFAULT_HOST_UNREACH_RETRY_COUNT 0
 #define DEFAULT_KEEPALIVE_TIME (NO_VAL)
+#define DEFAULT_KEEPALIVE_INTERVAL (NO_VAL)
+#define DEFAULT_KEEPALIVE_PROBES (NO_VAL)
 #define DEFAULT_KILL_ON_BAD_EXIT    0
 #define DEFAULT_KILL_TREE           0
 #define DEFAULT_KILL_WAIT           30
@@ -128,25 +112,21 @@ typedef struct node_record node_record_t;
 #define DEFAULT_MAX_JOB_COUNT       10000
 #define DEFAULT_MAX_JOB_ID          0x03ff0000
 #define DEFAULT_MAX_STEP_COUNT      40000
-#define DEFAULT_MCS_PLUGIN          "mcs/none"
 #define DEFAULT_MEM_PER_CPU         0
 #define DEFAULT_MAX_MEM_PER_CPU     0
 #define DEFAULT_MIN_JOB_AGE         300
-#define DEFAULT_MPI_DEFAULT         "none"
 #define DEFAULT_MSG_AGGR_WINDOW_MSGS 1
 #define DEFAULT_MSG_AGGR_WINDOW_TIME 100
 #define DEFAULT_MSG_TIMEOUT         10
-#define DEFAULT_POWER_PLUGIN        ""
 #if defined WITH_CGROUP
 #  define DEFAULT_PROCTRACK_TYPE      "proctrack/cgroup"
 #else
 #  define DEFAULT_PROCTRACK_TYPE      "proctrack/pgid"
 #endif
-#define DEFAULT_PREEMPT_TYPE        "preempt/none"
 #define DEFAULT_PREP_PLUGINS        "prep/script"
 #define DEFAULT_PRIORITY_DECAY      604800 /* 7 days */
 #define DEFAULT_PRIORITY_CALC_PERIOD 300 /* in seconds */
-#define DEFAULT_PRIORITY_TYPE       "priority/basic"
+#define DEFAULT_PRIORITY_TYPE       "priority/multifactor"
 #define DEFAULT_RECONF_KEEP_PART_STATE 0
 #define DEFAULT_RETURN_TO_SERVICE   0
 #define DEFAULT_RESUME_RATE         300
@@ -156,12 +136,7 @@ typedef struct node_record node_record_t;
 #define DEFAULT_SCHED_LOG_LEVEL     0
 #define DEFAULT_SCHED_TIME_SLICE    30
 #define DEFAULT_SCHEDTYPE           "sched/backfill"
-#if defined HAVE_NATIVE_CRAY
-#  define DEFAULT_SELECT_TYPE       "select/cray_aries"
-#else
-#  define DEFAULT_SELECT_TYPE       "select/linear"
-#endif
-#define DEFAULT_SITE_FACTOR_PLUGIN  "site_factor/none"
+#define DEFAULT_SELECT_TYPE         "select/cons_tres"
 #define DEFAULT_SLURMCTLD_PIDFILE   "/var/run/slurmctld.pid"
 #define DEFAULT_SLURMCTLD_TIMEOUT   120
 #define DEFAULT_SLURMD_PIDFILE      "/var/run/slurmd.pid"
@@ -175,43 +150,22 @@ typedef struct node_record node_record_t;
 #define DEFAULT_SUSPEND_RATE        60
 #define DEFAULT_SUSPEND_TIME        0
 #define DEFAULT_SUSPEND_TIMEOUT     30
-#if defined HAVE_NATIVE_CRAY
-#  define DEFAULT_SWITCH_TYPE         "switch/cray_aries"
-#else
-#  define DEFAULT_SWITCH_TYPE         "switch/none"
-#endif
-#define DEFAULT_TASK_PLUGIN         "task/none"
 #define DEFAULT_TCP_TIMEOUT         2
+#define DEFAULT_TLS_TYPE "tls/none"
 #define DEFAULT_TMP_FS              "/tmp"
-#if defined HAVE_3D
-#  define DEFAULT_TOPOLOGY_PLUGIN     "topology/3d_torus"
-#else
-#  define DEFAULT_TOPOLOGY_PLUGIN     "topology/none"
-#endif
+#define DEFAULT_TOPOLOGY_PLUGIN "topology/flat"
 #define DEFAULT_WAIT_TIME           0
-#  define DEFAULT_TREE_WIDTH        50
+#define DEFAULT_TREE_WIDTH	    16
 #define DEFAULT_UNKILLABLE_TIMEOUT  60 /* seconds */
-#define DEFAULT_BATCH_SCRIPT_LIMIT (4 * 1024 * 1024) /* 5MB */
+#define DEFAULT_BATCH_SCRIPT_LIMIT (4 * 1024 * 1024) /* 4MB */
 #define MAX_BATCH_SCRIPT_SIZE (512 * 1024 * 1024) /* 512MB */
+#define DEFAULT_MAX_SUBMIT_LINE_SIZE (1024 * 1024) /* 1MB */
+#define MAX_MAX_SUBMIT_LINE_SIZE (2 * 1024 * 1024) /* 2MB */
 
 /* MAX_TASKS_PER_NODE is defined in slurm.h
  */
 #define DEFAULT_MAX_TASKS_PER_NODE  MAX_TASKS_PER_NODE
 
-typedef struct slurm_conf_frontend {
-	char *allow_groups;		/* allowed group string */
-	char *allow_users;		/* allowed user string */
-	char *deny_groups;		/* denied group string */
-	char *deny_users;		/* denied user string */
-	char *frontends;		/* frontend node name */
-	char *addresses;		/* frontend node address */
-	uint16_t port;			/* frontend specific port */
-	char *reason;			/* reason for down frontend node */
-	uint16_t node_state;		/* enum node_states, ORed with
-					 * NODE_STATE_NO_RESPOND if not
-					 * responding */
-} slurm_conf_frontend_t;
-
 typedef struct slurm_conf_node {
 	char *nodenames;
 	char *hostnames;
@@ -231,8 +185,11 @@ typedef struct slurm_conf_node {
 	uint64_t real_memory;	/* MB real memory on the node */
 	uint64_t mem_spec_limit; /* MB real memory for memory specialization */
 	char *reason;
+	uint16_t res_cores_per_gpu; /* number of cores per GPU to allow
+				     * to only GPU jobs */
 	char *state;
 	uint32_t tmp_disk;	/* MB total storage in TMP_FS file system */
+	char *topology_str; /* topology address string */
 	char *tres_weights_str;	/* per TRES billing weight string */
 	uint32_t weight;	/* arbitrary priority of node for
 				 * scheduling work on */
@@ -263,9 +220,10 @@ typedef struct slurm_conf_partition {
 	uint8_t disable_root_jobs; /* if set then user root can't run jobs
 				    * if NO_VAL8, use global default */
 	bool exclusive_user; /* true if node allocations by user */
+	bool exclusive_topo; /* true if exclusive topology*/
 	uint32_t grace_time;	/* default grace time for partition */
 	bool     hidden_flag;	/* 1 if hidden by default */
-	List job_defaults_list;	/* List of job_defaults_t elements */
+	list_t *job_defaults_list; /* List of job_defaults_t elements */
 	bool     lln_flag;	/* 1 if nodes are selected in LLN order */
 	uint32_t max_cpus_per_node; /* maximum allocated CPUs per node */
 	uint32_t max_cpus_per_socket; /* maximum allocated CPUs per socket */
@@ -295,6 +253,7 @@ typedef struct slurm_conf_partition {
 				 * mode */
 	uint16_t suspend_timeout; /* time required in order to perform a node
 				   * suspend operation */
+	char *topology_name;
 	uint32_t total_nodes;	/* total number of nodes in the partition */
 	uint32_t total_cpus;	/* total number of cpus in the partition */
 } slurm_conf_partition_t;
@@ -318,25 +277,11 @@ typedef struct {
 
 typedef struct {
 	char *name;
-	List key_pairs;
+	list_t *key_pairs;
 } config_plugin_params_t;
 
-/*
- * Get result of configuration file test.
- * RET SLURM_SUCCESS or error code
- */
-extern int config_test_result(void);
-
-/*
- * Start configuration file test mode. Disables fatal errors.
- */
-extern void config_test_start(void);
-
-/* Destroy a front_end record built by slurm_conf_frontend_array() */
-extern void destroy_frontend(void *ptr);
-
 /* Copy list of job_defaults_t elements */
-extern List job_defaults_copy(List in_list);
+extern list_t *job_defaults_copy(list_t *in_list);
 
 /* Destroy list of job_defaults_t elements */
 extern void job_defaults_free(void *x);
@@ -347,13 +292,13 @@ extern void job_defaults_free(void *x);
  * out_list OUT - equivalent list of key=value pairs
  * Returns SLURM_SUCCESS or an error code
  */
-extern int job_defaults_list(char *in_str, List *out_list);
+extern int job_defaults_list(char *in_str, list_t **out_list);
 
 /*
  * Translate list of job_defaults_t elements into a string.
  * Return value must be released using xfree()
  */
-extern char *job_defaults_str(List in_list);
+extern char *job_defaults_str(list_t *in_list);
 
 /* Pack a job_defaults_t element. Used by slurm_pack_list() */
 extern void job_defaults_pack(void *in, uint16_t protocol_version,
@@ -387,12 +332,25 @@ extern void read_conf_recv_stepd(int fd);
  *
  * IN: list_t *key_pair_list
  * IN: char *key
- * IN: char *value
+ * IN: char *fmt - printf style format of "value" args that follow
  */
-extern void read_config_add_key_pair(list_t *key_pair_list,
-				     char *key,
-				     char *value);
+extern void add_key_pair(list_t *key_pair_list, const char *key,
+			 const char *fmt, ...)
+	__attribute__((format(printf, 3, 4)));
 
+/*
+ * Allocate a config_key_pair_t, duplicate the key, and set the value
+ * to "yes" or "no".
+ */
+extern void add_key_pair_bool(list_t *key_pair_list, const char *key,
+			      bool value);
+
+/*
+ * Allocate a config_key_pair_t, duplicate the key, and take ownership of the
+ * value string.
+ */
+extern void add_key_pair_own(list_t *key_pair_list, const char *key,
+			     char *value);
 
 /*
  * slurm_conf_init_stepd - Since the stepd does not read in the file and
@@ -400,24 +358,6 @@ extern void read_config_add_key_pair(list_t *key_pair_list,
  */
 extern void slurm_conf_init_stepd(void);
 
-/*
- * slurm_conf_init_load - load the slurm configuration from the a file.
- * IN file_name - name of the slurm configuration file to be read
- *	If file_name is NULL, then this routine tries to use
- *	the value in the SLURM_CONF env variable.  Failing that,
- *	it uses the compiled-in default file name.
- *	If the conf structures have already been initialized by a call to
- *	slurm_conf_init, any subsequent calls will do nothing until
- *	slurm_conf_destroy is called.
- * IN load_auth - If true, load the auth and hash plugins.
- *      NOTE: false should only be passed to this if doing unit testing where we
- *            know we are suppose to load any plugins.
- * RET SLURM_SUCCESS if conf file is initialized.  If the slurm conf
- *       was already initialied, return SLURM_ERROR.
- *
- */
-extern int slurm_conf_init_load(const char *file_name, bool load_auth);
-
 /*
  * slurm_conf_init - load the slurm configuration from the a file.
  * IN file_name - name of the slurm configuration file to be read
@@ -428,7 +368,7 @@ extern int slurm_conf_init_load(const char *file_name, bool load_auth);
  *	slurm_conf_init, any subsequent calls will do nothing until
  *	slurm_conf_destroy is called.
  * RET SLURM_SUCCESS if conf file is initialized.  If the slurm conf
- *       was already initialied, return SLURM_ERROR.
+ *       was already initialized, return SLURM_ERROR.
  * NOTE: Caller must NOT be holding slurm_conf_lock().
  */
 extern int slurm_conf_init(const char *file_name);
@@ -466,13 +406,7 @@ extern slurm_conf_t *slurm_conf_lock(void);
 extern void slurm_conf_unlock(void);
 
 
-/*
- * Set "ptr_array" with the pointer to an array of pointers to
- * slurm_conf_frontend_t structures.
- *
- * Return value is the length of the array.
- */
-extern int slurm_conf_frontend_array(slurm_conf_frontend_t **ptr_array[]);
+extern int slurm_conf_check_addr(const char *node_name, bool *dynamic);
 
 /*
  * Set "ptr_array" with the pointer to an array of pointers to
@@ -660,7 +594,7 @@ extern char *debug_flags2str(uint64_t debug_flags);
  * debug_str2flags - Convert a DebugFlags string to the equivalent uint64_t
  * Returns SLURM_ERROR if invalid
  */
-extern int debug_str2flags(char *debug_flags, uint64_t *flags_out);
+extern int debug_str2flags(const char *debug_flags, uint64_t *flags_out);
 
 /*
  * reconfig_flags2str - convert a ReconfigFlags uint16_t to the equivalent string
@@ -728,11 +662,15 @@ extern void slurm_conf_add_node(node_record_t *node_ptr);
  */
 extern void slurm_conf_remove_node(char *node_name);
 
-#ifdef HAVE_FRONT_END
 /*
- * Return the frontend port for the given hostname.
+ * Get substring from a csv-style string.
+ *
+ * IN opts - csv-style string to be parsed
+ * IN arg - the search string + delimiter between option and its value
+ * 	(EG: "planet=")
+ *
+ * RET xmalloc()'d string after the delimiter, and before next ","
  */
-extern uint16_t slurm_conf_get_frontend_port(char *node_hostname);
-#endif
+extern char *conf_get_opt_str(const char *opts, const char *arg);
 
 #endif /* !_READ_CONFIG_H */
