@@ -72,7 +72,6 @@
 
 /* bitstring.[ch] functions*/
 #define	bit_alloc		slurm_bit_alloc
-#define	bit_test		slurm_bit_test
 #define	bit_set			slurm_bit_set
 #define	bit_clear		slurm_bit_clear
 #define	bit_nclear		slurm_bit_nclear
@@ -88,7 +87,6 @@
 #define	bit_set_count		slurm_bit_set_count
 #define	bit_set_count_range	slurm_bit_set_count_range
 #define	bit_clear_count		slurm_bit_clear_count
-#define	bit_clear_count_range	slurm_bit_clear_count_range
 #define	bit_nset_max_count	slurm_bit_nset_max_count
 #define	bit_rotate_copy		slurm_bit_rotate_copy
 #define	bit_rotate		slurm_bit_rotate
@@ -99,8 +97,6 @@
 #define	bit_fmt_hexmask		slurm_bit_fmt_hexmask
 #define	bit_fmt_hexmask_trim	slurm_bit_fmt_hexmask_trim
 #define bit_unfmt_hexmask	slurm_bit_unfmt_hexmask
-#define	bit_fmt_binmask		slurm_bit_fmt_binmask
-#define bit_unfmt_binmask	slurm_bit_unfmt_binmask
 #define	bit_fls			slurm_bit_fls
 #define	bit_fls_from_bit	slurm_bit_fls_from_bit
 #define	bit_fill_gaps		slurm_bit_fill_gaps
@@ -118,16 +114,18 @@
 
 /* fd.[ch] functions */
 #define closeall		slurm_closeall
+#define closeall_except		slurm_closeall_except
 #define fd_set_blocking		slurm_fd_set_blocking
 #define fd_set_nonblocking	slurm_fd_set_nonblocking
 #define fd_get_socket_error	slurm_fd_get_socket_error
-#define send_fd_over_pipe	slurm_send_fd_over_pipe
-#define receive_fd_over_pipe	slurm_receive_fd_over_pipe
+#define send_fd_over_socket	slurm_send_fd_over_socket
+#define receive_fd_over_socket	slurm_receive_fd_over_socket
 #define rmdir_recursive		slurm_rmdir_recursive
 
 /* hostlist.[ch] functions */
 #define	hostlist_create_dims	slurm_hostlist_create_dims
 #define	hostlist_create		slurm_hostlist_create
+#define	hostlist_create_client	slurm_hostlist_create_client
 #define	hostlist_copy		slurm_hostlist_copy
 #define	hostlist_count		slurm_hostlist_count
 #define	hostlist_delete		slurm_hostlist_delete
@@ -163,6 +161,8 @@
 #define	hostlist_shift		slurm_hostlist_shift
 #define	hostlist_shift_dims	slurm_hostlist_shift_dims
 #define	hostlist_sort		slurm_hostlist_sort
+#define	hostlist_split_treewidth \
+				slurm_hostlist_split_treewidth
 #define	hostlist_cmp_first	slurm_hostlist_cmp_first
 #define	hostlist_uniq		slurm_hostlist_uniq
 #define	hostset_count		slurm_hostset_count
@@ -267,6 +267,7 @@
 #define rehash_node		slurm_rehash_node
 #define hostlist2bitmap		slurm_hostlist2bitmap
 #define bitmap2node_name	slurm_bitmap2node_name
+#define node_name2bitmap	slurm_node_name2bitmap
 #define find_node_record	slurm_find_node_record
 
 /* pack.[ch] functions */
@@ -304,6 +305,8 @@
 #define	unpackstr_xmalloc	slurm_unpackstr_xmalloc
 #define	unpackstr_xmalloc_escaped slurm_unpackstr_xmalloc_escaped
 #define	unpackstr_xmalloc_chooser slurm_unpackstr_xmalloc_chooser
+#define packstr_func slurm_packstr_func
+#define safe_unpackstr_func slurm_safe_unpackstr_func
 #define	packstr_array		slurm_packstr_array
 #define	unpackstr_array		slurm_unpackstr_array
 #define	packmem_array		slurm_packmem_array
@@ -343,20 +346,17 @@
 #define add_key_pair_own	slurm_add_key_pair_own
 
 /* run_in_daemon.[ch] functions */
-#define run_in_daemon           slurm_run_in_daemon
-#define running_in_daemon	slurm_running_in_daemon
-#define running_in_sackd        slurm_running_in_sackd
-#define running_in_slurmctld    slurm_running_in_slurmctld
-#define running_in_slurmd       slurm_running_in_slurmd
-#define running_in_slurmdbd     slurm_running_in_slurmdbd
-#define running_in_slurmd_stepd slurm_running_in_slurmd_stepd
-#define running_in_slurmrestd	slurm_running_in_slurmrestd
-#define running_in_slurmstepd   slurm_running_in_slurmstepd
+#define run_in_daemon slurm_run_in_daemon
 
 /* slurm_auth.[ch] functions
  * None exported today.
  * The header file used only for #define values. */
 
+/* state_save.[ch] functions */
+#define lock_state_files	slurm_lock_state_files
+#define unlock_state_files	slurm_unlock_state_files
+#define save_buf_to_state	slurm_save_buf_to_state
+
 /* strlcpy.[ch] functions */
 #ifndef HAVE_STRLCPY
 #define	strlcpy			slurm_strlcpy
@@ -535,6 +535,7 @@
 
 /* topology.[ch] functions */
 #define topology_g_build_config slurm_topology_g_build_config
+#define topology_g_destroy_config slurm_topology_g_destroy_config
 
 /* job_state_reason.[ch] functions */
 #define job_state_reason_string	slurm_job_state_reason_string
@@ -542,11 +543,23 @@
 #define job_state_reason_check slurm_job_state_reason_check
 
 /* serializer.[ch] functions */
+#define serializer_required slurm_serializer_required
 #define serializer_g_init slurm_serializer_g_init
 #define serialize_g_data_to_string slurm_serialize_g_data_to_string
 #define serialize_g_string_to_data slurm_serialize_g_string_to_data
 #define serializer_g_fini slurm_serializer_g_fini
 
+/* certgen.[ch] functions */
+#define certgen_g_init slurm_certgen_g_init
+#define certgen_g_fini slurm_certgen_g_fini
+#define certgen_g_self_signed slurm_certgen_g_self_signed
+
+/* fetch_config.[ch] functions */
+#define dump_to_memfd slurm_dump_to_memfd
+
+/* run_command.[ch] functions */
+#define run_command slurm_run_command
+
 #endif /* USE_ALIAS */
 
 #endif /*__SLURM_XLATOR_H__*/
