@@ -64,6 +64,7 @@ typedef struct {
 	int magic; /* MAGIC_EXTRACT_FD */
 	int input_fd;
 	int output_fd;
+	void *tls_conn; /* TLS state */
 	conmgr_extract_fd_func_t func;
 	const char *func_name;
 	void *func_arg;
@@ -73,7 +74,7 @@ typedef struct {
 #define MAGIC_WORK 0xD231444A
 	int magic; /* MAGIC_WORK */
 	conmgr_work_status_t status;
-	conmgr_fd_t *con;
+	conmgr_fd_ref_t *ref;
 	conmgr_callback_t callback;
 	conmgr_work_control_t control;
 } work_t;
@@ -114,6 +115,9 @@ typedef enum {
 	 * 	con (will not be moved)
 	 * 	arg
 	 *	FLAG_ON_DATA_TRIED
+	 *	tls
+	 *	tls_in
+	 *	tls_out
 	 *
 	 */
 	FLAG_WORK_ACTIVE = SLURM_BIT(8),
@@ -135,6 +139,18 @@ typedef enum {
 	FLAG_WATCH_READ_TIMEOUT = CON_FLAG_WATCH_READ_TIMEOUT,
 	/* @see CON_FLAG_WATCH_CONNECT_TIMEOUT */
 	FLAG_WATCH_CONNECT_TIMEOUT = CON_FLAG_WATCH_CONNECT_TIMEOUT,
+	/* @see CON_FLAG_TLS_SERVER */
+	FLAG_TLS_SERVER = CON_FLAG_TLS_SERVER,
+	/* @see CON_FLAG_TLS_CLIENT */
+	FLAG_TLS_CLIENT = CON_FLAG_TLS_CLIENT,
+	/* True if conn_g_create() completed */
+	FLAG_IS_TLS_CONNECTED = SLURM_BIT(20),
+	/* True if on_fingerprint() pending */
+	FLAG_WAIT_ON_FINGERPRINT = SLURM_BIT(21),
+	/* True if waiting for time delayed close of input_fd&output_fd */
+	FLAG_TLS_WAIT_ON_CLOSE = SLURM_BIT(22),
+	/* @see CON_FLAG_RPC_RECV_FORWARD */
+	FLAG_RPC_RECV_FORWARD = CON_FLAG_RPC_RECV_FORWARD,
 } con_flags_t;
 
 /* Mask over flags that track connection state */
@@ -142,8 +158,7 @@ typedef enum {
 	( FLAG_ON_DATA_TRIED | FLAG_IS_SOCKET | FLAG_IS_LISTEN | \
 	  FLAG_WAIT_ON_FINISH | FLAG_CAN_WRITE | FLAG_CAN_READ | \
 	  FLAG_READ_EOF | FLAG_IS_CONNECTED | FLAG_WORK_ACTIVE | \
-	  FLAG_QUIESCE | FLAG_CAN_QUERY_OUTPUT_BUFFER | \
-	  FLAG_IS_FIFO | FLAG_IS_CHR )
+	  FLAG_CAN_QUERY_OUTPUT_BUFFER | FLAG_IS_FIFO | FLAG_IS_CHR )
 
 /* con_flags_t macro helpers to test, set, and unset flags */
 #define con_flag(con, flag) ((con)->flags & (flag))
@@ -186,10 +201,16 @@ struct conmgr_fd_s {
 	slurm_addr_t address;
 	/* call backs for events */
 	const conmgr_events_t *events;
+	/* Opaque pointer to TLS state */
+	void *tls;
+	/* buffer holding incoming already read encrypted data */
+	buf_t *tls_in;
 	/* buffer holding incoming already read data */
 	buf_t *in;
 	/* timestamp when last read() got >0 bytes or when connect() called */
 	timespec_t last_read;
+	/* list of buf_t holding outgoing encrypted data */
+	list_t *tls_out;
 	/* list of buf_t to write (in order) */
 	list_t *out;
 	/* timestamp when last write() wrote >0 bytes */
@@ -341,6 +362,10 @@ typedef struct {
 		bool requested;
 		/* Has conmgr quiesced */
 		bool active;
+		/* Configured value of time to active timeout */
+		timespec_t conf_timeout;
+		/* Timestamp when quiesce requested */
+		timespec_t start;
 		/* Event to broadcast when conmgr enters quiesced state */
 		event_signal_t on_start_quiesced;
 		/* Event to broadcast when conmgr exits quiesced state */
@@ -382,7 +407,7 @@ extern conmgr_t mgr;
  * IN control - controls on when work is run
  * IN depend_mask - Apply mask against control.depend_type.
  * 	Mask is intended for work that generates new work (such as signal work)
- * 	to make it relatively clean to remove a now fullfilled dependency.
+ * 	to make it relatively clean to remove a now fulfilled dependency.
  * 	Ignored if depend_mask=0.
  * IN caller - __func__ from caller
  */
@@ -424,6 +449,20 @@ extern void add_work(bool locked, conmgr_fd_t *con, conmgr_callback_t callback,
 							    delay_nanoseconds),\
 		}, 0, __func__)
 
+#define add_work_con_delayed_abs_fifo(locked, con, _func, func_arg, timestamp) \
+	add_work(locked, con,                                                  \
+		 (conmgr_callback_t) {                                         \
+			 .func = _func,                                        \
+			 .arg = func_arg,                                      \
+			 .func_name = #_func,                                  \
+		 },                                                            \
+		 (conmgr_work_control_t) {                                     \
+			 .depend_type = CONMGR_WORK_DEP_TIME_DELAY,            \
+			 .schedule_type = CONMGR_WORK_SCHED_FIFO,              \
+			 .time_begin = timestamp,                              \
+		 },                                                            \
+		 0, __func__)
+
 extern void work_mask_depend(work_t *work, conmgr_work_depend_t depend_mask);
 extern void handle_work(bool locked, work_t *work);
 
@@ -431,6 +470,7 @@ extern void handle_work(bool locked, work_t *work);
  * Poll all connections and handle any events
  */
 extern void *watch(void *arg);
+extern void *watch_thread(void *arg);
 
 /*
  * Wait for _watch() to finish
@@ -445,7 +485,7 @@ extern void wait_for_watch(void);
 extern void close_con(bool locked, conmgr_fd_t *con);
 
 /*
- * Stop writting to connection and drop remaining outbound buffer(s)
+ * Stop writing to connection and drop remaining outbound buffer(s)
  */
 extern void close_con_output(bool locked, conmgr_fd_t *con);
 
@@ -475,8 +515,18 @@ extern void con_close_on_poll_error(conmgr_fd_t *con, int fd);
 extern void con_set_polling(conmgr_fd_t *con, pollctl_fd_type_t type,
 			    const char *caller);
 
+/*
+ * Write out list of buf_t to output_fd
+ */
+extern void write_output(conmgr_fd_t *con, const int out_count, list_t *out);
+
 extern void handle_write(conmgr_callback_args_t conmgr_args, void *arg);
 
+/*
+ * Read input_fd into buffer
+ */
+extern void read_input(conmgr_fd_t *con, buf_t *buf, const char *what);
+
 extern void handle_read(conmgr_callback_args_t conmgr_args, void *arg);
 
 /*
@@ -500,8 +550,9 @@ extern void wrap_on_data(conmgr_callback_args_t conmgr_args, void *arg);
  * IN addrlen - number of bytes in *addr or 0 if addr==NULL
  * IN is_listen - True if this is a listening socket
  * IN unix_socket_path - Named Unix Socket path in filesystem or NULL
+ * IN tls_conn - TLS connection state or NULL
  * IN arg - arbitrary pointer to hand to events
- * RET SLURM_SUCCESS or errror
+ * RET SLURM_SUCCESS or error
  */
 extern int add_connection(conmgr_con_type_t type,
 			  conmgr_fd_t *source, int input_fd,
@@ -510,7 +561,8 @@ extern int add_connection(conmgr_con_type_t type,
 			  conmgr_con_flags_t flags,
 			  const slurm_addr_t *addr,
 			  socklen_t addrlen, bool is_listen,
-			  const char *unix_socket_path, void *arg);
+			  const char *unix_socket_path, void *tls_conn,
+			  void *arg);
 
 extern void close_all_connections(void);
 
@@ -576,4 +628,34 @@ extern void wrap_on_connection(conmgr_callback_args_t conmgr_args, void *arg);
  */
 extern void extract_con_fd(conmgr_fd_t *con);
 
+/*
+ * Create new connection reference
+ * WARNING: caller must hold mgr.mutex
+ */
+extern conmgr_fd_ref_t *fd_new_ref(conmgr_fd_t *con);
+
+/*
+ * Release and free connection reference
+ * WARNING: caller must hold mgr.mutex
+ */
+extern void fd_free_ref(conmgr_fd_ref_t **ref_ptr);
+
+/*
+ * Get conmgr_fd_t pointer from reference
+ */
+extern conmgr_fd_t *fd_get_ref(conmgr_fd_ref_t *ref);
+
+/*
+ * handle connection states and apply actions required.
+ * IN locked - true if mgr->mutex is locked
+ * IN con - connection to process state
+ */
+extern void handle_connection(bool locked, conmgr_fd_t *con);
+
+/*
+ * Queue up wrap_on_connection() to call events->on_connection() callback
+ * NOTE: caller must hold mgr->mutex lock
+ */
+extern void queue_on_connection(conmgr_fd_t *con);
+
 #endif /* _CONMGR_MGR_H */
