@@ -51,6 +51,7 @@
 
 #include <mysql.h>
 #include <mysqld_error.h>
+#include <errmsg.h>
 
 typedef enum {
 	SLURM_MYSQL_PLUGIN_NOTSET,
@@ -64,8 +65,10 @@ typedef struct {
 	uint32_t flags;
 	pthread_mutex_t lock;
 	char *pre_commit_query;
-	List update_list;
+	list_t *update_list;
 	int conn;
+	uint64_t wsrep_trx_fragment_size_orig;
+	char *wsrep_trx_fragment_unit_orig;
 } mysql_conn_t;
 
 typedef struct {
@@ -91,7 +94,7 @@ extern int destroy_mysql_db_info(mysql_db_info_t *db_info);
 extern int mysql_db_get_db_connection(mysql_conn_t *mysql_conn, char *db_name,
 				   mysql_db_info_t *db_info);
 extern int mysql_db_close_db_connection(mysql_conn_t *mysql_conn);
-extern int mysql_db_cleanup();
+extern int mysql_db_cleanup(void);
 extern int mysql_db_query(mysql_conn_t *mysql_conn, char *query);
 extern int mysql_db_delete_affected_rows(mysql_conn_t *mysql_conn, char *query);
 extern int mysql_db_ping(mysql_conn_t *mysql_conn);
@@ -106,5 +109,12 @@ extern uint64_t mysql_db_insert_ret_id(mysql_conn_t *mysql_conn, char *query);
 
 extern int mysql_db_create_table(mysql_conn_t *mysql_conn, char *table_name,
 				 storage_field_t *fields, char *ending);
-
+extern int mysql_db_get_var_str(mysql_conn_t *mysql_conn,
+				const char *variable_name,
+				char **value);
+extern int mysql_db_get_var_u64(mysql_conn_t *mysql_conn,
+	    			 const char *variable_name,
+			    	 uint64_t *value);
+extern void mysql_db_enable_streaming_replication(mysql_conn_t *mysql_conn);
+extern void mysql_db_restore_streaming_replication(mysql_conn_t *mysql_conn);
 #endif
