@@ -50,7 +50,6 @@
 #include "src/common/bitstring.h"
 #include "src/common/group_cache.h"
 #include "src/common/identity.h"
-#include "src/common/io_hdr.h"
 #include "src/common/job_resources.h"
 #include "src/common/list.h"
 #include "src/common/log.h"
@@ -363,12 +362,12 @@ extern slurm_cred_arg_t *slurm_cred_verify(slurm_cred_t *cred)
 	/* NOTE: the verification checks that the credential was
 	 * created by SlurmUser or root */
 	if (!cred->verified) {
-		slurm_seterrno(ESLURMD_INVALID_JOB_CREDENTIAL);
+		errno = ESLURMD_INVALID_JOB_CREDENTIAL;
 		goto error;
 	}
 
 	if (now > (cred->ctime + cred_expire)) {
-		slurm_seterrno(ESLURMD_CREDENTIAL_EXPIRED);
+		errno = ESLURMD_CREDENTIAL_EXPIRED;
 		goto error;
 	}
 
@@ -376,9 +375,9 @@ extern slurm_cred_arg_t *slurm_cred_verify(slurm_cred_t *cred)
 	return cred->arg;
 
 error:
-	errnum = slurm_get_errno();
+	errnum = errno;
 	slurm_rwlock_unlock(&cred->mutex);
-	slurm_seterrno(errnum);
+	errno = errnum;
 	return NULL;
 }
 
@@ -522,11 +521,7 @@ extern void format_core_allocs(slurm_cred_t *credential, char *node_name,
 		      cred->job_hostlist);
 		return;
 	}
-#ifdef HAVE_FRONT_END
-	host_index = 0;
-#else
 	host_index = hostlist_find(hset, node_name);
-#endif
 	if ((host_index < 0) || (host_index >= cred->job_nhosts)) {
 		error("Invalid host_index %d for job %u",
 		      host_index, cred->step_id.job_id);
@@ -609,11 +604,7 @@ extern void get_cred_gres(slurm_cred_t *credential, char *node_name,
 		      cred->job_hostlist);
 		return;
 	}
-#ifdef HAVE_FRONT_END
-	host_index = 0;
-#else
 	host_index = hostlist_find(hset, node_name);
-#endif
 	hostlist_destroy(hset);
 	if ((host_index < 0) || (host_index >= cred->job_nhosts)) {
 		error("Invalid host_index %d for job %u",
@@ -750,6 +741,7 @@ extern sbcast_cred_t *unpack_sbcast_cred(buf_t *buffer, void *msg,
 extern void print_sbcast_cred(sbcast_cred_t *sbcast_cred)
 {
 	info("Sbcast_cred: JobId   %u", sbcast_cred->arg.job_id);
+	info("Sbcast_cred: HetJobId %u", sbcast_cred->arg.het_job_id);
 	info("Sbcast_cred: StepId  %u", sbcast_cred->arg.step_id);
 	info("Sbcast_cred: Nodes   %s", sbcast_cred->arg.nodes);
 	info("Sbcast_cred: ctime   %s", slurm_ctime2(&sbcast_cred->ctime));
