@@ -147,6 +147,8 @@ typedef struct {
 #define GRES_CONF_FROM_STATE SLURM_BIT(13) /* Flags from state, blow away once
 					      node checks in. */
 #define GRES_CONF_GLOBAL_INDEX SLURM_BIT(14) /* devices use global index */
+#define GRES_CONF_AUTODETECT SLURM_BIT(15) /* Conf was made with Autodetect */
+#define GRES_CONF_UPDATE_CONFIG SLURM_BIT(16) /* Flag to update gres config */
 
 #define GRES_CONF_ENV_SET    0x000008E0   /* Easy check if any of
 					   * GRES_CONF_ENV_* are set. */
@@ -735,7 +737,7 @@ extern char *gres_get_node_used(list_t *gres_list);
  * Give the total system count of a given GRES
  * Returns NO_VAL64 if name not found
  */
-extern uint64_t gres_get_system_cnt(char *name);
+extern uint64_t gres_get_system_cnt(char *name, bool case_insensitive);
 
 /*
  * Get the count of a node's GRES
@@ -764,16 +766,21 @@ extern int gres_node_count(list_t *gres_list, int arr_len,
  * IN gres_list - generated by gres_job_config_validate()
  * IN/OUT buffer - location to write state to
  */
-extern int gres_prep_pack(list_t *gres_list, buf_t *buffer,
-			  uint16_t protocol_version);
+extern void gres_prep_pack(void *in, uint16_t protocol_version, buf_t *buffer);
+
+extern int gres_prep_pack_legacy(list_t *gres_list, buf_t *buffer,
+				 uint16_t protocol_version);
 
 /*
  * Unpack a job's allocated gres information for use by prolog/epilog
  * OUT gres_list - restored state stored by gres_prep_pack()
  * IN/OUT buffer - location to read state from
  */
-extern int gres_prep_unpack(list_t **gres_list, buf_t *buffer,
-			    uint16_t protocol_version);
+extern int gres_prep_unpack_list(list_t **gres_list, buf_t *buffer,
+				 uint16_t protocol_version);
+
+extern int gres_prep_unpack_legacy(list_t **gres_list, buf_t *buffer,
+				   uint16_t protocol_version);
 
 /*
  * Build List of information needed to set job's Prolog or Epilog environment
@@ -802,7 +809,7 @@ extern void gres_g_prep_set_env(char ***prep_env_ptr,
  * Note: This function can be used for a new request with gres_list==NULL or
  *	 used to update an existing job, in which case gres_list is a copy
  *	 of the job's original value (so we can clear fields as needed)
- * IN/OUT gres_js_val - with ->*tres* set approriately to be processed into.
+ * IN/OUT gres_js_val - with ->*tres* set appropriately to be processed into.
  * RET SLURM_SUCCESS or ESLURM_INVALID_GRES
  */
 extern int gres_job_state_validate(gres_job_state_validate_t *gres_js_val);
