@@ -128,7 +128,7 @@ static int _setup_federation_rec_limits(slurmdb_federation_rec_t *fed,
  * IN: exceptions - list of clusters to not remove.
  */
 static int _remove_all_clusters_from_fed(mysql_conn_t *mysql_conn,
-					 const char *fed, List exceptions)
+					 const char *fed, list_t *exceptions)
 {
 	int   rc    = SLURM_SUCCESS;
 	char *query = NULL;
@@ -146,9 +146,9 @@ static int _remove_all_clusters_from_fed(mysql_conn_t *mysql_conn,
 		list_iterator_destroy(itr);
 	}
 
-	xstrfmtcat(query, "UPDATE %s "
-		   	  "SET federation='', fed_id=0, fed_state=%u "
-			  "WHERE federation='%s' and deleted=0",
+	xstrfmtcat(query,
+		   "UPDATE %s SET federation='', fed_id=0, fed_state=%u "
+		   "WHERE federation='%s' and deleted=0",
 		   cluster_table, CLUSTER_FED_STATE_NA, fed);
 	if (exception_names)
 		xstrfmtcat(query, " AND name NOT IN (%s)", exception_names);
@@ -166,7 +166,7 @@ static int _remove_all_clusters_from_fed(mysql_conn_t *mysql_conn,
 	return rc;
 }
 
-static int _remove_clusters_from_fed(mysql_conn_t *mysql_conn, List clusters)
+static int _remove_clusters_from_fed(mysql_conn_t *mysql_conn, list_t *clusters)
 {
 	int   rc    = SLURM_SUCCESS;
 	char *query = NULL;
@@ -178,11 +178,11 @@ static int _remove_clusters_from_fed(mysql_conn_t *mysql_conn, List clusters)
 
 	itr = list_iterator_create(clusters);
 	while ((name = list_next(itr)))
-	       xstrfmtcat(names, "%s'%s'", names ? "," : "", name );
+		xstrfmtcat(names, "%s'%s'", names ? "," : "", name );
 
-	xstrfmtcat(query, "UPDATE %s "
-		   	  "SET federation='', fed_id=0, fed_state=%u "
-			  "WHERE name IN (%s) and deleted=0",
+	xstrfmtcat(query,
+		   "UPDATE %s SET federation='', fed_id=0, fed_state=%u "
+		   "WHERE name IN (%s) and deleted=0",
 		   cluster_table, CLUSTER_FED_STATE_NA, names);
 
 	DB_DEBUG(FEDR, mysql_conn->conn, "query\n%s", query);
@@ -196,7 +196,7 @@ static int _remove_clusters_from_fed(mysql_conn_t *mysql_conn, List clusters)
 	return rc;
 }
 
-static int _add_clusters_to_fed(mysql_conn_t *mysql_conn, List clusters,
+static int _add_clusters_to_fed(mysql_conn_t *mysql_conn, list_t *clusters,
 				const char *fed)
 {
 	int   rc      = SLURM_SUCCESS;
@@ -227,12 +227,12 @@ static int _add_clusters_to_fed(mysql_conn_t *mysql_conn, List clusters,
 	 * in the fed_state case statement must happen before fed_state is set
 	 * or the federation will always equal the federation in the case
 	 * statement.  */
-	xstrfmtcat(query, "UPDATE %s "
-		   	  "SET "
-			  "fed_state = CASE WHEN federation='%s' THEN fed_state ELSE %u END, "
-			  "fed_id = CASE %s END, "
-		   	  "federation='%s' "
-			  "WHERE name IN (%s) and deleted=0",
+	xstrfmtcat(query,
+		   "UPDATE %s SET "
+		   "fed_state = CASE WHEN federation='%s' THEN fed_state ELSE %u END, "
+		   "fed_id = CASE %s END, "
+		   "federation='%s' "
+		   "WHERE name IN (%s) and deleted=0",
 		   cluster_table, fed, CLUSTER_FED_STATE_ACTIVE, indexes, fed,
 		   names);
 
@@ -254,11 +254,11 @@ end_it:
 
 static int _assign_clusters_to_federation(mysql_conn_t *mysql_conn,
 					  const char *federation,
-					  List cluster_list)
+					  list_t *cluster_list)
 {
 	int  rc       = SLURM_SUCCESS;
-	List add_list = NULL;
-	List rem_list = NULL;
+	list_t *add_list = NULL;
+	list_t *rem_list = NULL;
 	list_itr_t *itr = NULL;
 	bool clear_clusters = false;
 	slurmdb_cluster_rec_t *tmp_cluster = NULL;
@@ -309,13 +309,13 @@ end_it:
 }
 
 extern int as_mysql_add_federations(mysql_conn_t *mysql_conn, uint32_t uid,
-				    List federation_list)
+				    list_t *federation_list)
 {
 	list_itr_t *itr = NULL;
 	int rc = SLURM_SUCCESS;
 	slurmdb_federation_rec_t *object = NULL;
 	char *cols = NULL, *vals = NULL, *extra = NULL, *query = NULL,
-	     *tmp_extra = NULL;
+		*tmp_extra = NULL;
 	time_t now = time(NULL);
 	char *user_name = NULL;
 	int affect_rows = 0;
@@ -422,13 +422,14 @@ extern int as_mysql_add_federations(mysql_conn_t *mysql_conn, uint32_t uid,
 	return rc;
 }
 
-extern List as_mysql_get_federations(mysql_conn_t *mysql_conn, uid_t uid,
-				     slurmdb_federation_cond_t *federation_cond)
+extern list_t *as_mysql_get_federations(
+	mysql_conn_t *mysql_conn, uid_t uid,
+	slurmdb_federation_cond_t *federation_cond)
 {
 	char *query = NULL;
 	char *extra = NULL;
 	char *tmp = NULL;
-	List federation_list = NULL;
+	list_t *federation_list = NULL;
 	int i=0;
 	MYSQL_RES *result = NULL;
 	MYSQL_ROW row;
@@ -473,7 +474,7 @@ empty:
 
 	while ((row = mysql_fetch_row(result))) {
  		slurmdb_cluster_cond_t clus_cond;
- 		List tmp_list = NULL;
+		list_t *tmp_list = NULL;
  		fed = xmalloc(sizeof(slurmdb_federation_rec_t));
  		list_append(federation_list, fed);
 
@@ -498,17 +499,17 @@ empty:
 	return federation_list;
 }
 
-extern List as_mysql_modify_federations(
-				mysql_conn_t *mysql_conn, uint32_t uid,
-				slurmdb_federation_cond_t *fed_cond,
-				slurmdb_federation_rec_t *fed)
+extern list_t *as_mysql_modify_federations(
+	mysql_conn_t *mysql_conn, uint32_t uid,
+	slurmdb_federation_cond_t *fed_cond,
+	slurmdb_federation_rec_t *fed)
 {
-	List ret_list = NULL;
+	list_t *ret_list = NULL;
 	int rc = SLURM_SUCCESS;
 	int req_inx = 0;
 	char *object = NULL;
 	char *vals = NULL, *extra = NULL, *query = NULL,
-	     *name_char = NULL, *fed_items = NULL;
+		*name_char = NULL, *fed_items = NULL;
 	char *tmp_char1 = NULL, *tmp_char2 = NULL;
 	time_t now = time(NULL);
 	MYSQL_RES *result = NULL;
@@ -631,17 +632,21 @@ extern List as_mysql_modify_federations(
 	return ret_list;
 }
 
-extern List as_mysql_remove_federations(mysql_conn_t *mysql_conn, uint32_t uid,
-					slurmdb_federation_cond_t *fed_cond)
+extern list_t *as_mysql_remove_federations(mysql_conn_t *mysql_conn,
+					   uint32_t uid,
+					   slurmdb_federation_cond_t *fed_cond)
 {
-	List ret_list = NULL;
 	int rc = SLURM_SUCCESS;
-	char *extra = NULL, *query = NULL, *name_char = NULL;
-	time_t now = time(NULL);
-	char *user_name = NULL;
+	char *extra = NULL, *query = NULL;
 	MYSQL_RES *result = NULL;
 	MYSQL_ROW row;
 
+	remove_common_args_t args = {
+		.mysql_conn = mysql_conn,
+		.table = federation_table,
+		.type = DBD_REMOVE_FEDERATIONS,
+	};
+
 	if (!fed_cond) {
 		error("we need something to change");
 		return NULL;
@@ -673,7 +678,7 @@ extern List as_mysql_remove_federations(mysql_conn_t *mysql_conn, uint32_t uid,
 		return NULL;
 	}
 	rc = 0;
-	ret_list = list_create(xfree_ptr);
+	args.ret_list = list_create(xfree_ptr);
 
 	if (!mysql_num_rows(result)) {
 		mysql_free_result(result);
@@ -681,38 +686,38 @@ extern List as_mysql_remove_federations(mysql_conn_t *mysql_conn, uint32_t uid,
 		DB_DEBUG(FEDR, mysql_conn->conn,
 		         "didn't affect anything\n%s", query);
 		xfree(query);
-		return ret_list;
+		return args.ret_list;
 	}
 	xfree(query);
 
-	user_name = uid_to_string((uid_t) uid);
+	args.user_name = uid_to_string((uid_t) uid);
+	args.now = time(NULL);
+
 	while ((row = mysql_fetch_row(result))) {
 		char *object = xstrdup(row[0]);
-		list_append(ret_list, object);
+		list_append(args.ret_list, object);
 
 		if ((rc = _remove_all_clusters_from_fed(mysql_conn, object,
 							NULL)))
 			break;
 
-		xfree(name_char);
-		xstrfmtcat(name_char, "name='%s'", object);
+		xfree(args.name_char);
+		xstrfmtcat(args.name_char, "name='%s'", object);
 
-		if ((rc = remove_common(mysql_conn, DBD_REMOVE_FEDERATIONS, now,
-					user_name, federation_table, name_char,
-					NULL, NULL, ret_list, NULL, NULL)))
+		if ((rc = remove_common(&args)))
 			break;
 	}
 	mysql_free_result(result);
-	xfree(user_name);
-	xfree(name_char);
+	xfree(args.user_name);
+	xfree(args.name_char);
 
 	if (rc != SLURM_SUCCESS) {
-		FREE_NULL_LIST(ret_list);
+		FREE_NULL_LIST(args.ret_list);
 		return NULL;
 	} else
 		as_mysql_add_feds_to_update_list(mysql_conn);
 
-	return ret_list;
+	return args.ret_list;
 }
 
 extern int as_mysql_add_feds_to_update_list(mysql_conn_t *mysql_conn)
