@@ -5,7 +5,7 @@
  *  Copyright (C) 2013 Bull S. A. S.
  *		Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois.
  *
- *  Portions Copyright (C) 2013 SchedMD LLC.
+ *  Copyright (C) SchedMD LLC.
  *
  *  Initially written by Rod Schultz <rod.schultz@bull.com> @ Bull
  *  and Danny Auble <da@schedmd.com> @ SchedMD.
@@ -64,6 +64,7 @@
 #include "src/common/slurm_time.h"
 #include "src/common/xstring.h"
 #include "src/interfaces/proctrack.h"
+#include "src/slurmd/common/privileges.h"
 #include "hdf5_api.h"
 
 #define HDF5_CHUNK_SIZE 10
@@ -102,14 +103,6 @@ const char plugin_name[] = "AcctGatherProfile hdf5 plugin";
 const char plugin_type[] = "acct_gather_profile/hdf5";
 const uint32_t plugin_version = SLURM_VERSION_NUMBER;
 
-struct priv_state {
-	uid_t	saved_uid;
-	gid_t	saved_gid;
-	gid_t * gid_list;
-	int	ngids;
-	char	saved_cwd [4096];
-};
-
 typedef struct {
 	char *dir;
 	uint32_t def;
@@ -142,88 +135,6 @@ static table_t *tables = NULL;
 static size_t   tables_max_len = 0;
 static size_t   tables_cur_len = 0;
 
-/* If get_list is false make sure ps->gid_list is initialized before
- * hand to prevent xfree.
- */
-static int
-_drop_privileges(stepd_step_rec_t *job, bool do_setuid,
-		 struct priv_state *ps, bool get_list)
-{
-	ps->saved_uid = getuid();
-	ps->saved_gid = getgid();
-
-	if (!getcwd (ps->saved_cwd, sizeof (ps->saved_cwd))) {
-		error ("Unable to get current working directory: %m");
-		strlcpy(ps->saved_cwd, "/tmp", sizeof(ps->saved_cwd));
-	}
-
-	ps->ngids = getgroups(0, NULL);
-	if (ps->ngids == -1) {
-		error("%s: getgroups(): %m", __func__);
-		return -1;
-	}
-	if (get_list) {
-		ps->gid_list = (gid_t *) xmalloc(ps->ngids * sizeof(gid_t));
-
-		if (getgroups(ps->ngids, ps->gid_list) == -1) {
-			error("%s: couldn't get %d groups: %m",
-			      __func__, ps->ngids);
-			xfree(ps->gid_list);
-			return -1;
-		}
-	}
-
-	/*
-	 * No need to drop privileges if we're not running as root
-	 */
-	if (getuid() != (uid_t) 0)
-		return SLURM_SUCCESS;
-
-	if (setegid(job->gid) < 0) {
-		error("setegid: %m");
-		return -1;
-	}
-
-	if (setgroups(job->ngids, job->gids) < 0) {
-		error("setgroups: %m");
-		return -1;
-	}
-
-	if (do_setuid && seteuid(job->uid) < 0) {
-		error("seteuid: %m");
-		return -1;
-	}
-
-	return SLURM_SUCCESS;
-}
-
-static int
-_reclaim_privileges(struct priv_state *ps)
-{
-	int rc = SLURM_SUCCESS;
-
-	/*
-	 * No need to reclaim privileges if our uid == job->uid
-	 */
-	if (geteuid() == ps->saved_uid)
-		goto done;
-	else if (seteuid(ps->saved_uid) < 0) {
-		error("seteuid: %m");
-		rc = -1;
-	} else if (setegid(ps->saved_gid) < 0) {
-		error("setegid: %m");
-		rc = -1;
-	} else if (setgroups(ps->ngids, ps->gid_list) < 0) {
-		error("setgroups: %m");
-		rc = -1;
-	}
-
-done:
-	xfree(ps->gid_list);
-
-	return rc;
-}
-
 static void _reset_slurm_profile_conf(void)
 {
 	xfree(hdf5_conf.dir);
@@ -270,7 +181,9 @@ static void _create_directories(void)
 	slash[0] = '\0';
 	hdf5_dir_rel = slash + 1;
 
-	parent_dirfd = open(parent_dir, O_DIRECTORY | O_NOFOLLOW);
+	if ((parent_dirfd = open(parent_dir, O_DIRECTORY | O_NOFOLLOW)) < 0)
+		fatal("Could not open ProfileHDF5Dir parent directory '%s': %m",
+		      parent_dir);
 
 	/*
 	 * Use *at family of syscalls to prevent TOCTOU abuse by working
@@ -438,7 +351,7 @@ extern int acct_gather_profile_p_node_step_start(stepd_step_rec_t* job)
 		 acct_gather_profile_to_string(g_profile_running),
 		 profile_file_name);
 
-	if (_drop_privileges(g_job, true, &sprivs, false) < 0) {
+	if (drop_privileges(g_job, true, &sprivs, false) < 0) {
 		error("%s: Unable to drop privileges", __func__);
 		xfree(profile_file_name);
 		return SLURM_ERROR;
@@ -450,7 +363,7 @@ extern int acct_gather_profile_p_node_step_start(stepd_step_rec_t* job)
 	file_id = H5Fcreate(profile_file_name, H5F_ACC_TRUNC, H5P_DEFAULT,
 			    H5P_DEFAULT);
 
-	if (_reclaim_privileges(&sprivs) < 0) {
+	if (reclaim_privileges(&sprivs) < 0) {
 		error("%s: Unable to reclaim privileges", __func__);
 		xfree(profile_file_name);
 		return SLURM_ERROR;
@@ -727,24 +640,11 @@ extern int acct_gather_profile_p_add_sample_data(int table_id, void *data,
 	return SLURM_SUCCESS;
 }
 
-extern void acct_gather_profile_p_conf_values(List *data)
+extern void acct_gather_profile_p_conf_values(list_t **data)
 {
-	config_key_pair_t *key_pair;
-
-	xassert(*data);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("ProfileHDF5Dir");
-	key_pair->value = xstrdup(hdf5_conf.dir);
-	list_append(*data, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("ProfileHDF5Default");
-	key_pair->value = xstrdup(acct_gather_profile_to_string(hdf5_conf.def));
-	list_append(*data, key_pair);
-
-	return;
-
+	add_key_pair(*data, "ProfileHDF5Dir", "%s", hdf5_conf.dir);
+	add_key_pair(*data, "ProfileHDF5Default", "%s",
+		     acct_gather_profile_to_string(hdf5_conf.def));
 }
 
 extern bool acct_gather_profile_p_is_active(uint32_t type)
