@@ -95,7 +95,7 @@ extern int data_parser_p_dump(args_t *args, data_parser_type_t type, void *src,
 		return ESLURM_NOT_SUPPORTED;
 	}
 
-	return dump(src, src_bytes, NULL, parser, dst, args);
+	return dump(src, src_bytes, parser, dst, args);
 }
 
 extern int data_parser_p_parse(args_t *args, data_parser_type_t type, void *dst,
@@ -302,3 +302,17 @@ extern const char *data_parser_p_resolve_type_string(args_t *args,
 
 	return parser->type_string;
 }
+
+extern bool data_parser_p_is_complex(args_t *args)
+{
+	xassert(args->magic == MAGIC_ARGS);
+	return is_complex_mode(args);
+}
+
+extern int data_parser_p_dump_flags(args_t *args, data_t *dst)
+{
+	xassert(args->magic == MAGIC_ARGS);
+
+	return data_parser_p_dump(args, DATA_PARSER_FLAGS, &args->flags,
+				  sizeof(args->flags), dst);
+}
