@@ -38,8 +38,6 @@
 
 #include "src/interfaces/data_parser.h"
 
-extern const char plugin_type[];
-
 /*
  * These macros are defined by the Makefile.am:
  * DATA_VERSION
@@ -47,6 +45,21 @@ extern const char plugin_type[];
  */
 
 #define MAGIC_ARGS 0x2ea1bebb
+#define is_fast_mode(args) (args->flags & FLAG_FAST)
+#define is_complex_mode(args) (args->flags & FLAG_COMPLEX_VALUES)
+#define is_prefer_refs_mode(args) (args->flags & FLAG_PREFER_REFS)
+
+typedef enum {
+	FLAG_NONE = 0,
+	/* only dump the OpenAPI Specification instead of the requested data */
+	FLAG_SPEC_ONLY = SLURM_BIT(0),
+	/* attempt to run as fast as possible, skipping more expensive checks */
+	FLAG_FAST = SLURM_BIT(1),
+	/* use null/false/Infinity/NaN for *_NO_VALs */
+	FLAG_COMPLEX_VALUES = SLURM_BIT(2),
+	/* Prefer $ref over expanding schema inline in OpenAPI specification */
+	FLAG_PREFER_REFS = SLURM_BIT(3),
+} data_parser_flags_t;
 
 typedef struct {
 	int magic; /* MAGIC_ARGS */
@@ -60,28 +73,10 @@ typedef struct {
 	void *warn_arg;
 	void *db_conn;
 	bool close_db_conn;
-	List tres_list;
-	List qos_list;
-	List assoc_list;
+	list_t *tres_list;
+	list_t *qos_list;
+	list_t *assoc_list;
+	data_parser_flags_t flags;
 } args_t;
 
-extern args_t *data_parser_p_new(data_parser_on_error_t on_parse_error,
-				 data_parser_on_error_t on_dump_error,
-				 data_parser_on_error_t on_query_error,
-				 void *error_arg,
-				 data_parser_on_warn_t on_parse_warn,
-				 data_parser_on_warn_t on_dump_warn,
-				 data_parser_on_warn_t on_query_warn,
-				 void *warn_arg, const char *params);
-extern void data_parser_p_free(args_t *args);
-
-extern int data_parser_p_assign(args_t *args, data_parser_attr_type_t type,
-				void *obj);
-
-extern int data_parser_p_dump(args_t *args, data_parser_type_t type, void *src,
-			      ssize_t src_bytes, data_t *dst);
-extern int data_parser_p_parse(args_t *args, data_parser_type_t type, void *dst,
-			       ssize_t dst_bytes, data_t *src,
-			       data_t *parent_path);
-
 #endif
