@@ -203,9 +203,7 @@ static bool _should_be_ref(const parser_t *parser, spec_args_t *sargs)
 	}
 
 	if ((parser->obj_openapi == OPENAPI_FORMAT_OBJECT) ||
-	    ((parser->obj_openapi == OPENAPI_FORMAT_ARRAY) &&
-	     !((is_inline_enums_mode(sargs->args)) &&
-	       (parser->model == PARSER_MODEL_FLAG_ARRAY))))
+	    (parser->obj_openapi == OPENAPI_FORMAT_ARRAY))
 		return true;
 
 	if (parser->array_type || parser->pointer_type || parser->list_type ||
