@@ -45,16 +45,15 @@ typedef struct parser_s parser_t;
 
 typedef enum {
 	NEED_NONE = 0, /* parser has no pre-reqs for data */
-	NEED_AUTH = SLURM_BIT(0),
 	NEED_TRES = SLURM_BIT(1),
 	NEED_QOS = SLURM_BIT(2),
-	NEED_ASSOC = SLURM_BIT(3),
 } need_t;
 
 typedef enum {
 	FLAG_BIT_TYPE_INVALID = 0, /* aka not initialized */
 	FLAG_BIT_TYPE_EQUAL, /* entire masked value must match for flag */
 	FLAG_BIT_TYPE_BIT, /* only need bit(s) to match */
+	FLAG_BIT_TYPE_REMOVED, /* flag removed but needs to still parse correct */
 	FLAG_BIT_TYPE_MAX /* place holder */
 } flag_bit_type_t;
 
@@ -70,29 +69,46 @@ typedef struct {
 	uint64_t value; /* bits set by flag */
 	char *flag_name;
 	size_t flag_size;
+	const char *description;
+	bool hidden; /* hide from OpenAPI spec generation */
+	uint16_t deprecated; /* protocol version when deprecated */
 } flag_bit_t;
 
 typedef enum {
 	PARSER_MODEL_INVALID = 0, /* aka not initialized */
 	PARSER_MODEL_ARRAY, /* parser array to parse every field in a struct */
 	PARSER_MODEL_ARRAY_LINKED_FIELD, /* link to parser in a parser array */
+	PARSER_MODEL_ARRAY_LINKED_EXPLODED_FLAG_ARRAY_FIELD, /* link to parser in a parser array of exploded flag array */
 	PARSER_MODEL_ARRAY_SKIP_FIELD, /* parser to mark field as not being parsed in a parser array */
 	PARSER_MODEL_ARRAY_REMOVED_FIELD, /* parser to mark field as placeholder for field already removed from struct */
 
-	PARSER_MODEL_SIMPLE, /* parser for single field */
+	PARSER_MODEL_SIMPLE, /* parser for single field in struct */
 	PARSER_MODEL_COMPLEX, /* parser for uses multiple fields in struct */
-	PARSER_MODEL_FLAG_ARRAY, /* parser for single bool field in a parser array */
+	PARSER_MODEL_FLAG_ARRAY, /* parser for list of flags */
 	PARSER_MODEL_LIST, /* parser for list_t's */
 	PARSER_MODEL_PTR, /* parser for pointer */
 	PARSER_MODEL_NT_PTR_ARRAY, /* parser for NULL terminated array of pointers */
 	/* NT_ARRAY objects must not require an special initializer */
 	PARSER_MODEL_NT_ARRAY, /* parser for NULL terminated array of objects */
+	PARSER_MODEL_REMOVED, /* parser for removed types */
+
+	/*
+	 * Alias for another parser.
+	 *
+	 * Only for use in maintaining the same OAS name when a new parser name
+	 * is needed in newer plugins.
+	 */
+	PARSER_MODEL_ALIAS,
 
 	PARSER_MODEL_MAX /* place holder */
 } parser_model_t;
 
 #define MAGIC_PARSER 0xa3bafa05
 
+typedef void *(*parser_new_func_t)(void);
+/* must be compatible with ListDelF */
+typedef void (*parser_free_func_t)(void *ptr);
+
 typedef struct parser_s {
 	int magic; /* MAGIC_PARSER */
 	parser_model_t model;
@@ -104,6 +120,9 @@ typedef struct parser_s {
 	char *obj_type_string; /* stringified C type */
 	openapi_type_format_t obj_openapi; /* OpenAPI format for object */
 	ssize_t size; /* size of target obj */
+	parser_new_func_t new; /* function to create new instance of obj being pointed at */
+	parser_free_func_t free; /* function to release instance of obj being pointed at */
+	uint16_t deprecated; /* protocol version when deprecated */
 
 	/* Linked model properties ------------------------------------------ */
 	char *field_name; /* name of field in struct if there is a ptr_offset */
@@ -112,8 +131,12 @@ typedef struct parser_s {
 	ssize_t ptr_offset; /* offset from parent object or NO_VAL */
 	bool required;
 
+	/* Alias model properties ----------------------------------------- */
+	type_t alias_type;
+
 	/* Pointer model properties ----------------------------------------- */
 	type_t pointer_type;
+	bool allow_null_pointer; /* leave destination as null type when source pointer is NULL while dumping */
 
 	/* NULL terminated array of pointers model properties --------------- */
 	type_t array_type;
@@ -121,6 +144,7 @@ typedef struct parser_s {
 	/* Flag array model properties -------------------------------------- */
 	const flag_bit_t *flag_bit_array;
 	uint8_t flag_bit_array_count; /* number of entries in flag_bit_array */
+	bool single_flag; /* false to be list or true to be string for 1 flag */
 
 	/* List model properties -------------------------------------------- */
 	type_t list_type;
@@ -135,20 +159,6 @@ typedef struct parser_s {
 	int (*parse)(const parser_t *const parser, void *dst, data_t *src,
 		     args_t *args, data_t *parent_path);
 	need_t needs;
-	/*
-	 * Populates OpenAPI specification.
-	 * 	For parsers where the normal OpenAPI specification generation is
-	 * 	insufficient. This allows the parser to explicitly set the
-	 * 	specification for the type. General goal is to not to need to
-	 * 	use this function but some output formats are just too different
-	 * 	the original data source.
-	 * IN parser - parser needing specification
-	 * IN args - parser args
-	 * IN spec - ptr to entire OpenAPI specification
-	 * IN dst - entry in specificaiton needed to be populated
-	 */
-	void (*openapi_spec)(const parser_t *const parser, args_t *args,
-			     data_t *spec, data_t *dst);
 } parser_t;
 
 /*
@@ -192,6 +202,11 @@ extern void verify_parser_not_sliced_funcname(const parser_t *const parser,
 
 extern const parser_t *const find_parser_by_type(type_t type);
 
+/*
+ * Resolve aliased or pointer model parsers to final unaliased parser
+ */
+extern const parser_t *unalias_parser(const parser_t *parser);
+
 extern void get_parsers(const parser_t **parsers_ptr, int *count_ptr);
 
 #endif
