@@ -2,7 +2,7 @@
  *  gres_mps.c - Support MPS as a generic resources.
  *  MPS or CUDA Multi-Process Services is a mechanism to share GPUs.
  *****************************************************************************
- *  Copyright (C) 2018 SchedMD LLC
+ *  Copyright (C) SchedMD LLC.
  *  Written by Morris Jette
  *
  *  This file is part of Slurm, a resource management program.
@@ -39,7 +39,6 @@
 
 #include <ctype.h>
 #include <inttypes.h>
-#include <sched.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -91,7 +90,7 @@ const char plugin_name[] = "Gres MPS plugin";
 const char	plugin_type[]		= "gres/mps";
 const uint32_t	plugin_version		= SLURM_VERSION_NUMBER;
 
-static List	gres_devices		= NULL;
+static list_t *gres_devices = NULL;
 
 extern int init(void)
 {
@@ -113,7 +112,7 @@ extern int fini(void)
  * This only validates that the configuration was specified in gres.conf.
  * In the general case, no code would need to be changed.
  */
-extern int gres_p_node_config_load(List gres_conf_list,
+extern int gres_p_node_config_load(list_t *gres_conf_list,
 				   node_config_load_t *config)
 {
 	return gres_c_s_init_share_devices(
@@ -123,7 +122,7 @@ extern int gres_p_node_config_load(List gres_conf_list,
 /* Given a global device ID, return its gres/mps count */
 static uint64_t _get_dev_count(int global_id)
 {
-	ListIterator itr;
+	list_itr_t *itr;
 	shared_dev_info_t *mps_ptr;
 	uint64_t count = NO_VAL64;
 
@@ -235,15 +234,15 @@ extern void gres_p_step_set_env(char ***step_env_ptr,
  */
 extern void gres_p_task_set_env(char ***task_env_ptr,
 				bitstr_t *gres_bit_alloc,
+				uint64_t gres_cnt,
 				bitstr_t *usable_gres,
-				uint64_t gres_per_node,
 				gres_internal_flags_t flags)
 {
 	common_gres_env_t gres_env = {
 		.bit_alloc = gres_bit_alloc,
 		.env_ptr = task_env_ptr,
 		.flags = flags,
-		.gres_cnt = gres_per_node,
+		.gres_cnt = gres_cnt,
 		.is_task = true,
 		.usable_gres = usable_gres,
 	};
@@ -271,46 +270,11 @@ extern void gres_p_recv_stepd(buf_t *buffer)
 	return;
 }
 
-/*
- * get data from a job's GRES data structure
- * IN job_gres_data  - job's GRES data structure
- * IN node_inx - zero-origin index of the node within the job's allocation
- *	for which data is desired
- * IN data_type - type of data to get from the job's data
- * OUT data - pointer to the data from job's GRES data structure
- *            DO NOT FREE: This is a pointer into the job's data structure
- * RET - SLURM_SUCCESS or error code
- */
-extern int gres_p_get_job_info(gres_job_state_t *gres_js,
-			       uint32_t node_inx,
-			       enum gres_job_data_type data_type, void *data)
-{
-	return EINVAL;
-}
-
-/*
- * get data from a step's GRES data structure
- * IN gres_ss  - step's GRES data structure
- * IN node_inx - zero-origin index of the node within the job's allocation
- *	for which data is desired. Note this can differ from the step's
- *	node allocation index.
- * IN data_type - type of data to get from the step's data
- * OUT data - pointer to the data from step's GRES data structure
- *            DO NOT FREE: This is a pointer into the step's data structure
- * RET - SLURM_SUCCESS or error code
- */
-extern int gres_p_get_step_info(gres_step_state_t *gres_ss,
-				uint32_t node_inx,
-				enum gres_step_data_type data_type, void *data)
-{
-	return EINVAL;
-}
-
 /*
  * Return a list of devices of this type. The list elements are of type
  * "gres_device_t" and the list should be freed using FREE_NULL_LIST().
  */
-extern List gres_p_get_devices(void)
+extern list_t *gres_p_get_devices(void)
 {
 	return gres_devices;
 }
@@ -367,7 +331,7 @@ extern void gres_p_prep_set_env(char ***prep_env_ptr,
 	int dev_inx = -1, global_id = -1, i;
 	uint64_t count_on_dev, gres_per_node = 0, percentage;
 	gres_device_t *gres_device;
-	ListIterator iter;
+	list_itr_t *iter;
 
 	if (gres_common_prep_set_env(prep_env_ptr,
 				     gres_prep, node_inx,
