@@ -117,18 +117,6 @@ typedef struct {
 	uint32_t user_id;
 } foreach_part_list_args_t;
 
-/* These are defined here so when we link with something other than
- * the slurmctld we will have these symbols defined.  They will get
- * overwritten when linking with the slurmctld.
- */
-#if defined (__APPLE__)
-extern uint16_t accounting_enforce __attribute__((weak_import));
-extern void *acct_db_conn  __attribute__((weak_import));
-#else
-uint16_t accounting_enforce = 0;
-void *acct_db_conn = NULL;
-#endif
-
 /*****************************************************************************\
  * We've provided a simple example of the type of things you can do with this
  * plugin. If you develop another plugin that may be of interest to others
@@ -143,13 +131,31 @@ static char *_get_default_account(uint32_t user_id)
 	memset(&user, 0, sizeof(slurmdb_user_rec_t));
 	user.uid = user_id;
 	if (assoc_mgr_fill_in_user(acct_db_conn, &user, accounting_enforce,
-				   NULL, false) != SLURM_ERROR) {
+				   NULL, false) == SLURM_SUCCESS) {
 		return user.default_acct;
 	} else {
 		return NULL;
 	}
 }
 
+static int _fill_assoc(uint32_t user_id, char *account, char *partition,
+		       slurmdb_assoc_rec_t *assoc)
+{
+	xassert(assoc);
+
+	memset(assoc, 0, sizeof(slurmdb_assoc_rec_t));
+	assoc->uid = user_id;
+	assoc->partition = partition;
+	if (account) {
+		assoc->acct = account;
+	} else {
+		assoc->acct = _get_default_account(user_id);
+	}
+
+	return assoc_mgr_fill_in_assoc(acct_db_conn, assoc, accounting_enforce,
+				       NULL, false);
+}
+
 /* Get the default QOS for an association (or NULL if not present) */
 static char *_get_default_qos(uint32_t user_id, char *account, char *partition)
 {
@@ -157,17 +163,7 @@ static char *_get_default_qos(uint32_t user_id, char *account, char *partition)
 	slurmdb_qos_rec_t qos;
 	uint32_t qos_id = 0;
 
-	memset(&assoc, 0, sizeof(slurmdb_assoc_rec_t));
-	assoc.uid = user_id;
-	assoc.partition = partition;
-	if (account) {
-		assoc.acct = account;
-	} else {
-		assoc.acct = _get_default_account(user_id);
-	}
-
-	if (assoc_mgr_fill_in_assoc(acct_db_conn, &assoc, accounting_enforce,
-				    NULL, false) != SLURM_ERROR)
+	if (_fill_assoc(user_id, account, partition, &assoc) == SLURM_SUCCESS)
 		qos_id = assoc.def_qos_id;
 
 	if (!qos_id)
@@ -175,31 +171,65 @@ static char *_get_default_qos(uint32_t user_id, char *account, char *partition)
 
 	memset(&qos, 0, sizeof(slurmdb_qos_rec_t));
 	qos.id = qos_id;
-	if (assoc_mgr_fill_in_qos(acct_db_conn, &qos, accounting_enforce,
-				  NULL, false) != SLURM_ERROR) {
+	if (assoc_mgr_fill_in_qos(acct_db_conn, &qos, accounting_enforce, NULL,
+				  false) == SLURM_SUCCESS) {
 		return qos.name;
 	} else {
 		return NULL;
 	}
 }
 
+static int _qos_id_to_qos_name(void *x, void *arg)
+{
+	char *qos_id = x;
+	list_t *qos_name_list = arg;
+	slurmdb_qos_rec_t qos = { 0 };
+
+	qos.id = atoi(qos_id);
+
+	if (assoc_mgr_fill_in_qos(acct_db_conn, &qos, accounting_enforce, NULL,
+				  false) != SLURM_SUCCESS) {
+		return 0;
+	}
+
+	slurm_addto_char_list(qos_name_list, qos.name);
+
+	return 0;
+}
+
+/* Get all possible QOS for an association (or NULL if not present) */
+static char *_get_assoc_qos(uint32_t user_id, char *account, char *partition)
+{
+	slurmdb_assoc_rec_t assoc;
+	list_t *qos_name_list;
+	char *qos_name_list_str;
+	list_t *qos_list = NULL;
+
+	if (_fill_assoc(user_id, account, partition, &assoc) == SLURM_SUCCESS)
+		qos_list = assoc.qos_list;
+
+	if (!qos_list)
+		return NULL;
+
+	qos_name_list = list_create(xfree_ptr);
+	list_for_each_ro(qos_list, _qos_id_to_qos_name, qos_name_list);
+
+	qos_name_list_str = slurm_char_list_to_xstr(qos_name_list);
+
+	list_destroy(qos_name_list);
+
+	return qos_name_list_str;
+}
+
 /* Get the comment for an association (or NULL if not present) */
-static char *_get_assoc_comment(uint32_t user_id, char *account)
+static char *_get_assoc_comment(uint32_t user_id, char *account,
+				char *partition)
 {
 	slurmdb_assoc_rec_t assoc;
 	char *comment = NULL;
 
-	memset(&assoc, 0, sizeof(slurmdb_assoc_rec_t));
-	assoc.uid = user_id;
-	if (account) {
-		assoc.acct = account;
-	} else {
-		assoc.acct = _get_default_account(user_id);
-	}
-
-	if (assoc_mgr_fill_in_assoc(acct_db_conn, &assoc, accounting_enforce,
-				    NULL, false) != SLURM_ERROR)
-		comment = xstrdup(assoc.comment);
+	if (_fill_assoc(user_id, account, partition, &assoc) == SLURM_SUCCESS)
+		comment = assoc.comment;
 
 	return comment;
 }
@@ -525,7 +555,14 @@ static int _get_job_req_field(const job_desc_msg_t *job_desc, const char *name)
 		lua_pushstring(L, job_desc->array_inx);
 	} else if (!xstrcmp(name, "assoc_comment")) {
 		lua_pushstring(L, _get_assoc_comment(job_desc->user_id,
-						     job_desc->account));
+						     job_desc->account,
+						     job_desc->partition));
+	} else if (!xstrcmp(name, "assoc_qos")) {
+		char *assoc_qos_str = _get_assoc_qos(job_desc->user_id,
+						     job_desc->account,
+						     job_desc->partition);
+		lua_pushstring(L, assoc_qos_str);
+		xfree(assoc_qos_str);
 	} else if (!xstrcmp(name, "batch_features")) {
 		lua_pushstring(L, job_desc->batch_features);
 	} else if (!xstrcmp(name, "begin_time")) {
@@ -1363,7 +1400,7 @@ int init(void)
 	return slurm_lua_loadscript(&L, "job_submit/lua",
 				    lua_script_path, req_fxns,
 				    &lua_script_last_loaded,
-				    _loadscript_extra);
+				    _loadscript_extra, NULL);
 }
 
 int fini(void)
@@ -1391,7 +1428,8 @@ extern int job_submit(job_desc_msg_t *job_desc, uint32_t submit_uid,
 
 	rc = slurm_lua_loadscript(&L, "job_submit/lua",
 				  lua_script_path, req_fxns,
-				  &lua_script_last_loaded, _loadscript_extra);
+				  &lua_script_last_loaded, _loadscript_extra,
+				  NULL);
 
 	if (rc != SLURM_SUCCESS)
 		goto out;
@@ -1445,9 +1483,10 @@ extern int job_modify(job_desc_msg_t *job_desc, job_record_t *job_ptr,
 
 	rc = slurm_lua_loadscript(&L, "job_submit/lua",
 				  lua_script_path, req_fxns,
-				  &lua_script_last_loaded, _loadscript_extra);
+				  &lua_script_last_loaded, _loadscript_extra,
+				  NULL);
 
-	if (rc == SLURM_ERROR)
+	if (rc != SLURM_SUCCESS)
 		goto out;
 
 	/*
