@@ -1,9 +1,7 @@
 /*****************************************************************************\
  *  common_jag.c - slurm job accounting gather common plugin functions.
  *****************************************************************************
- *  Copyright (C) 2013 SchedMD LLC
- *  Written by Danny Auble <da@schedmd.com>, who borrowed heavily
- *  from the original code in jobacct_gather/linux
+ *  Copyright (C) SchedMD LLC.
  *
  *  This file is part of Slurm, a resource management program.
  *  For details, see <https://slurm.schedmd.com/>.
@@ -74,7 +72,7 @@ char **assoc_mgr_tres_name_array;
 
 static int cpunfo_frequency = 0;
 static long conv_units = 0;
-List prec_list = NULL;
+list_t *prec_list = NULL;
 
 static int my_pagesize = 0;
 static int energy_profile = ENERGY_DATA_NODE_ENERGY_UP;
@@ -378,7 +376,7 @@ static int _get_process_data_line(int in, jag_prec_t *prec) {
 
 	/*
 	 * Store unnormalized times, we will normalize in when
-	 * transfering to a struct jobacctinfo in job_common_poll_data()
+	 * transferring to a struct jobacctinfo in job_common_poll_data()
 	 */
 	prec->usec = (double)utime;
 	prec->ssec = (double)stime;
@@ -523,13 +521,19 @@ static void _handle_stats(pid_t pid, jag_callbacks_t *callbacks, int tres_count)
 {
 	static int no_share_data = -1;
 	static int use_pss = -1;
+	static int disable_gpu_acct = -1;
 	char *proc_file = NULL;
 	FILE *stat_fp = NULL;
 	FILE *io_fp = NULL;
 	int fd, fd2;
 	jag_prec_t *prec = NULL;
 
-	if (no_share_data == -1) {
+	/* UsePSS and NoShare are only compatible with the linux plugin. */
+	if ((no_share_data == -1) &&
+	    (!xstrcasestr(slurm_conf.job_acct_gather_type, "linux"))) {
+		use_pss = 0;
+		no_share_data = 0;
+	} else if (no_share_data == -1) {
 		if (xstrcasestr(slurm_conf.job_acct_gather_params, "NoShare"))
 			no_share_data = 1;
 		else
@@ -541,6 +545,15 @@ static void _handle_stats(pid_t pid, jag_callbacks_t *callbacks, int tres_count)
 			use_pss = 0;
 	}
 
+	if (disable_gpu_acct == -1) {
+		if (xstrcasestr(slurm_conf.job_acct_gather_params,
+				"DisableGPUAcct")) {
+			disable_gpu_acct = 1;
+			log_flag(JAG, "GPU accounting disabled as JobAcctGatherParams=DisableGpuAcct is set.");
+		} else
+			disable_gpu_acct = 0;
+	}
+
 	xstrfmtcat(proc_file, "/proc/%u/stat", pid);
 	if (!(stat_fp = fopen(proc_file, "r")))
 		return;  /* Assume the process went away */
@@ -582,7 +595,8 @@ static void _handle_stats(pid_t pid, jag_callbacks_t *callbacks, int tres_count)
 
 	fclose(stat_fp);
 
-	gpu_g_usage_read(pid, prec->tres_data);
+	if (!disable_gpu_acct)
+		gpu_g_usage_read(pid, prec->tres_data);
 
 	/* Remove shared data from rss */
 	if (no_share_data) {
@@ -624,8 +638,17 @@ bail_out:
 	return;
 }
 
-static List _get_precs(List task_list, uint64_t cont_id,
-		       jag_callbacks_t *callbacks)
+static int _mark_as_completed(void *x, void *empty)
+{
+	jag_prec_t *prec = (jag_prec_t *) x;
+
+	prec->completed = true;
+
+	return SLURM_SUCCESS;
+}
+
+static list_t *_get_precs(list_t *task_list, uint64_t cont_id,
+			  jag_callbacks_t *callbacks)
 {
 	int npids = 0;
 	struct jobacctinfo *jobacct = NULL;
@@ -635,6 +658,16 @@ static List _get_precs(List task_list, uint64_t cont_id,
 
 	jobacct = list_peek(task_list);
 
+	/*
+	 * Mark all the processes as completed as if they were terminated,
+	 * even if they might still be alive. If that is the case, the next call
+	 * to _handle_stats will reset this flag for each pid which is found to
+	 * be alive. Otherwise the pid statistics will be aggregated into its
+	 * ancestor and the prec be removed from the list in order to avoid
+	 * aggregating it on each iteration.
+	 */
+	list_for_each(prec_list, _mark_as_completed, NULL);
+
 	/* get only the processes in the proctrack container */
 	proctrack_g_get_pids(cont_id, &pids, &npids);
 	if (npids) {
@@ -956,18 +989,21 @@ static void _aggregate_prec(jag_prec_t *prec, jag_prec_t *ancestor)
  *			tree.
  *	pid		The process for which we are currently looking
  *			for offspring.
- *
- * OUT:	none.
+ * IN/OUT:
+ *      permanent_anc Pointer to the original ancestor. Changes to
+ *	              it are saved, so we can permanently save
+ *		      the values from completed processes.
  *
  * RETVAL:	none.
  *
  * THREADSAFE! Only one thread ever gets here.
  */
-static void _get_offspring_data(List prec_list, jag_prec_t *ancestor, pid_t pid)
+static void _get_offspring_data(list_t *prec_list, jag_prec_t *ancestor,
+				pid_t pid, jag_prec_t *permanent_anc)
 {
 	jag_prec_t *prec = NULL;
 	jag_prec_t *prec_tmp = NULL;
-	List tmp_list = NULL;
+	list_t *tmp_list = NULL;
 
 	/* reset all precs to be not visited */
 	(void)list_for_each(prec_list, (ListForF)_reset_visited, NULL);
@@ -986,6 +1022,18 @@ static void _get_offspring_data(List prec_list, jag_prec_t *ancestor, pid_t pid)
 					      _list_find_prec_by_ppid,
 					       &(prec_tmp->pid)))) {
 			_aggregate_prec(prec, ancestor);
+			/*
+			 * If the prec disappeared (pid is dead) aggregate its
+			 * statistics and remove it from the prec_list to avoid
+			 * having to agreggate it on every iteration.
+			 */
+			if (prec->completed) {
+				_aggregate_prec(prec, permanent_anc);
+				log_flag(JAG, "Removing completed process %d",
+					 prec->pid);
+				list_remove_first(prec_list, _find_prec,
+						  &prec->pid);
+			}
 			list_append(tmp_list, prec);
 		}
 	}
@@ -994,13 +1042,13 @@ static void _get_offspring_data(List prec_list, jag_prec_t *ancestor, pid_t pid)
 	return;
 }
 
-extern void jag_common_poll_data(List task_list, uint64_t cont_id,
+extern void jag_common_poll_data(list_t *task_list, uint64_t cont_id,
 				 jag_callbacks_t *callbacks, bool profile)
 {
 	/* Update the data */
 	uint64_t total_job_mem = 0, total_job_vsize = 0;
 	uint32_t last_taskid = NO_VAL;
-	ListIterator itr;
+	list_itr_t *itr;
 	jag_prec_t *prec = NULL, tmp_prec;
 	struct jobacctinfo *jobacct = NULL;
 	static int processing = 0;
@@ -1040,70 +1088,89 @@ extern void jag_common_poll_data(List task_list, uint64_t cont_id,
 	while ((jobacct = list_next(itr))) {
 		double cpu_calc;
 		double last_total_cputime;
-		if (!(prec = list_find_first(prec_list, _find_prec,
-					     &jobacct->pid)))
-			continue;
-		/*
-		 * We can't use the prec from the list as we need to keep it in
-		 * the original state without offspring since we reuse this list
-		 * keeping around precs after they end.
-		 */
-		memcpy(&tmp_prec, prec, sizeof(*prec));
-		prec = &tmp_prec;
-
-		if (acct_gather_filesystem_g_get_data(prec->tres_data) < 0) {
-			log_flag(JAG, "problem retrieving filesystem data");
-		}
-
-		if (acct_gather_interconnect_g_get_data(prec->tres_data) < 0) {
-			log_flag(JAG, "problem retrieving interconnect data");
-		}
-		/* find all my descendents */
-		if (callbacks->get_offspring_data)
-			(*(callbacks->get_offspring_data))
-				(prec_list, prec, prec->pid);
-
-		/*
-		 * Only jobacct_gather/cgroup uses prec_extra, and we want to
-		 * make sure we call it once per task, so call it here as we
-		 * iterate through the tasks instead of in get_precs.
-		 */
-		if (callbacks->prec_extra) {
-			if (last_taskid == jobacct->id.taskid) {
-				log_flag(JAG, "skipping prec_extra() call against nodeid:%u taskid:%u",
-					 jobacct->id.nodeid,
-					 jobacct->id.taskid);
+		jag_prec_t *permanent_anc;
+		if (jobacct->pid) {
+			if (!(prec = list_find_first(prec_list, _find_prec,
+						     &jobacct->pid)))
 				continue;
-			} else {
-				log_flag(JAG, "calling prec_extra() call against nodeid:%u taskid:%u",
-					 jobacct->id.nodeid,
-					 jobacct->id.taskid);
+			/*
+			 * We can't use the prec from the list as we need to
+			 * keep it in the original state without offspring since
+			 * we reuse this list keeping around precs after they
+			 * end.
+			 */
+			memcpy(&tmp_prec, prec, sizeof(*prec));
+			permanent_anc = prec;
+			prec = &tmp_prec;
+
+			if (acct_gather_filesystem_g_get_data(prec->tres_data) <
+			    0) {
+				log_flag(JAG, "problem retrieving filesystem data");
 			}
 
-			last_taskid = jobacct->id.taskid;
-			(*(callbacks->prec_extra))(prec, jobacct->id.taskid);
+			if (acct_gather_interconnect_g_get_data(
+				    prec->tres_data) < 0) {
+				log_flag(JAG, "problem retrieving interconnect data");
+			}
+			/* find all my descendents */
+			if (callbacks->get_offspring_data)
+				(*(callbacks->get_offspring_data))(
+					prec_list, prec, prec->pid,
+					permanent_anc);
+
+			/*
+			 * Only jobacct_gather/cgroup uses prec_extra, and we
+			 * want to make sure we call it once per task, so call
+			 * it here as we iterate through the tasks instead of
+			 * in get_precs.
+			 */
+			if (callbacks->prec_extra) {
+				if (last_taskid == jobacct->id.taskid) {
+					log_flag(JAG, "skipping prec_extra() call against nodeid:%u taskid:%u",
+						 jobacct->id.nodeid,
+						 jobacct->id.taskid);
+					continue;
+				} else {
+					log_flag(JAG, "calling prec_extra() call against nodeid:%u taskid:%u",
+						 jobacct->id.nodeid,
+						 jobacct->id.taskid);
+				}
+
+				last_taskid = jobacct->id.taskid;
+				(*(callbacks->prec_extra))(prec,
+							   jobacct->id.taskid);
+			}
+
+			log_flag(JAG, "pid:%u ppid:%u %s:%" PRIu64 " B",
+				 prec->pid, prec->ppid,
+				 (xstrcasestr(slurm_conf.job_acct_gather_params,
+					      "UsePss") ?  "pss" : "rss"),
+				 prec->tres_data[TRES_ARRAY_MEM].size_read);
+
+			last_total_cputime =
+				(double) jobacct->
+				tres_usage_in_tot[TRES_ARRAY_CPU];
+
+			cpu_calc =
+				(prec->ssec + prec->usec) / (double) conv_units;
+
+			/*
+			 * Since we are not storing things as a double anymor
+			 * make it bigger so we don't loose precision.
+			 */
+			cpu_calc *= CPU_TIME_ADJ;
+
+			prec->tres_data[TRES_ARRAY_CPU].size_read =
+				(uint64_t) cpu_calc;
+		} else {
+			prec = xmalloc(sizeof(jag_prec_t));
+			prec->tres_count = jobacct->tres_count;
+			prec->tres_data = xcalloc(prec->tres_count,
+						  sizeof(acct_gather_data_t));
+			_init_tres(prec, NULL);
+			cpu_calc = last_total_cputime = 0;
 		}
 
-		log_flag(JAG, "pid:%u ppid:%u %s:%" PRIu64 " B",
-			 prec->pid, prec->ppid,
-			 (xstrcasestr(slurm_conf.job_acct_gather_params,
-				      "UsePss") ?  "pss" : "rss"),
-			 prec->tres_data[TRES_ARRAY_MEM].size_read);
-
-
-		last_total_cputime =
-			(double)jobacct->tres_usage_in_tot[TRES_ARRAY_CPU];
-
-		cpu_calc = (prec->ssec + prec->usec) / (double) conv_units;
-
-		/*
-		 * Since we are not storing things as a double anymore make it
-		 * bigger so we don't loose precision.
-		 */
-		cpu_calc *= CPU_TIME_ADJ;
-
-		prec->tres_data[TRES_ARRAY_CPU].size_read = (uint64_t)cpu_calc;
-
 		/* get energy consumption
 		 * only once is enough since we
 		 * report per node energy consumption.
@@ -1131,6 +1198,32 @@ extern void jag_common_poll_data(List task_list, uint64_t cont_id,
 		for (i = 0; i < jobacct->tres_count; i++) {
 			if (prec->tres_data[i].size_read == INFINITE64)
 				continue;
+
+			/* Do this before the max for polling/profiling */
+			jobacct->tres_usage_in_tot[i] =
+				prec->tres_data[i].size_read;
+
+			/*
+			 * Check for support of MaxRSS direct reading.
+			 *
+			 * In cgroup/v1 and cgroup/v2 we can get the value
+			 * directly in the memory.peak or
+			 * memory.max_usage_in_bytes interfaces.
+			 *
+			 * If available it will be in size_write.
+			 *
+			 * If it is not available then the MaxRSS will be the
+			 * one gathered and calculated through memory.current,
+			 * memory.usage_in_bytes or linux /proc.
+			 */
+			if ((i == TRES_ARRAY_MEM) &&
+			    (prec->tres_data[i].size_write != INFINITE64)) {
+				prec->tres_data[i].size_read =
+					prec->tres_data[i].size_write;
+				prec->tres_data[i].size_write =
+					INFINITE64;
+			}
+
 			if (jobacct->tres_usage_in_max[i] == INFINITE64)
 				jobacct->tres_usage_in_max[i] =
 					prec->tres_data[i].size_read;
@@ -1149,8 +1242,6 @@ extern void jag_common_poll_data(List task_list, uint64_t cont_id,
 			 */
 			jobacct->tres_usage_in_min[i] =
 				jobacct->tres_usage_in_max[i];
-			jobacct->tres_usage_in_tot[i] =
-				prec->tres_data[i].size_read;
 
 			if (jobacct->tres_usage_out_max[i] == INFINITE64)
 				jobacct->tres_usage_out_max[i] =
@@ -1220,6 +1311,8 @@ extern void jag_common_poll_data(List task_list, uint64_t cont_id,
 
 			jobacct->last_time = jobacct->cur_time;
 		}
+		if (!jobacct->pid)
+			destroy_jag_prec(prec);
 	}
 	list_iterator_destroy(itr);
 
