@@ -43,11 +43,12 @@
 #define _GNU_SOURCE
 
 #include <pthread.h>
-#include <sched.h>
 #include <poll.h>
 #include <arpa/inet.h>
 #include <time.h>
 
+#include "src/interfaces/conn.h"
+
 #include "pmixp_common.h"
 #include "pmixp_server.h"
 #include "pmixp_client.h"
@@ -173,8 +174,8 @@ static int _abort_conn_close(eio_obj_t *obj, list_t *objs)
 
 static int _abort_conn_read(eio_obj_t *obj, list_t *objs)
 {
+	void *tls_conn = NULL;
 	slurm_addr_t abort_client;
-	int client_fd;
 	int shutdown = 0;
 
 	while (1) {
@@ -190,14 +191,14 @@ static int _abort_conn_read(eio_obj_t *obj, list_t *objs)
 			return SLURM_SUCCESS;
 		}
 
-		client_fd = slurm_accept_msg_conn(obj->fd, &abort_client);
-		if (client_fd < 0) {
-			PMIXP_ERROR("slurm_accept_msg_conn: %m");
+		if (!(tls_conn =
+			      slurm_accept_msg_conn(obj->fd, &abort_client))) {
+			PMIXP_ERROR("slurm_accept_conn: %m");
 			continue;
 		}
 		PMIXP_DEBUG("New abort client: %pA", &abort_client);
-		pmixp_abort_handle(client_fd);
-		close(client_fd);
+		pmixp_abort_handle(tls_conn);
+		conn_g_destroy(tls_conn, true);
 	}
 	return SLURM_SUCCESS;
 }
@@ -211,7 +212,7 @@ static int _timer_conn_read(eio_obj_t *obj, list_t *objs)
 	while (32 == pmixp_read_buf(obj->fd, tmpbuf, 32, &shutdown, false))
 		;
 	if (shutdown) {
-		PMIXP_ERROR("readin from timer fd, shouldn't happen");
+		PMIXP_ERROR("read-in from timer fd, shouldn't happen");
 		obj->shutdown = true;
 	}
 
