@@ -124,7 +124,7 @@ static bool _set_max_node_gres(sock_gres_t *sock_gres, uint64_t val)
  * OUT near_gpus - Count of GPUs available on sockets with available CPUs
  * RET - 0 if job can use this node, -1 otherwise (some GRES limit prevents use)
  */
-extern int gres_select_filter_remove_unusable(List sock_gres_list,
+extern int gres_select_filter_remove_unusable(list_t *sock_gres_list,
 					      uint64_t avail_mem,
 					      uint16_t max_cpus,
 					      bool enforce_binding,
@@ -312,6 +312,10 @@ static void _init_gres_per_bit_select(gres_job_state_t *gres_js, int node_inx)
 		gres_js->gres_per_bit_select = xcalloc(gres_js->total_node_cnt,
 						       sizeof(bitstr_t *));
 	}
+
+	if (gres_js->gres_per_bit_select[node_inx])
+		xfree(gres_js->gres_per_bit_select[node_inx]);
+
 	gres_js->gres_per_bit_select[node_inx] = xcalloc(
 		bit_size(gres_js->gres_bit_select[node_inx]), sizeof(uint64_t));
 }
@@ -352,8 +356,9 @@ static void _pick_shared_gres_topo(sock_gres_t *sock_gres, bool use_busy_dev,
 		if (use_busy_dev && (gres_ns->topo_gres_cnt_alloc[t] == 0))
 			continue;
 		cnt_avail = gres_ns->topo_gres_cnt_avail[t] -
-			gres_ns->topo_gres_cnt_alloc[t] -
 			gres_js->gres_per_bit_select[node_inx][t];
+		if (!sock_gres->use_total_gres) /* Subtract other jobs */
+			cnt_avail -= gres_ns->topo_gres_cnt_alloc[t];
 		if  (cnt_avail < (use_single_dev ? *gres_needed : 1))
 			continue; /* Insufficient resources */
 		if (!bit_test(sock_bits, t))
@@ -431,8 +436,8 @@ static void _pick_shared_gres(uint64_t *gres_needed, uint32_t *used_sock,
 
 	if (use_single_dev && total_res_gres && (sock_with_res_cnt > 1)) {
 		/*
-		 * Have to allocate gres accross more then one socket.
-		 * This is assuming one socket per gres configueration line.
+		 * Have to allocate gres across more than one socket.
+		 * This is assuming one socket per gres configuration line.
 		 */
 		*satisfy_res_gres = false;
 		return;
@@ -660,8 +665,9 @@ static int _set_shared_task_bits(int node_inx,
 					      job_id, node_inx);
 					rc = ESLURM_INVALID_GRES;
 				} else if (gres_needed) {
-					error("Not enough shared gres available to satisfy gres per task request for job %u on node %d",
-					      job_id, node_inx);
+					error("Not enough shared gres available to satisfy gres per task request for job %u on node %d (%"PRIu64"/%"PRIu64" still needed)",
+					      job_id, node_inx, gres_needed,
+					      gres_js->gres_per_task);
 					rc = ESLURM_INVALID_GRES;
 					break;
 				}
@@ -725,9 +731,11 @@ static uint64_t _pick_gres_topo(sock_gres_t *sock_gres, int gres_needed,
 		int g = sorted_gres ? sorted_gres[i] : i;
 		if (!bit_test(sock_bits, g))
 			continue; /* GRES not on this socket */
-		if (bit_test(gres_js->gres_bit_select[node_inx], g) ||
+		if (bit_test(gres_js->gres_bit_select[node_inx], g))
+			continue; /* Already selected for this job */
+		if (!sock_gres->use_total_gres &&
 		    bit_test(gres_ns->gres_bit_alloc, g))
-			continue; /* Already allocated GRES */
+			continue; /* Already allocated to other jobs */
 		bit_set(gres_js->gres_bit_select[node_inx], g);
 		gres_js->gres_cnt_node_select[node_inx]++;
 		gres_still_needed--;
@@ -927,7 +935,7 @@ static int _set_job_bits1(int node_inx, int job_node_inx, int rem_nodes,
 			  sock_gres_t *sock_gres, uint32_t job_id,
 			  gres_mc_data_t *tres_mc_ptr, uint16_t cpus_per_core,
 			  uint32_t *cores_on_sock, uint32_t total_cores,
-			  uint32_t total_res_gres)
+			  uint32_t total_res_gres, bool enforce_binding)
 {
 	int gres_cnt;
 	uint16_t sock_cnt = 0;
@@ -938,6 +946,7 @@ static int _set_job_bits1(int node_inx, int job_node_inx, int rem_nodes,
 	int max_gres, pick_gres;
 	int fini = 0;
 	uint16_t cpus_per_gres = 0;
+	float gres_needed_per_core;
 
 	gres_js = sock_gres->gres_state_job->gres_data;
 	gres_ns = sock_gres->gres_state_node->gres_data;
@@ -958,6 +967,9 @@ static int _set_job_bits1(int node_inx, int job_node_inx, int rem_nodes,
 			tres_mc_ptr->cpus_per_task;
 	}
 	if (cpus_per_gres) {
+		if (tres_mc_ptr->threads_per_core)
+			cpus_per_core = MIN(cpus_per_core,
+					    tres_mc_ptr->threads_per_core);
 		max_gres = MIN(max_gres,
 			       ((total_cores * cpus_per_core) /
 				cpus_per_gres));
@@ -977,23 +989,28 @@ static int _set_job_bits1(int node_inx, int job_node_inx, int rem_nodes,
 		 * least one gpu on the node anyway.
 		 * For example --gpus=typeA:2,typeB:1 where there is only one
 		 * typeA on each node then two nodes are required. Because this
-		 * is not a heterogenous job a typeB gpu does have to be
+		 * is not a heterogeneous job a typeB gpu does have to be
 		 * allocated on each node even though they only requested one
 		 * for the job.
 		 */
 		pick_gres = MAX(max_gres, 1);
 	}
+
+	gres_needed_per_core = (float)pick_gres / (float)total_cores;
+
 	/*
 	 * Now pick specific GRES for these sockets.
-	 * First select all GRES that we might possibly use, starting with
-	 * those not constrained by socket, then contrained by socket.
+	 * First select all GRES that we might possibly use
 	 * Then remove those which are not required and not "best".
 	 */
 	for (s = 0; ((s < sock_cnt) && (alloc_gres_cnt < pick_gres)); s++) {
 		if (!cores_on_sock[s])
 			continue;
+		int sock_gres_needed = MIN(pick_gres - alloc_gres_cnt,
+					   (int)(cores_on_sock[s] *
+						 gres_needed_per_core));
 		alloc_gres_cnt += _pick_gres_topo(sock_gres,
-						  (pick_gres - alloc_gres_cnt),
+						  sock_gres_needed,
 						  node_inx, s, NULL, NULL);
 	}
 	if (alloc_gres_cnt < pick_gres)
@@ -1001,7 +1018,15 @@ static int _set_job_bits1(int node_inx, int job_node_inx, int rem_nodes,
 						  (pick_gres - alloc_gres_cnt),
 						  node_inx, ANY_SOCK_TEST, NULL,
 						  NULL);
-	if (alloc_gres_cnt == 0) {
+	for (s = 0; ((s < sock_cnt) && (alloc_gres_cnt < pick_gres)); s++) {
+		if (!cores_on_sock[s])
+			continue;
+		alloc_gres_cnt += _pick_gres_topo(sock_gres,
+						  (pick_gres - alloc_gres_cnt),
+						  node_inx, s, NULL, NULL);
+	}
+
+	if (alloc_gres_cnt == 0 && !enforce_binding) {
 		for (s = 0; ((s < sock_cnt) && (alloc_gres_cnt == 0)); s++) {
 			if (cores_on_sock[s])
 				continue;
@@ -1199,7 +1224,7 @@ static void _set_node_bits(int node_inx, int job_node_inx,
 
 	/*
 	 * Now pick specific GRES for these sockets.
-	 * First: Try to place one GRES per socket in this job's allocation.
+	 * First: Try to place GRES on sockets relative to allocated cores cnts.
 	 * Second: Try to place additional GRES on allocated sockets.
 	 * Third: Use any additional available GRES.
 	 */
@@ -1215,31 +1240,20 @@ static void _set_node_bits(int node_inx, int job_node_inx,
 	}
 
 	if (gres_needed) {
-		gres_needed -= _pick_gres_topo(sock_gres, 1, node_inx,
+		gres_needed -= _pick_gres_topo(sock_gres, gres_needed, node_inx,
 					       ANY_SOCK_TEST, sorted_gres,
 					       links_cnt);
 	}
 
-	/*
-	 * Try to place additional GRES on allocated sockets. Favor use of
-	 * GRES which are best linked to GRES which have already been selected.
-	 */
+	/* Try to place additional GRES on allocated sockets. */
 	for (s = 0; ((s < sock_cnt) && gres_needed); s++) {
 		if (!used_cores_on_sock[s])
 			continue;
 		gres_needed -= _pick_gres_topo(sock_gres, gres_needed, node_inx,
 					       s, sorted_gres, links_cnt);
 	}
-	if (gres_needed) {
-		gres_needed -= _pick_gres_topo(sock_gres, gres_needed,
-					       node_inx, ANY_SOCK_TEST,
-					       sorted_gres, links_cnt);
-	}
 
-	/*
-	 * Use any additional available GRES. Again, favor use of GRES
-	 * which are best linked to GRES which have already been selected.
-	 */
+	/* Use any additional available GRES.  */
 	if (gres_needed && !enforce_binding) {
 		for (s = 0; ((s < sock_cnt) && gres_needed); s++) {
 			/* Sockets we ignored before */
@@ -1299,7 +1313,7 @@ static void _set_task_bits(int node_inx, sock_gres_t *sock_gres,
 		gres_js->gres_per_task;
 	gres_needed -= total_res_gres;
 
-	/* First pick GRES for acitve sockets */
+	/* First pick GRES for active sockets */
 	for (s = 0; s < sock_cnt; s++) {
 		if (!tasks_per_socket[s])
 			continue;
@@ -1341,12 +1355,13 @@ static void _set_task_bits(int node_inx, sock_gres_t *sock_gres,
 }
 
 /* Build array to identify task count for each node-socket pair */
-static uint32_t **_build_tasks_per_node_sock(struct job_resources *job_res,
-					     uint8_t overcommit,
-					     gres_mc_data_t *tres_mc_ptr)
+static int _build_tasks_per_node_sock(struct job_resources *job_res,
+				      uint8_t overcommit,
+				      gres_mc_data_t *tres_mc_ptr,
+				      uint32_t ***tasks_per_node_socket_ptr)
 {
 	uint32_t **tasks_per_node_socket;
-	int j, node_cnt, job_node_inx = 0;
+	int rc = SLURM_SUCCESS, j, node_cnt, job_node_inx = 0;
 	int c, s, core_offset;
 	int cpus_per_task = 1, cpus_per_node, cpus_per_core;
 	int task_per_node_limit = 0;
@@ -1372,10 +1387,16 @@ static uint32_t **_build_tasks_per_node_sock(struct job_resources *job_res,
 		tasks_per_node_socket[i] = xcalloc(sock_cnt, sizeof(uint32_t));
 		if (tres_mc_ptr->ntasks_per_node) {
 			task_per_node_limit = tres_mc_ptr->ntasks_per_node;
+			cpus_per_task = MAX(1, job_res->cpus[job_node_inx] /
+						       job_res->tasks_per_node
+							       [job_node_inx]);
 		} else if (job_res->tasks_per_node &&
 			   job_res->tasks_per_node[job_node_inx]) {
 			task_per_node_limit =
 				job_res->tasks_per_node[job_node_inx];
+			cpus_per_task =
+				MAX(1, job_res->cpus[job_node_inx] /
+				    job_res->tasks_per_node[job_node_inx]);
 		} else {
 			/*
 			 * NOTE: We should never get here.
@@ -1402,7 +1423,12 @@ static uint32_t **_build_tasks_per_node_sock(struct job_resources *job_res,
 		}
 		core_offset = get_job_resources_offset(job_res, job_node_inx++,
 						       0, 0);
-		cpus_per_core = node_ptr->tpc;
+		if (tres_mc_ptr->threads_per_core)
+			cpus_per_core = MIN(node_ptr->tpc,
+					    tres_mc_ptr->threads_per_core);
+		else
+			cpus_per_core = node_ptr->tpc;
+
 		for (s = 0; s < sock_cnt; s++) {
 			int tasks_per_socket = 0, tpc, skip_cores = 0;
 			for (c = 0; c < cores_per_socket_cnt; c++) {
@@ -1482,10 +1508,13 @@ static uint32_t **_build_tasks_per_node_sock(struct job_resources *job_res,
 			}
 		}
 	}
-	if (rem_tasks > 0)	/* This should never happen */
+	if (rem_tasks > 0) { /* This should never happen */
 		error("%s: rem_tasks not zero (%d > 0)", __func__, rem_tasks);
+		rc = ESLURM_INVALID_GRES;
+	}
 
-	return tasks_per_node_socket;
+	*tasks_per_node_socket_ptr = tasks_per_node_socket;
+	return rc;
 }
 
 static void _free_tasks_per_node_sock(uint32_t **tasks_per_node_socket,
@@ -1671,8 +1700,8 @@ static int _set_res_core_bits(uint32_t **res_gres_per_sock,
 		if (*sock_with_res_cnt > 1 &&
 		    !(slurm_conf.select_type_param & MULTIPLE_SHARING_GRES_PJ)){
 			/*
-			 * Have to allocate gres accross more then one socket.
-			 * This is assuming one socket per gres configueration
+			 * Have to allocate gres across more then one socket.
+			 * This is assuming one socket per gres configuration
 			 * line.
 			 */
 			error("Restricted gpu cores on multiple sockets which requires MULTIPLE_SHARING_GRES_PJ to be set");
@@ -1765,8 +1794,12 @@ static int _select_and_set_node(void *x, void *arg)
 	}
 	if (gres_js->gres_per_task && /* Data needed */
 	    !*args->tasks_per_node_socket) { /* Not built yet */
-		*args->tasks_per_node_socket = _build_tasks_per_node_sock(
-			job_res, job_ptr->details->overcommit, tres_mc_ptr);
+		*rc = _build_tasks_per_node_sock(job_res,
+						 job_ptr->details->overcommit,
+						 tres_mc_ptr,
+						 args->tasks_per_node_socket);
+		if (*rc != SLURM_SUCCESS)
+			return -1;
 	}
 
 	tasks_per_node_socket = *args->tasks_per_node_socket;
@@ -1864,7 +1897,8 @@ static int _select_and_set_node(void *x, void *arg)
 					 sock_gres, job_id, tres_mc_ptr,
 					 node_ptr->tpc,
 					 args->used_cores_on_sock,
-					 args->used_core_cnt, total_res_gres);
+					 args->used_core_cnt, total_res_gres,
+					 enforce_binding);
 		if (*job_fini != 0)
 			*job_fini = tmp;
 	} else {
@@ -1889,7 +1923,7 @@ static int _select_and_set_node(void *x, void *arg)
  * tres_mc_ptr IN - job's multi-core options
  * RET SLURM_SUCCESS or error code
  */
-extern int gres_select_filter_select_and_set(List *sock_gres_list,
+extern int gres_select_filter_select_and_set(list_t **sock_gres_list,
 					     job_record_t *job_ptr,
 					     gres_mc_data_t *tres_mc_ptr)
 {
