@@ -75,9 +75,6 @@ typedef struct {
 	uint16_t tree_width;
 } _foreach_part_split_hostlist_t;
 
-static int _split_hostlist_treewidth(hostlist_t *hl, hostlist_t ***sp_hl,
-				     int *count, uint16_t tree_width);
-
 static int _part_split_hostlist(void *x, void *y)
 {
 	part_record_t *part_ptr = x;
@@ -98,8 +95,8 @@ static int _part_split_hostlist(void *x, void *y)
 	hl = bitmap2hostlist(arg->fwd_bitmap);
 
 	/* Generate FW tree hostlist array from partition's hostlist */
-	hl_depth = _split_hostlist_treewidth(hl, &p_hl, &hl_count,
-					     arg->tree_width);
+	hl_depth = hostlist_split_treewidth(hl, &p_hl, &hl_count,
+					    arg->tree_width);
 	hostlist_destroy(hl);
 
 	/* Make size for FW tree hostlist array in the main hostlist array */
@@ -124,7 +121,8 @@ static int _route_part_split_hostlist(hostlist_t *hl, hostlist_t ***sp_hl,
 				      int *count, uint16_t tree_width)
 {
 	slurmctld_lock_t node_read_lock = {
-		.node = READ_LOCK, .part = READ_LOCK
+		.node = READ_LOCK,
+		.part = READ_LOCK,
 	};
 	bitstr_t *nodes_bitmap = NULL;
 	_foreach_part_split_hostlist_t part_split;
@@ -199,142 +197,6 @@ static int _route_part_split_hostlist(hostlist_t *hl, hostlist_t ***sp_hl,
 	return part_split.depth;
 }
 
-/* this is used to set how many nodes are going to be on each branch
- * of the tree.
- * IN total       - total number of nodes to send to
- * IN tree_width  - how wide the tree should be on each hop
- * OUT span       - pointer to int array tree_width in length each space
- *		    containing the number of nodes to send to each hop
- *		    on the span.
- * RET int	  - the number of levels opened in the tree, or SLURM_ERROR
- */
-static int _set_span(int total, uint16_t tree_width, int **span)
-{
-	int depth = 0;
-
-	/* This should not happen. This is an error. */
-	if (!span || total < 1)
-		return SLURM_ERROR;
-
-	/* If default span */
-	if (!tree_width)
-		tree_width = slurm_conf.tree_width;
-
-	/* Safeguard from leaks */
-	if (*span)
-		xfree(*span);
-
-	/*
-	 * Memory optimization:
-	 * Don't allocate if we are in the last step to the leaves, as this is
-	 * considered direct communication and we don't really need it.
-	 */
-	if (total <= tree_width)
-		return 1;
-
-	/* Each cell will contain the #nodes below this specific branch */
-	*span = xcalloc(tree_width, sizeof(int));
-
-	/*
-	 * Try to fill levels until no more nodes are available.
-	 *
-	 * Each time a new level is created, it is exponentially bigger than the
-	 * previous one
-	 */
-	for (int branch_capacity = 1, level_capacity = tree_width; total;
-	     branch_capacity *= tree_width, level_capacity *= tree_width) {
-		/* Remaining nodes can fill a whole new level up, or not */
-		if (level_capacity <= total) {
-			for (int i = 0; i < tree_width; i++)
-				(*span)[i] += branch_capacity;
-			total -= level_capacity;
-		} else {
-			/* Evenly distribute remaining nodes */
-			branch_capacity = total / tree_width;
-			/* But left the division remainder ones */
-			level_capacity = branch_capacity * tree_width;
-			/* Fill current level up, as much as possible */
-			for (int i = 0; i < tree_width; i++)
-				(*span)[i] += branch_capacity;
-			total -= level_capacity;
-
-			/* Evenly distribute the remainder nodes */
-			for (int i = 0; total; i++, total--)
-				(*span)[i]++;
-
-			/* total == 0 always at this point */
-		}
-
-		/* One level more has been added */
-		depth++;
-
-		/* The level needed all the nodes, no more levels are added */
-		if (!total)
-			break;
-	}
-
-	/* Inform the caller about the number of levels below itself */
-	return depth;
-}
-
-static int _split_hostlist_treewidth(hostlist_t *hl, hostlist_t ***sp_hl,
-				     int *count, uint16_t tree_width)
-{
-	int host_count = hostlist_count(hl), depth, *span = NULL;
-	char *name;
-
-	/* If default span */
-	if (!tree_width)
-		tree_width = slurm_conf.tree_width;
-
-	/* This should not happen. This is an error. */
-	if ((depth = _set_span(host_count, tree_width, &span)) < 0)
-		return SLURM_ERROR;
-
-	/*
-	 * Memory optimization:
-	 * _set_span doesn't return array for direct communication
-	 * (if depth == 1 then span == NULL), so we just fill the hostlist array
-	 * directly.
-	 */
-	if (depth == 1)
-		tree_width = host_count;
-
-	/* Each cell will contain the hostlist below this specific branch */
-	*sp_hl = xcalloc(tree_width, sizeof(hostlist_t *));
-
-	/*
-	 * Fill the hostlists for each branch according to the distribution in
-	 * set_span.
-	 *
-	 * Additionally, try to preserve network locality (based on distance)
-	 * for subtrees, by assuming consecutive nodes are placed one next to
-	 * each other
-	 */
-	for (*count = 0; (*count < tree_width) && (name = hostlist_shift(hl));
-	     (*count)++) {
-		/* Open the new branch, and add the 1st one */
-		(*sp_hl)[*count] = hostlist_create(name);
-		free(name);
-
-		/* Consecutively add the rest of nodes for this branch */
-		for (int i = 1; span && (i < span[*count]); i++) {
-			name = hostlist_shift(hl);
-			hostlist_push_host((*sp_hl)[*count], name);
-			free(name);
-		}
-		if (slurm_conf.debug_flags & DEBUG_FLAG_ROUTE) {
-			char *buf =
-			hostlist_ranged_string_xmalloc((*sp_hl)[*count]);
-			debug("ROUTE: ... sublist[%d] %s", *count, buf);
-			xfree(buf);
-		}
-	}
-
-	xfree(span);
-	return depth;
-}
-
 extern int common_topo_split_hostlist_treewidth(hostlist_t *hl,
 						hostlist_t ***sp_hl,
 						int *count, uint16_t tree_width)
@@ -342,17 +204,14 @@ extern int common_topo_split_hostlist_treewidth(hostlist_t *hl,
 	if (running_in_slurmctld() && common_topo_route_part())
 		return _route_part_split_hostlist(hl, sp_hl, count, tree_width);
 
-	return _split_hostlist_treewidth(hl, sp_hl, count, tree_width);
+	return hostlist_split_treewidth(hl, sp_hl, count, tree_width);
 }
 
 extern int common_topo_get_node_addr(char *node_name, char **addr,
 				     char **pattern)
 {
-
-#ifndef HAVE_FRONT_END
 	if (find_node_record(node_name) == NULL)
 		return SLURM_ERROR;
-#endif
 
 	*addr = xstrdup(node_name);
 	*pattern = xstrdup("node");
@@ -474,6 +333,18 @@ extern int common_topo_choose_nodes(topology_eval_t *topo_eval)
 			    (avail_res_array[i]->avail_res_cnt <= count)) {
 				if (req_node_map && bit_test(req_node_map, i))
 					continue;
+				/*
+				 * We adjust avail_res_cnt to the minimum needed
+				 * for the evaluated nodes on every
+				 * eval_nodes().
+				 * So, we need to check again if some more nodes
+				 * can be removed in the updated nodeset before
+				 * increasing the count, or we could end up
+				 * removing more (possibly valid) nodes than
+				 * needed.
+				 */
+				if (nochange)
+					count--;
 				nochange = 0;
 				bit_clear(topo_eval->node_map, i);
 				bit_clear(orig_node_map, i);
