@@ -40,14 +40,14 @@
 
 #include "src/common/xstring.h"
 
-static void _topo_add_dist(uint32_t *dist, int inx)
+static void _topo_add_dist(uint32_t *dist, int inx, tree_context_t *ctx)
 {
-	for (int i = 0; i < switch_record_cnt; i++) {
-		if (switch_record_table[inx].switches_dist[i] == INFINITE ||
+	for (int i = 0; i < ctx->switch_count; i++) {
+		if (ctx->switch_table[inx].switches_dist[i] == INFINITE ||
 		    dist[i] == INFINITE) {
 			dist[i] = INFINITE;
 		} else {
-			dist[i] += switch_record_table[inx].switches_dist[i];
+			dist[i] += ctx->switch_table[inx].switches_dist[i];
 		}
 	}
 }
@@ -59,7 +59,8 @@ static void _topo_add_dist(uint32_t *dist, int inx)
  */
 static int _topo_compare_switches(int i, int j, int rem_nodes,
 				  int *switch_node_cnt, int rem_cpus,
-				  uint32_t *switch_cpu_cnt, bool *i_fit_out)
+				  uint32_t *switch_cpu_cnt, bool *i_fit_out,
+				  tree_context_t *ctx)
 {
 	while (1) {
 		bool i_fit = ((switch_node_cnt[i] >= rem_nodes) &&
@@ -80,12 +81,12 @@ static int _topo_compare_switches(int i, int j, int rem_nodes,
 			return -1;
 		}
 
-		if (((switch_record_table[i].parent != i) ||
-		     (switch_record_table[j].parent != j)) &&
-		    (switch_record_table[i].parent !=
-		     switch_record_table[j].parent)) {
-			i = switch_record_table[i].parent;
-			j = switch_record_table[j].parent;
+		if (((ctx->switch_table[i].parent != i) ||
+		     (ctx->switch_table[j].parent != j)) &&
+		    (ctx->switch_table[i].parent !=
+		     ctx->switch_table[j].parent)) {
+			i = ctx->switch_table[i].parent;
+			j = ctx->switch_table[j].parent;
 			continue;
 		}
 
@@ -96,9 +97,9 @@ static int _topo_compare_switches(int i, int j, int rem_nodes,
 		return 1;
 	if (switch_node_cnt[i] < switch_node_cnt[j])
 		return -1;
-	if (switch_record_table[i].level < switch_record_table[j].level)
+	if (ctx->switch_table[i].level < ctx->switch_table[j].level)
 		return 1;
-	if (switch_record_table[i].level > switch_record_table[j].level)
+	if (ctx->switch_table[i].level > ctx->switch_table[j].level)
 		return -1;
 	return 0;
 
@@ -106,7 +107,8 @@ static int _topo_compare_switches(int i, int j, int rem_nodes,
 
 static void _topo_choose_best_switch(uint32_t *dist, int *switch_node_cnt,
 				     int rem_nodes, uint32_t *switch_cpu_cnt,
-				     int rem_cpus, int i, int *best_switch)
+				     int rem_cpus, int i, int *best_switch,
+				     tree_context_t *ctx)
 {
 	int tcs = 0;
 	bool i_fit = false;
@@ -122,7 +124,7 @@ static void _topo_choose_best_switch(uint32_t *dist, int *switch_node_cnt,
 
 	tcs = _topo_compare_switches(i, *best_switch, rem_nodes,
 				     switch_node_cnt, rem_cpus, switch_cpu_cnt,
-				     &i_fit);
+				     &i_fit, ctx);
 	if (((dist[i] < dist[*best_switch]) && i_fit) ||
 	    ((dist[i] == dist[*best_switch]) && (tcs > 0))) {
 		/*
@@ -170,6 +172,7 @@ static int _eval_nodes_dfly(topology_eval_t *topo_eval)
 	uint32_t min_nodes = topo_eval->min_nodes;
 	uint32_t req_nodes = topo_eval->req_nodes;
 	uint64_t maxtasks;
+	tree_context_t *ctx = topo_eval->tctx->plugin_ctx;
 
 	topo_eval->avail_cpus = 0;
 
@@ -300,25 +303,25 @@ static int _eval_nodes_dfly(topology_eval_t *topo_eval)
 	 * Identify the highest level switch to be used.
 	 * Note that nodes can be on multiple non-overlapping switches.
 	 */
-	switch_gres = xcalloc(switch_record_cnt, sizeof(list_t *));
-	switch_node_bitmap = xcalloc(switch_record_cnt, sizeof(bitstr_t *));
-	switch_node_cnt    = xcalloc(switch_record_cnt, sizeof(int));
-	switch_required    = xcalloc(switch_record_cnt, sizeof(int));
+	switch_gres = xcalloc(ctx->switch_count, sizeof(list_t *));
+	switch_node_bitmap = xcalloc(ctx->switch_count, sizeof(bitstr_t *));
+	switch_node_cnt = xcalloc(ctx->switch_count, sizeof(int));
+	switch_required = xcalloc(ctx->switch_count, sizeof(int));
 
 	if (!req_nodes_bitmap)
 		nw = list_peek(node_weight_list);
-	for (i = 0, switch_ptr = switch_record_table; i < switch_record_cnt;
+	for (i = 0, switch_ptr = ctx->switch_table; i < ctx->switch_count;
 	     i++, switch_ptr++) {
 		switch_node_bitmap[i] = bit_copy(switch_ptr->node_bitmap);
 		if (req_nodes_bitmap &&
 		    bit_overlap_any(req_nodes_bitmap, switch_node_bitmap[i])) {
 			switch_required[i] = 1;
-			if (switch_record_table[i].level == 0) {
+			if (ctx->switch_table[i].level == 0) {
 				leaf_switch_count++;
 			}
 			if ((top_switch_inx == -1) ||
-			    (switch_record_table[i].level >
-			     switch_record_table[top_switch_inx].level)) {
+			    (ctx->switch_table[i].level >
+			     ctx->switch_table[top_switch_inx].level)) {
 				top_switch_inx = i;
 			}
 		}
@@ -327,8 +330,8 @@ static int _eval_nodes_dfly(topology_eval_t *topo_eval)
 				     eval_nodes_topo_node_find,
 				     switch_node_bitmap[i]))) {
 			if ((top_switch_inx == -1) ||
-			    (switch_record_table[i].level >
-			     switch_record_table[top_switch_inx].level)) {
+			    (ctx->switch_table[i].level >
+			     ctx->switch_table[top_switch_inx].level)) {
 				top_switch_inx = i;
 			}
 		}
@@ -346,7 +349,7 @@ static int _eval_nodes_dfly(topology_eval_t *topo_eval)
 		goto fini;
 	}
 
-	/* Check that all specificly required nodes are on shared network */
+	/* Check that all specifically required nodes are on shared network */
 	if (req_nodes_bitmap &&
 	    !bit_super_set(req_nodes_bitmap,
 			   switch_node_bitmap[top_switch_inx])) {
@@ -360,7 +363,7 @@ static int _eval_nodes_dfly(topology_eval_t *topo_eval)
 	 * Remove nodes from consideration that can not be reached from this
 	 * top level switch
 	 */
-	for (i = 0; i < switch_record_cnt; i++) {
+	for (i = 0; i < ctx->switch_count; i++) {
 		if (top_switch_inx != i) {
 			  bit_and(switch_node_bitmap[i],
 				  switch_node_bitmap[top_switch_inx]);
@@ -472,14 +475,14 @@ static int _eval_nodes_dfly(topology_eval_t *topo_eval)
 			rem_max_cpus -= topo_eval->avail_cpus;
 		}
 
-		for (i = 0, switch_ptr = switch_record_table;
-		     i < switch_record_cnt; i++, switch_ptr++) {
+		for (i = 0, switch_ptr = ctx->switch_table;
+		     i < ctx->switch_count; i++, switch_ptr++) {
 			if (switch_required[i])
 				continue;
 			if (bit_overlap_any(req2_nodes_bitmap,
 					    switch_node_bitmap[i])) {
 				switch_required[i] = 1;
-				if (switch_record_table[i].level == 0) {
+				if (ctx->switch_table[i].level == 0) {
 					leaf_switch_count++;
 				}
 			}
@@ -505,11 +508,11 @@ static int _eval_nodes_dfly(topology_eval_t *topo_eval)
 
 	/*
 	 * Construct a set of switch array entries.
-	 * Use the same indexes as switch_record_table in slurmctld.
+	 * Use the same indexes as ctx->switch_table in slurmctld.
 	 */
 	bit_or(best_nodes_bitmap, topo_eval->node_map);
 	avail_nodes_bitmap = bit_alloc(node_record_count);
-	for (i = 0, switch_ptr = switch_record_table; i < switch_record_cnt;
+	for (i = 0, switch_ptr = ctx->switch_table; i < ctx->switch_count;
 	     i++, switch_ptr++) {
 		bit_and(switch_node_bitmap[i], best_nodes_bitmap);
 		bit_or(avail_nodes_bitmap, switch_node_bitmap[i]);
@@ -517,18 +520,18 @@ static int _eval_nodes_dfly(topology_eval_t *topo_eval)
 	}
 
 	if (slurm_conf.debug_flags & DEBUG_FLAG_SELECT_TYPE) {
-		for (i = 0; i < switch_record_cnt; i++) {
+		for (i = 0; i < ctx->switch_count; i++) {
 			char *node_names = NULL;
 			if (switch_node_cnt[i]) {
 				node_names =
 					bitmap2node_name(switch_node_bitmap[i]);
 			}
 			info("switch=%s level=%d nodes=%u:%s required:%u speed:%u",
-			     switch_record_table[i].name,
-			     switch_record_table[i].level,
+			     ctx->switch_table[i].name,
+			     ctx->switch_table[i].level,
 			     switch_node_cnt[i], node_names,
 			     switch_required[i],
-			     switch_record_table[i].link_speed);
+			     ctx->switch_table[i].link_speed);
 			xfree(node_names);
 		}
 	}
@@ -547,8 +550,8 @@ static int _eval_nodes_dfly(topology_eval_t *topo_eval)
 	 */
 	if (leaf_switch_count == 0) {
 		int best_switch_inx = -1;
-		for (i = 0; i < switch_record_cnt; i++) {
-			if (switch_record_table[i].level != 0)
+		for (i = 0; i < ctx->switch_count; i++) {
+			if (ctx->switch_table[i].level != 0)
 				continue;
 			if ((best_switch_inx == -1) ||
 			    (switch_node_cnt[i] >
@@ -569,9 +572,9 @@ static int _eval_nodes_dfly(topology_eval_t *topo_eval)
 		best_cpu_cnt = 0;
 		best_node_cnt = 0;
 		FREE_NULL_LIST(best_gres);
-		for (i = 0; i < switch_record_cnt; i++) {
+		for (i = 0; i < ctx->switch_count; i++) {
 			if (!switch_required[i] || !switch_node_bitmap[i] ||
-			    (switch_record_table[i].level != 0))
+			    (ctx->switch_table[i].level != 0))
 				continue;
 			for (j = 0; next_node_bitmap(switch_node_bitmap[i], &j);
 			     j++) {
@@ -598,7 +601,7 @@ static int _eval_nodes_dfly(topology_eval_t *topo_eval)
 			sufficient = gres_sched_sufficient(
 				job_ptr->gres_list_req, best_gres);
 		}
-		if (sufficient && (i < switch_record_cnt)) {
+		if (sufficient && (i < ctx->switch_count)) {
 			/* Complete request using this one leaf switch */
 			for (j = 0; next_node_bitmap(switch_node_bitmap[i], &j);
 			     j++) {
@@ -646,9 +649,9 @@ static int _eval_nodes_dfly(topology_eval_t *topo_eval)
 		if (prev_rem_nodes == rem_nodes)
 			break;	/* Stalled */
 		prev_rem_nodes = rem_nodes;
-		for (i = 0; i < switch_record_cnt; i++) {
+		for (i = 0; i < ctx->switch_count; i++) {
 			if (!switch_node_bitmap[i] ||
-			    (switch_record_table[i].level != 0))
+			    (ctx->switch_table[i].level != 0))
 				continue;
 			for (j = 0; next_node_bitmap(switch_node_bitmap[i], &j);
 			     j++) {
@@ -704,9 +707,9 @@ fini:
 		leaf_switch_count = 0;
 
 		/* count up leaf switches */
-		for (i = 0, switch_ptr = switch_record_table;
-		     i < switch_record_cnt; i++, switch_ptr++) {
-			if (switch_record_table[i].level != 0)
+		for (i = 0, switch_ptr = ctx->switch_table;
+		     i < ctx->switch_count; i++, switch_ptr++) {
+			if (ctx->switch_table[i].level != 0)
 				continue;
 			if (bit_overlap_any(switch_node_bitmap[i], topo_eval->node_map))
 				leaf_switch_count++;
@@ -738,7 +741,7 @@ fini:
 	xfree(avail_cpu_per_node);
 	xfree(switch_gres);
 	if (switch_node_bitmap) {
-		for (i = 0; i < switch_record_cnt; i++)
+		for (i = 0; i < ctx->switch_count; i++)
 			FREE_NULL_BITMAP(switch_node_bitmap[i]);
 		xfree(switch_node_bitmap);
 	}
@@ -748,9 +751,9 @@ fini:
 }
 
 static void _decrement_node_cnt(int num_nodes_taken, int switch_index,
-				int *switch_node_cnt)
+				int *switch_node_cnt, tree_context_t *ctx)
 {
-	for (int i = switch_index; i >= 0; i = switch_record_table[i].parent) {
+	for (int i = switch_index; i >= 0; i = ctx->switch_table[i].parent) {
 		if (switch_node_cnt[i] <= num_nodes_taken) {
 			switch_node_cnt[i] = 0;
 		} else {
@@ -758,7 +761,7 @@ static void _decrement_node_cnt(int num_nodes_taken, int switch_index,
 		}
 
 		/* end once we've reached root switch */
-		if (switch_record_table[i].parent == SWITCH_NO_PARENT)
+		if (ctx->switch_table[i].parent == SWITCH_NO_PARENT)
 			break;
 	}
 }
@@ -802,6 +805,7 @@ static int _eval_nodes_topo(topology_eval_t *topo_eval)
 	uint32_t req_nodes = topo_eval->req_nodes;
 	uint32_t org_max_nodes = topo_eval->max_nodes;
 	uint64_t maxtasks;
+	tree_context_t *ctx = topo_eval->tctx->plugin_ctx;
 
 	topo_eval->avail_cpus = 0;
 
@@ -910,14 +914,15 @@ static int _eval_nodes_topo(topology_eval_t *topo_eval)
 	 * Identify the highest level switch to be used.
 	 * Note that nodes can be on multiple non-overlapping switches.
 	 */
-	switch_cpu_cnt = xcalloc(switch_record_cnt, sizeof(uint32_t));
-	switch_node_bitmap = xcalloc(switch_record_cnt, sizeof(bitstr_t *));
-	start_switch_node_bitmap = xcalloc(switch_record_cnt, sizeof(bitstr_t *));
-	switch_node_cnt    = xcalloc(switch_record_cnt, sizeof(int));
-	switch_required    = xcalloc(switch_record_cnt, sizeof(int));
-	req_switch_required = xcalloc(switch_record_cnt, sizeof(int));
+	switch_cpu_cnt = xcalloc(ctx->switch_count, sizeof(uint32_t));
+	switch_node_bitmap = xcalloc(ctx->switch_count, sizeof(bitstr_t *));
+	start_switch_node_bitmap =
+		xcalloc(ctx->switch_count, sizeof(bitstr_t *));
+	switch_node_cnt = xcalloc(ctx->switch_count, sizeof(int));
+	switch_required = xcalloc(ctx->switch_count, sizeof(int));
+	req_switch_required = xcalloc(ctx->switch_count, sizeof(int));
 
-	for (i = 0, switch_ptr = switch_record_table; i < switch_record_cnt;
+	for (i = 0, switch_ptr = ctx->switch_table; i < ctx->switch_count;
 	     i++, switch_ptr++) {
 		uint32_t switch_cpus = 0;
 		switch_node_bitmap[i] = bit_copy(switch_ptr->node_bitmap);
@@ -936,8 +941,8 @@ static int _eval_nodes_topo(topology_eval_t *topo_eval)
 		    bit_overlap_any(req_nodes_bitmap, switch_node_bitmap[i])) {
 			switch_required[i] = 1;
 			if ((top_switch_inx == -1) ||
-			    (switch_record_table[i].level >
-			     switch_record_table[top_switch_inx].level)) {
+			    (ctx->switch_table[i].level >
+			     ctx->switch_table[top_switch_inx].level)) {
 				top_switch_inx = i;
 			}
 		}
@@ -950,8 +955,8 @@ static int _eval_nodes_topo(topology_eval_t *topo_eval)
 					  eval_nodes_topo_node_find,
 				    switch_node_bitmap[i]))) {
 			if ((top_switch_inx == -1) ||
-			    ((switch_record_table[i].level >=
-			      switch_record_table[top_switch_inx].level) &&
+			    ((ctx->switch_table[i].level >=
+			      ctx->switch_table[top_switch_inx].level) &&
 			     (nw->weight <= top_switch_lowest_weight))) {
 				top_switch_inx = i;
 				top_switch_lowest_weight = nw->weight;
@@ -976,7 +981,7 @@ static int _eval_nodes_topo(topology_eval_t *topo_eval)
 		goto fini;
 	}
 
-	/* Check that all specificly required nodes are on shared network */
+	/* Check that all specifically required nodes are on shared network */
 	if (req_nodes_bitmap &&
 	    !bit_super_set(req_nodes_bitmap,
 			   switch_node_bitmap[top_switch_inx])) {
@@ -990,7 +995,7 @@ static int _eval_nodes_topo(topology_eval_t *topo_eval)
 	 * Remove nodes from consideration that can not be reached from this
 	 * top level switch.
 	 */
-	for (i = 0; i < switch_record_cnt; i++) {
+	for (i = 0; i < ctx->switch_count; i++) {
 		if (top_switch_inx != i) {
 			  bit_and(switch_node_bitmap[i],
 				  switch_node_bitmap[top_switch_inx]);
@@ -1017,8 +1022,8 @@ static int _eval_nodes_topo(topology_eval_t *topo_eval)
 
 	start_node_map = bit_copy(topo_eval->node_map);
 	memcpy(req_switch_required, switch_required,
-	       switch_record_cnt * sizeof(int));
-	for (i = 0; i < switch_record_cnt; i++)
+	       ctx->switch_count * sizeof(int));
+	for (i = 0; i < ctx->switch_count; i++)
 		start_switch_node_bitmap[i] = bit_copy(switch_node_bitmap[i]);
 
 try_again:
@@ -1144,8 +1149,8 @@ try_again:
 			rem_max_cpus -= topo_eval->avail_cpus;
 		}
 
-		for (i = 0, switch_ptr = switch_record_table;
-		     i < switch_record_cnt; i++, switch_ptr++) {
+		for (i = 0, switch_ptr = ctx->switch_table;
+		     i < ctx->switch_count; i++, switch_ptr++) {
 			if (switch_required[i])
 				continue;
 			if (bit_overlap_any(req2_nodes_bitmap,
@@ -1175,11 +1180,11 @@ try_again:
 
 	/*
 	 * Construct a set of switch array entries.
-	 * Use the same indexes as switch_record_table in slurmctld.
+	 * Use the same indexes as ctx->switch_table in slurmctld.
 	 */
 	bit_or(best_nodes_bitmap, topo_eval->node_map);
 	avail_nodes_bitmap = bit_alloc(node_record_count);
-	for (i = 0, switch_ptr = switch_record_table; i < switch_record_cnt;
+	for (i = 0, switch_ptr = ctx->switch_table; i < ctx->switch_count;
 	     i++, switch_ptr++) {
 		bit_and(switch_node_bitmap[i], best_nodes_bitmap);
 		bit_or(avail_nodes_bitmap, switch_node_bitmap[i]);
@@ -1187,18 +1192,18 @@ try_again:
 	}
 
 	if (slurm_conf.debug_flags & DEBUG_FLAG_SELECT_TYPE) {
-		for (i = 0; i < switch_record_cnt; i++) {
+		for (i = 0; i < ctx->switch_count; i++) {
 			char *node_names = NULL;
 			if (switch_node_cnt[i]) {
 				node_names =
 					bitmap2node_name(switch_node_bitmap[i]);
 			}
 			info("switch=%s level=%d nodes=%u:%s required:%u speed:%u",
-			     switch_record_table[i].name,
-			     switch_record_table[i].level,
+			     ctx->switch_table[i].name,
+			     ctx->switch_table[i].level,
 			     switch_node_cnt[i], node_names,
 			     switch_required[i],
-			     switch_record_table[i].link_speed);
+			     ctx->switch_table[i].link_speed);
 			xfree(node_names);
 		}
 	}
@@ -1206,9 +1211,9 @@ try_again:
 	/* Add additional resources for already required leaf switches */
 	if (req_nodes_bitmap || req2_nodes_bitmap) {
 		int num_nodes_taken = 0;
-		for (i = 0; i < switch_record_cnt; i++) {
+		for (i = 0; i < ctx->switch_count; i++) {
 			if (!switch_required[i] || !switch_node_bitmap[i] ||
-			    (switch_record_table[i].level != 0))
+			    (ctx->switch_table[i].level != 0))
 				continue;
 			for (j = 0; next_node_bitmap(switch_node_bitmap[i], &j);
 			     j++) {
@@ -1237,18 +1242,26 @@ try_again:
 					rc = SLURM_SUCCESS;
 					goto fini;
 				}
+
+				if (topo_eval->max_nodes <= 0) {
+					rc = SLURM_ERROR;
+					log_flag(SELECT_TYPE,
+						 "%pJ reached maximum node limit",
+						 job_ptr);
+					goto fini;
+				}
 			}
 
-			_decrement_node_cnt(num_nodes_taken, i,
-					    switch_node_cnt);
+			_decrement_node_cnt(num_nodes_taken, i, switch_node_cnt,
+					    ctx);
 		}
 	}
 
-	switches_dist = xcalloc(switch_record_cnt, sizeof(uint32_t));
+	switches_dist = xcalloc(ctx->switch_count, sizeof(uint32_t));
 
-	for (i = 0; i < switch_record_cnt; i++) {
+	for (i = 0; i < ctx->switch_count; i++) {
 		if (switch_required[i])
-			_topo_add_dist(switches_dist, i);
+			_topo_add_dist(switches_dist, i, ctx);
 	}
 	/* Add additional resources as required from additional leaf switches */
 	prev_rem_nodes = rem_nodes + 1;
@@ -1259,19 +1272,19 @@ try_again:
 			break; 	/* Stalled */
 		prev_rem_nodes = rem_nodes;
 
-		for (i = 0; i < switch_record_cnt; i++) {
+		for (i = 0; i < ctx->switch_count; i++) {
 			if (switch_required[i] || !switch_node_bitmap[i] ||
-			    (switch_record_table[i].level != 0))
+			    (ctx->switch_table[i].level != 0))
 				continue;
 			_topo_choose_best_switch(switches_dist, switch_node_cnt,
 						 rem_nodes, switch_cpu_cnt,
-						 rem_cpus, i, &best_switch_inx);
-
+						 rem_cpus, i, &best_switch_inx,
+						 ctx);
 		}
 		if (best_switch_inx == -1)
 			break;
 
-		_topo_add_dist(switches_dist, best_switch_inx);
+		_topo_add_dist(switches_dist, best_switch_inx, ctx);
 		/*
 		 * NOTE: Ideally we would add nodes in order of resource
 		 * availability rather than in order of bitmap position, but
@@ -1305,9 +1318,17 @@ try_again:
 				rc = SLURM_SUCCESS;
 				goto fini;
 			}
+
+			if (topo_eval->max_nodes <= 0) {
+				rc = SLURM_ERROR;
+				log_flag(SELECT_TYPE,
+					 "%pJ reached maximum node limit",
+					 job_ptr);
+				goto fini;
+			}
 		}
 		_decrement_node_cnt(switch_node_cnt[best_switch_inx],
-				    best_switch_inx, switch_node_cnt);
+				    best_switch_inx, switch_node_cnt, ctx);
 		switch_node_cnt[best_switch_inx] = 0;	/* Used all */
 	}
 	if ((min_rem_nodes <= 0) && (rem_cpus <= 0) &&
@@ -1326,9 +1347,9 @@ fini:
 		int leaf_switch_count = 0;
 
 		/* Count up leaf switches. */
-		for (i = 0, switch_ptr = switch_record_table;
-		     i < switch_record_cnt; i++, switch_ptr++) {
-			if (switch_record_table[i].level != 0)
+		for (i = 0, switch_ptr = ctx->switch_table;
+		     i < ctx->switch_count; i++, switch_ptr++) {
+			if (ctx->switch_table[i].level != 0)
 				continue;
 			if (bit_overlap_any(switch_node_bitmap[i], topo_eval->node_map))
 				leaf_switch_count++;
@@ -1356,10 +1377,10 @@ fini:
 				xfree(switches_dist);
 				bit_copybits(topo_eval->node_map, start_node_map);
 				memcpy(switch_required, req_switch_required,
-				       switch_record_cnt * sizeof(int));
+				       ctx->switch_count * sizeof(int));
 				memset(avail_cpu_per_node, 0,
 				       node_record_count * sizeof(uint16_t));
-				for (i = 0; i < switch_record_cnt; i++)
+				for (i = 0; i < ctx->switch_count; i++)
 					bit_copybits(
 						switch_node_bitmap[i],
 						start_switch_node_bitmap[i]);
@@ -1389,12 +1410,12 @@ fini:
 	xfree(avail_cpu_per_node);
 	xfree(switch_cpu_cnt);
 	if (switch_node_bitmap) {
-		for (i = 0; i < switch_record_cnt; i++)
+		for (i = 0; i < ctx->switch_count; i++)
 			FREE_NULL_BITMAP(switch_node_bitmap[i]);
 		xfree(switch_node_bitmap);
 	}
 	if (start_switch_node_bitmap) {
-		for (i = 0; i < switch_record_cnt; i++)
+		for (i = 0; i < ctx->switch_count; i++)
 			FREE_NULL_BITMAP(start_switch_node_bitmap[i]);
 		xfree(start_switch_node_bitmap);
 	}
@@ -1423,8 +1444,8 @@ extern int eval_nodes_tree(topology_eval_t *topo_eval)
 		set = true;
 	}
 
-	xassert(switch_record_cnt);
-	xassert(switch_record_table);
+	xassert(((tree_context_t *) topo_eval->tctx->plugin_ctx)->switch_count);
+	xassert(((tree_context_t *) topo_eval->tctx->plugin_ctx)->switch_table);
 
 	if (!details_ptr->contiguous &&
 	    ((topo_optional == false) || topo_eval->job_ptr->req_switch)) {
