@@ -61,8 +61,6 @@
 #include "src/interfaces/jobcomp.h"
 #include "src/common/print_fields.h"
 
-#define ERROR 2
-
 #define BRIEF_FIELDS "jobid,state,exitcode"
 #define BRIEF_COMP_FIELDS "jobid,uid,state"
 #define DEFAULT_FIELDS "jobid,jobname,partition,account,alloccpus,state,exitcode"
@@ -167,7 +165,9 @@ typedef enum {
 		PRINT_REQ_NODES,
 		PRINT_RESERVATION,
 		PRINT_RESERVATION_ID,
+		PRINT_RESERVATION_REQ,
 		PRINT_RESTART_CNT,
+		PRINT_SEGMENT_SIZE,
 		PRINT_SLUID,
 		PRINT_START,
 		PRINT_STATE,
@@ -216,7 +216,6 @@ typedef struct {
 	bool opt_federation;	/* --federation */
 	char *opt_field_list;	/* --fields= */
 	gid_t opt_gid;		/* running persons gid */
-	int opt_help;		/* --help */
 	bool opt_local;		/* --local */
 	int opt_noheader;	/* can only be cleared */
 	uid_t opt_uid;		/* running persons uid */
@@ -245,7 +244,6 @@ void print_fields(type_t type, void *object);
 /* options.c */
 int  get_data(void);
 void parse_command_line(int argc, char **argv);
-void do_help(void);
 void do_list(int argc, char **argv);
 void do_list_completion(void);
 void sacct_init(void);
