@@ -162,13 +162,13 @@ static void _parse_create(int argc, char **argv)
 			state.console_socket = xstrdup(optarg);
 			break;
 		case OPT_LONG_NO_PIVOT:
-			info("WARNING: ignoring --no-pivot argument");
+			debug("WARNING: ignoring --no-pivot argument");
 			break;
 		case OPT_LONG_NO_NEW_KEYRING:
-			info("WARNING: ignoring --no-new-keyring argument");
+			debug("WARNING: ignoring --no-new-keyring argument");
 			break;
 		case OPT_LONG_PRESERVE_FDS:
-			info("WARNING: ignoring --preserve-fds argument");
+			debug("WARNING: ignoring --preserve-fds argument");
 			break;
 		case OPT_LONG_PID_FILE:
 			xfree(state.pid_file);
@@ -252,7 +252,7 @@ static void _parse_kill(int argc, char **argv)
 				&option_index)) != -1) {
 		switch (c) {
 		case OPT_LONG_ALL:
-			info("WARNING: ignoring --all argument");
+			debug("WARNING: ignoring --all argument");
 			break;
 		default:
 			fatal("unknown argument: %s", argv[optopt]);
@@ -524,7 +524,7 @@ static int _parse_commandline(int argc, char **argv)
 				&option_index)) != -1) {
 		switch (c) {
 		case OPT_LONG_CGROUP_MANAGER:
-			info("WARNING: ignoring --cgroup-manager argument");
+			debug("WARNING: ignoring --cgroup-manager argument");
 			break;
 		case OPT_LONG_LOG_FILE :
 			xfree(log_file);
@@ -554,10 +554,10 @@ static int _parse_commandline(int argc, char **argv)
 			state.root_dir = xstrdup(optarg);
 			break;
 		case OPT_LONG_ROOTLESS:
-			info("WARNING: ignoring --rootless argument");
+			debug("WARNING: ignoring --rootless argument");
 			break;
 		case OPT_LONG_SYSTEMD_CGROUP:
-			info("WARNING: ignoring --systemd-cgroup argument");
+			debug("WARNING: ignoring --systemd-cgroup argument");
 			break;
 		case '?':
 		case OPT_LONG_HELP:
@@ -640,7 +640,7 @@ static void _set_root()
 		return;
 
 	if (!getuid())
-		fatal("scrun is being run as root and is likely inside of a username space. Refusing to guess path for --root. It must be explicilty provided.");
+		fatal("scrun is being run as root and is likely inside of a username space. Refusing to guess path for --root. It must be explicitly provided.");
 
 	path = xstrdup_printf("/run/user/%d/", getuid());
 	rc = _try_tmp_path(path);
@@ -684,9 +684,7 @@ extern int main(int argc, char **argv)
 	if (!state.root_dir || !state.root_dir[0])
 		_set_root();
 
-	if ((rc = serializer_g_init(MIME_TYPE_JSON_PLUGIN, NULL)))
-		fatal("%s: error loading JSON parser: %s", __func__,
-		      slurm_strerror(rc));
+	serializer_required(MIME_TYPE_JSON);
 
 	if (get_log_level() >= LOG_LEVEL_DEBUG2) {
 		for (int i = 0; i < argc; i++)
@@ -721,7 +719,6 @@ extern int main(int argc, char **argv)
 	xfree(slurm_conf_filename);
 	xfree(command_argv);
 	fini_setproctitle();
-	select_g_fini();
 	slurm_fini();
 	log_fini();
 #endif /* MEMORY_LEAK_DEBUG */
