@@ -2,7 +2,7 @@
  *  gang.c - Gang scheduler functions.
  *****************************************************************************
  *  Copyright (C) 2008 Hewlett-Packard Development Company, L.P.
- *  Portions Copyright (C) 2010 SchedMD <https://www.schedmd.com>.
+ *  Copyright (C) SchedMD LLC.
  *  Written by Chris Holmes
  *  CODE-OCEC-09-009. All rights reserved.
  *
@@ -65,7 +65,7 @@ static pthread_mutex_t term_lock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_cond_t  term_cond = PTHREAD_COND_INITIALIZER;
 static pthread_mutex_t thread_flag_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_t timeslicer_thread_id = (pthread_t) 0;
-static List preempt_job_list = (List) NULL;
+static list_t *preempt_job_list = NULL;
 
 /* timeslicer flags and structures */
 enum entity_type {
@@ -148,11 +148,10 @@ struct gs_part {
 /* global variables */
 static uint32_t timeslicer_seconds = 0;
 static uint16_t gr_type = GS_NODE;
-static List gs_part_list = NULL;
+static list_t *gs_part_list = NULL;
 static uint32_t default_job_list_size = 64;
 static pthread_mutex_t data_mutex = PTHREAD_MUTEX_INITIALIZER;
 
-static uint16_t *gs_bits_per_node = NULL;
 static uint32_t num_sorted_part = 0;
 
 /* function declarations */
@@ -244,41 +243,6 @@ static uint16_t _get_part_gr_type(part_record_t *part_ptr)
 	return gr_type;
 }
 
-/* For GS_CPU and GS_CPU2 gs_bits_per_node is the total number of CPUs per node.
- * For GS_CORE and GS_SOCKET gs_bits_per_node is the total number of
- *	cores per per node.
- */
-static void _load_phys_res_cnt(void)
-{
-	uint16_t bit = 0;
-	uint32_t i, bit_index = 0;
-	node_record_t *node_ptr;
-
-	xfree(gs_bits_per_node);
-
-	if (gr_type == GS_NODE)
-		return;
-
-	gs_bits_per_node = xmalloc(node_record_count * sizeof(uint16_t));
-
-	for (int i = 0; (node_ptr = next_node(&i)); i++) {
-		if (gr_type == GS_CPU) {
-			bit = node_ptr->config_ptr->cpus;
-		} else {
-			bit  = node_ptr->tot_cores;
-		}
-
-		gs_bits_per_node[bit_index++] = bit;
-	}
-
-	if (slurm_conf.debug_flags & DEBUG_FLAG_GANG) {
-		for (i = 0; i < bit_index; i++) {
-			info("gang: _load_phys_res_cnt: bits_per_node[%d]=%u",
-			     i, gs_bits_per_node[i]);
-		}
-	}
-}
-
 static uint16_t _get_phys_bit_cnt(int node_index)
 {
 	node_record_t *node_ptr = node_record_table_ptr[node_index];
@@ -314,7 +278,7 @@ static void _destroy_parts(void *x)
  * once a job is added. */
 static void _build_parts(void)
 {
-	ListIterator part_iterator;
+	list_itr_t *part_iterator;
 	part_record_t *p_ptr;
 	struct gs_part *gs_part_ptr;
 	int num_parts;
@@ -402,8 +366,7 @@ static int _job_fits_in_active_row(job_record_t *job_ptr,
 	job_gr_type = _get_part_gr_type(job_ptr->part_ptr);
 	if ((job_gr_type == GS_CPU2) || (job_gr_type == GS_CORE) ||
 	    (job_gr_type == GS_SOCKET)) {
-		return job_fits_into_cores(job_res, p_ptr->active_resmap,
-					   gs_bits_per_node);
+		return job_fits_into_cores(job_res, p_ptr->active_resmap);
 	}
 
 	/* job_gr_type == GS_NODE || job_gr_type == GS_CPU */
@@ -479,8 +442,7 @@ static void _add_job_to_active(job_record_t *job_ptr, struct gs_part *p_ptr)
 	    (job_gr_type == GS_SOCKET)) {
 		if (p_ptr->jobs_active == 0 && p_ptr->active_resmap)
 			bit_clear_all(p_ptr->active_resmap);
-		add_job_to_cores(job_res, &(p_ptr->active_resmap),
-				 gs_bits_per_node);
+		add_job_to_cores(job_res, &(p_ptr->active_resmap));
 		if (job_gr_type == GS_SOCKET)
 			_fill_sockets(job_res->node_bitmap, p_ptr);
 	} else { /* GS_NODE or GS_CPU */
@@ -544,7 +506,7 @@ static int _suspend_job(job_record_t *job_ptr)
 	msg.job_id = job_ptr->job_id;
 	msg.job_id_str = NULL;
 	msg.op = SUSPEND_JOB;
-	rc = job_suspend(&msg, 0, -1, false, NO_VAL16);
+	rc = job_suspend(NULL, &msg, 0, false, NO_VAL16);
 	/* job_suspend() returns ESLURM_DISABLED if job is already suspended */
 	if (rc == SLURM_SUCCESS) {
 		if (slurm_conf.debug_flags & DEBUG_FLAG_GANG)
@@ -566,7 +528,7 @@ static void _resume_job(job_record_t *job_ptr)
 	msg.job_id = job_ptr->job_id;
 	msg.job_id_str = NULL;
 	msg.op = RESUME_JOB;
-	rc = job_suspend(&msg, 0, -1, false, NO_VAL16);
+	rc = job_suspend(NULL, &msg, 0, false, NO_VAL16);
 	if (rc == SLURM_SUCCESS) {
 		if (slurm_conf.debug_flags & DEBUG_FLAG_GANG)
 			info("gang: resuming %pJ", job_ptr);
@@ -638,8 +600,6 @@ static void _preempt_job_dequeue(void)
 			}
 		}
 	}
-
-	return;
 }
 
 /* This is the reverse order defined by list.h so to generated a list in
@@ -648,23 +608,18 @@ static int _sort_partitions(void *part1, void *part2)
 {
 	struct gs_part *g1;
 	struct gs_part *g2;
-	int prio1;
-	int prio2;
 
 	g1 = *(struct gs_part **)part1;
 	g2 = *(struct gs_part **)part2;
 
-	prio1 = g1->priority;
-	prio2 = g2->priority;
-
-	return prio2 - prio1;
+	return slurm_sort_uint16_list_desc(&g1->priority, &g2->priority);
 }
 
 /* Scan the partition list. Add the given job as a "shadow" to every
  * partition with a lower priority than the given partition */
 static void _cast_shadow(struct gs_job *j_ptr, uint16_t priority)
 {
-	ListIterator part_iterator;
+	list_itr_t *part_iterator;
 	struct gs_part *p_ptr;
 	int i;
 
@@ -704,7 +659,7 @@ static void _cast_shadow(struct gs_job *j_ptr, uint16_t priority)
 /* Remove the given job as a "shadow" from all partitions */
 static void _clear_shadow(struct gs_job *j_ptr)
 {
-	ListIterator part_iterator;
+	list_itr_t *part_iterator;
 	struct gs_part *p_ptr;
 	int i;
 
@@ -836,7 +791,7 @@ static void _update_active_row(struct gs_part *p_ptr, int add_new_jobs)
  */
 static void _update_all_active_rows(void)
 {
-	ListIterator part_iterator;
+	list_itr_t *part_iterator;
  	struct gs_part *p_ptr;
 
 	/* Sort the partitions. This way the shadows of any high-priority
@@ -894,8 +849,6 @@ static void _remove_job_from_part(uint32_t job_id, struct gs_part *p_ptr,
 	}
 	j_ptr->job_ptr = NULL;
 	xfree(j_ptr);
-
-	return;
 }
 
 /* Add the given job to the given partition, and if it remains running
@@ -998,7 +951,7 @@ static void _scan_slurm_job_list(void)
 	job_record_t *job_ptr;
 	struct gs_part *p_ptr;
 	int i;
-	ListIterator job_iterator;
+	list_itr_t *job_iterator;
 	char *part_name;
 
 	if (!job_list) {	/* no jobs */
@@ -1055,8 +1008,6 @@ static void _scan_slurm_job_list(void)
 	/* now that all of the old jobs have been flushed out,
 	 * update the active row of all partitions */
 	_update_all_active_rows();
-
-	return;
 }
 
 
@@ -1107,9 +1058,6 @@ extern void gs_init(void)
 	gr_type = _get_gr_type();
 	preempt_job_list = list_create(xfree_ptr);
 
-	/* load the physical resource count data */
-	_load_phys_res_cnt();
-
 	slurm_mutex_lock(&data_mutex);
 	_build_parts();
 	/* load any currently running jobs */
@@ -1153,7 +1101,6 @@ extern void gs_fini(void)
 	slurm_mutex_lock(&data_mutex);
 	FREE_NULL_LIST(gs_part_list);
 	gs_part_list = NULL;
-	xfree(gs_bits_per_node);
 	slurm_mutex_unlock(&data_mutex);
 	log_flag(GANG, "gang: leaving gs_fini");
 }
@@ -1207,7 +1154,7 @@ extern void gs_job_start(job_record_t *job_ptr)
 extern void gs_wake_jobs(void)
 {
 	job_record_t *job_ptr;
-	ListIterator job_iterator;
+	list_itr_t *job_iterator;
 
 	if (!job_list)	/* no jobs */
 		return;
@@ -1286,9 +1233,9 @@ extern void gs_job_fini(job_record_t *job_ptr)
 extern void gs_reconfig(void)
 {
 	int i;
-	ListIterator part_iterator;
+	list_itr_t *part_iterator;
 	struct gs_part *p_ptr, *newp_ptr;
-	List old_part_list;
+	list_t *old_part_list = NULL;
 	job_record_t *job_ptr;
 	struct gs_job *j_ptr;
 
@@ -1309,7 +1256,6 @@ extern void gs_reconfig(void)
 
 	/* reset global data */
 	gr_type = _get_gr_type();
-	_load_phys_res_cnt();
 	_build_parts();
 
 	/* scan the old part list and add existing jobs to the new list */
@@ -1506,7 +1452,7 @@ static void *_timeslicer_thread(void *arg)
 	/* Write locks on job and read lock on nodes */
 	slurmctld_lock_t job_write_lock = {
 		NO_LOCK, WRITE_LOCK, READ_LOCK, NO_LOCK, READ_LOCK };
-	ListIterator part_iterator;
+	list_itr_t *part_iterator;
 	struct gs_part *p_ptr;
 
 	log_flag(GANG, "gang: starting timeslicer loop");
