@@ -33,6 +33,9 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
 \*****************************************************************************/
 
+#define _GNU_SOURCE
+#include <pthread.h>
+
 #include "src/common/macros.h"
 #include "src/common/xahash.h"
 #include "src/common/xstring.h"
@@ -47,6 +50,17 @@
 #define ONLY_DEBUG(...)
 #endif
 
+/*
+ * Favor writer lock acquisition to avoid delaying the scheduling thread
+ * when under heavy client load. Clients can be safely delayed, job launch
+ * is most important.
+ */
+#ifdef PTHREAD_RWLOCK_WRITER_NONRECURSIVE_INITIALIZER_NP
+#define CACHE_LOCK_INIT PTHREAD_RWLOCK_WRITER_NONRECURSIVE_INITIALIZER_NP
+#else
+#define CACHE_LOCK_INIT PTHREAD_RWLOCK_INITIALIZER
+#endif
+
 #define JOB_STATE_MIMIC_RECORD(js)                                             \
 	&(const job_record_t)                                                  \
 	{                                                                      \
@@ -117,6 +131,7 @@ typedef struct {
 	int rc;
 	uint32_t count;
 	job_state_response_job_t *jobs;
+	int jobs_count; /* Number of entries in jobs[] */
 	bool count_only;
 } job_state_args_t;
 
@@ -187,7 +202,7 @@ static xahash_table_t *array_job_cache_table = NULL;
  *	Maintains table_size for number of jobs reserved in hashtable
  */
 static xahash_table_t *array_task_cache_table = NULL;
-static pthread_rwlock_t cache_lock = PTHREAD_RWLOCK_INITIALIZER;
+static pthread_rwlock_t cache_lock = CACHE_LOCK_INIT;
 
 #ifndef NDEBUG
 
@@ -277,7 +292,7 @@ static void _log_job_state_change(const job_record_t *job_ptr,
 #define _log_array_job_chain(js, caller, fmt, ...) {}
 #define _check_all_jobs(compare_job_ptrs) {}
 #define _is_debug() (false)
-#define LOG(fmt, ...) do {} while (false)
+#define LOG(fmt, ...) log_flag(TRACE_JOBS, "%s: " fmt, __func__, ##__VA_ARGS__)
 #define _check_job_id(job_id_ptr) {(void) job_id_ptr;}
 #define _check_job_magic(js) {(void) js;}
 #define _check_array_job_magic(ajs) {(void) ajs;}
@@ -343,6 +358,9 @@ static job_state_response_job_t *_append_job_state(job_state_args_t *args)
 	if (args->count_only)
 		return NULL;
 
+	if (args->count > args->jobs_count)
+		return NULL;
+
 	index = args->count - 1;
 	rjob = &args->jobs[index];
 	xassert(!rjob->job_id);
@@ -405,7 +423,7 @@ static void _dump_job_state_locked(job_state_args_t *args,
 	}
 }
 
-static void _add_cache_job(job_state_args_t *args, const job_state_cached_t *js)
+static int _add_cache_job(job_state_args_t *args, const job_state_cached_t *js)
 {
 	job_state_response_job_t *rjob;
 
@@ -415,11 +433,12 @@ static void _add_cache_job(job_state_args_t *args, const job_state_cached_t *js)
 	rjob = _append_job_state(args);
 
 	if (args->count_only)
-		return;
+		return SLURM_SUCCESS;
 
 	if (!rjob) {
-		xassert(rjob);
-		return;
+		LOG("[%pJ] packing at %d/%d failed",
+		    JOB_STATE_MIMIC_RECORD(js), args->count, args->jobs_count);
+		return ERANGE;
 	}
 
 	if (_is_debug()) {
@@ -445,6 +464,8 @@ static void _add_cache_job(job_state_args_t *args, const job_state_cached_t *js)
 		rjob->array_task_id_bitmap = bit_copy(js->task_id_bitmap);
 	rjob->het_job_id = js->het_job_id;
 	rjob->state = js->job_state;
+
+	return SLURM_SUCCESS;
 }
 
 static xahash_foreach_control_t _foreach_cache_job(void *entry, void *state_ptr,
@@ -458,7 +479,9 @@ static xahash_foreach_control_t _foreach_cache_job(void *entry, void *state_ptr,
 	xassert(args->magic == MAGIC_JOB_STATE_ARGS);
 	xassert(js->magic == MAGIC_JOB_STATE_CACHED);
 
-	_add_cache_job(args, js);
+	if (_add_cache_job(args, js))
+		return XAHASH_FOREACH_FAIL;
+
 	return XAHASH_FOREACH_CONT;
 }
 
@@ -476,7 +499,8 @@ static void _find_job_state_cached_by_job_id(job_state_args_t *args,
 
 	LOG("[%pJ] Resolved from JobId=%u", JOB_STATE_MIMIC_RECORD(js), job_id);
 
-	_add_cache_job(args, js);
+	if (_add_cache_job(args, js))
+		return;
 
 	if (!resolve) {
 		LOG("[%pJ] Not fully resolving job", JOB_STATE_MIMIC_RECORD(js));
@@ -507,7 +531,8 @@ static void _find_job_state_cached_by_job_id(job_state_args_t *args,
 				    JOB_STATE_MIMIC_RECORD(js),
 				    JOB_STATE_MIMIC_RECORD(next),
 				    ARRAY_JOB_STATE_MIMIC_RECORD(ajs));
-				_add_cache_job(args, next);
+				if (_add_cache_job(args, next))
+					break;
 			} else {
 				fatal_abort("Unable to resolve next_job_id");
 			}
@@ -529,7 +554,8 @@ static void _find_job_state_cached_by_job_id(job_state_args_t *args,
 				LOG("[%pJ] Resolved HetJobId=%u+%u to %pJ",
 				    JOB_STATE_MIMIC_RECORD(js), job_id, i,
 				    JOB_STATE_MIMIC_RECORD(hjs));
-				_add_cache_job(args, hjs);
+				if (_add_cache_job(args, hjs))
+					break;
 			} else {
 				/*
 				 * Next job not found or not part of the HetJob
@@ -657,6 +683,8 @@ extern int dump_job_state(const uint32_t filter_jobs_count,
 			goto cleanup;
 		}
 
+		args.jobs_count = args.count;
+
 		/* reset count */
 		args.count_only = false;
 		args.count = 0;
@@ -670,7 +698,7 @@ extern int dump_job_state(const uint32_t filter_jobs_count,
 	}
 
 	*jobs_pptr = args.jobs;
-	*jobs_count_ptr = args.count;
+	*jobs_count_ptr = args.jobs_count;
 cleanup:
 	if (!use_cache)
 		unlock_slurmctld(job_read_lock);
@@ -741,7 +769,7 @@ static void _sync_job_task_id_bitmap(const job_record_t *job_ptr,
 		}
 
 		if (!js->task_id_bitmap) {
-			LOG("[%pJ] mimicing array without task_id_bitmap with new bitmap[%u]",
+			LOG("[%pJ] mimicking array without task_id_bitmap with new bitmap[%u]",
 			    JOB_STATE_MIMIC_RECORD(js), task_cnt);
 			js->task_id_bitmap = bit_alloc(task_cnt);
 		}
@@ -753,7 +781,7 @@ static void _sync_job_task_id_bitmap(const job_record_t *job_ptr,
 
 		if (_is_debug()) {
 			char *map = bit_fmt_full(js->task_id_bitmap);
-			LOG("[%pJ] mimicing array without bitmap as task_id_bitmap[%lu]: %s",
+			LOG("[%pJ] mimicking array without bitmap as task_id_bitmap[%lu]: %s",
 			    JOB_STATE_MIMIC_RECORD(js),
 			    bit_size(js->task_id_bitmap), map);
 			xfree(map);
@@ -1316,7 +1344,7 @@ static bool _array_task_match(void *entry, const void *key,
 	_check_array_task_magic(ats_key);
 	xassert(sizeof(*ats_key) == key_bytes);
 
-	/* treat NO_VAL and INFINTE as * for arrays */
+	/* treat NO_VAL and INFINITE as * for arrays */
 	if ((ats_key->array_task_id < NO_VAL) &&
 	    (ats->array_task_id != ats_key->array_task_id))
 		return false;
