@@ -226,6 +226,7 @@ static void _parse_exc_states(void)
 					  NODE_STATE_DRAIN |
 					  NODE_STATE_DYNAMIC_FUTURE |
 					  NODE_STATE_DYNAMIC_NORM |
+					  NODE_STATE_EXTERNAL |
 					  NODE_STATE_FAIL |
 					  NODE_STATE_INVALID_REG |
 					  NODE_STATE_MAINT |
@@ -445,7 +446,7 @@ static void _do_power_work(time_t now)
 	}
 
 	/*
-	 * Buid job to node mapping for json output
+	 * Build job to node mapping for json output
 	 * all_nodes = all nodes that need to be resumed this iteration
 	 * jobs[] - list of job to node mapping of nodes that the job needs to
 	 * be resumed for job. Multiple jobs can request the same nodes. Report
@@ -674,6 +675,7 @@ static void _do_power_work(time_t now)
 		 * Down nodes as if not resumed by ResumeTimeout
 		 */
 		if (bit_test(booting_node_bitmap, node_ptr->index) &&
+		    (node_ptr->resume_timeout != INFINITE16) &&
 		    (now >
 		     (node_ptr->boot_req_time + node_ptr->resume_timeout)) &&
 		    IS_NODE_POWERING_UP(node_ptr) &&
@@ -695,7 +697,7 @@ static void _do_power_work(time_t now)
 			 *
 			 * Call AFTER setting state adding POWERED_DOWN so that
 			 * the node is marked as "planned down" in the usage
-			 * tables becase:
+			 * tables because:
 			 * set_node_down_ptr()->_make_node_down()->
 			 * clusteracct_storage_g_node_down().
 			 */
@@ -736,15 +738,18 @@ static void _do_power_work(time_t now)
 	}
 
 	if (wake_node_bitmap) {
+		int rc;
 		char *nodes, *json = NULL;
 		nodes = bitmap2node_name(wake_node_bitmap);
 
 		data_set_string(data_key_set(resume_json_data,
 					     "all_nodes_resume"),
 				nodes);
-		if (serialize_g_data_to_string(&json, NULL, resume_json_data,
-					       MIME_TYPE_JSON,
-					       SER_FLAGS_COMPACT))
+		rc = serialize_g_data_to_string(&json, NULL, resume_json_data,
+						MIME_TYPE_JSON,
+						SER_FLAGS_COMPACT);
+		if ((rc != SLURM_SUCCESS) &&
+		    (rc != ESLURM_DATA_UNKNOWN_MIME_TYPE))
 			error("failed to generate json for resume job/node list");
 
 		if (nodes)
@@ -938,7 +943,7 @@ static void power_save_rl_setup(void)
 
 	/*
 	 * Power save either runs nominally close to power_save_interval
-	 * or, at worst, at the minumum rate. Either way, we'll want the
+	 * or, at worst, at the minimum rate. Either way, we'll want the
 	 * larger value for worst-case scenario in sizing bucket.
 	 */
 	effective_max_interval = MAX(1,
@@ -948,7 +953,7 @@ static void power_save_rl_setup(void)
 	if (resume_rate) {
 		/*
 		 * If the rate is high and/or the power save interval is large,
-		 * the bucket must be larger to accomodate large token
+		 * the bucket must be larger to accommodate large token
 		 * accumulation between executions of _do_power_work().
 		 * units are: (tokens) = ((tokens/min) * seconds) /
 		 *	                 (seconds / min)
@@ -1271,7 +1276,7 @@ extern void power_save_set_timeouts(bool *partition_suspend_time_set)
 	xassert(verify_lock(NODE_LOCK, WRITE_LOCK));
 	xassert(verify_lock(PART_LOCK, READ_LOCK));
 
-	/* Reset timeouts so new values can be caluclated. */
+	/* Reset timeouts so new values can be calculated. */
 	for (int i = 0; (node_ptr = next_node(&i)); i++) {
 		node_ptr->suspend_time = NO_VAL;
 		node_ptr->suspend_timeout = NO_VAL16;
