@@ -1,7 +1,7 @@
 /*****************************************************************************\
  *  bb_info.c - Functions related to Burst Buffer display mode of sview.
  *****************************************************************************
- *  Copyright (C) 2014-2015 SchedMD LLC.
+ *  Copyright (C) SchedMD LLC.
  *  Written by Nathan Yee <nyee32@shedmd.com>
  *
  *  This file is part of Slurm, a resource management program.
@@ -383,12 +383,12 @@ static void _append_bb_record(sview_bb_info_t *sview_bb_info_ptr,
 	_update_bb_record(sview_bb_info_ptr, treestore);
 }
 
-/* Update the Burst Buffer inforamtion record */
-static void _update_info_bb(List info_list, GtkTreeView *tree_view)
+/* Update the Burst Buffer information record */
+static void _update_info_bb(list_t *info_list, GtkTreeView *tree_view)
 {
 	GtkTreeModel *model = gtk_tree_view_get_model(tree_view);
 	char *name = NULL;
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	sview_bb_info_t *sview_bb_info = NULL;
 
 	set_for_update(model, SORTID_UPDATED);
@@ -426,11 +426,11 @@ static void _update_info_bb(List info_list, GtkTreeView *tree_view)
 	last_model = model;
 }
 
-static List _create_bb_info_list(burst_buffer_info_msg_t *bb_info_ptr)
+static list_t *_create_bb_info_list(burst_buffer_info_msg_t *bb_info_ptr)
 {
-	static List info_list = NULL;
-	List last_list = NULL;
-	ListIterator last_list_itr = NULL;
+	static list_t *info_list = NULL;
+	list_t *last_list = NULL;
+	list_itr_t *last_list_itr = NULL;
 	int i, j, pos = 0;
 	static burst_buffer_info_msg_t *last_bb_info_ptr = NULL;
 	sview_bb_info_t *sview_bb_info_ptr = NULL;
@@ -513,14 +513,14 @@ static List _create_bb_info_list(burst_buffer_info_msg_t *bb_info_ptr)
 	return info_list;
 }
 
-static void _display_info_bb(List info_list, popup_info_t *popup_win)
+static void _display_info_bb(list_t *info_list, popup_info_t *popup_win)
 {
 	specific_info_t *spec_info = popup_win->spec_info;
 	char *name = (char *)spec_info->search_info->gchar_data;
 	//int found = 0;
 	burst_buffer_resv_t *bb_ptr = NULL;
 	GtkTreeView *treeview = NULL;
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	sview_bb_info_t *sview_bb_info = NULL;
 	int update = 0;
 	char bb_name_id[32];
@@ -616,7 +616,7 @@ extern int get_new_info_bb(burst_buffer_info_msg_t **info_ptr, int force)
 		if (error_code == SLURM_SUCCESS) {
 			slurm_free_burst_buffer_info_msg(g_bb_info_ptr);
 			changed = 1;
-		} else if (slurm_get_errno() == SLURM_NO_CHANGE_IN_DATA) {
+		} else if (errno == SLURM_NO_CHANGE_IN_DATA) {
 			error_code = SLURM_NO_CHANGE_IN_DATA;
 			new_bb_ptr = g_bb_info_ptr;
 			changed = 0;
@@ -640,7 +640,7 @@ end_it:
 /* Create the model with types with known values */
 extern GtkListStore *create_model_bb(int type)
 {
-/* Since none of the values can be editted this is left blank */
+/* Since none of the values can be edited this is left blank */
 /* NOP */
 	return NULL;
 }
@@ -657,7 +657,7 @@ extern void admin_edit_bb(GtkCellRendererText *cell,
 extern void get_info_bb(GtkTable *table, display_data_t *display_data)
 {
 	int error_code = SLURM_SUCCESS;
-	List info_list = NULL;
+	list_t *info_list = NULL;
 	static int view = -1;
 	static burst_buffer_info_msg_t *bb_info_ptr = NULL;
 	char error_char[100];
@@ -715,7 +715,7 @@ extern void get_info_bb(GtkTable *table, display_data_t *display_data)
 			gtk_widget_destroy(display_widget);
 		view = ERROR_VIEW;
 		sprintf(error_char, "slurm_load_reservations: %s",
-			slurm_strerror(slurm_get_errno()));
+			slurm_strerror(errno));
 		label = gtk_label_new(error_char);
 		gtk_table_attach_defaults(table, label, 0, 1, 0, 1);
 		gtk_widget_show(label);
@@ -787,10 +787,10 @@ extern void specific_info_bb(popup_info_t *popup_win)
 	char error_char[100];
 	GtkWidget *label = NULL;
 	GtkTreeView *tree_view = NULL;
-	List bb_list = NULL;
-	List send_bb_list = NULL;
+	list_t *bb_list = NULL;
+	list_t *send_bb_list = NULL;
 	sview_bb_info_t *sview_bb_info_ptr = NULL;
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 
 	if (!spec_info->display_widget) {
 		setup_popup_info(popup_win, display_data_bb, SORTID_CNT);
@@ -814,7 +814,7 @@ extern void specific_info_bb(popup_info_t *popup_win)
 		if (spec_info->display_widget)
 			gtk_widget_destroy(spec_info->display_widget);
 		sprintf(error_char, "get_new_info_bb: %s",
-			slurm_strerror(slurm_get_errno()));
+			slurm_strerror(errno));
 		label = gtk_label_new(error_char);
 		gtk_table_attach_defaults(popup_win->table,
 					  label,
@@ -901,7 +901,7 @@ end_it:
 	return;
 }
 
-/* creates a popup windo depending on what is clicked */
+/* creates a popup window depending on what is clicked */
 extern void set_menus_bb(void *arg, void *arg2, GtkTreePath *path, int type)
 {
 	GtkTreeView *tree_view = (GtkTreeView *)arg;
@@ -947,7 +947,7 @@ extern void popup_all_bb(GtkTreeModel *model, GtkTreeIter *iter, int id)
 {
 	char *name = NULL;
 	char title[100] = {0};
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	popup_info_t *popup_win = NULL;
 	GError *error = NULL;
 
@@ -1002,7 +1002,7 @@ extern void popup_all_bb(GtkTreeModel *model, GtkTreeIter *iter, int id)
 	default:
 		g_print("Burst Buffer got unknown type %d\n", id);
 	}
-	if (!sview_thread_new((gpointer)popup_thr, popup_win, false, &error)) {
+	if (!sview_thread_new((gpointer)popup_thr, popup_win, &error)) {
 		g_printerr ("Failed to create burst buffer popup thread: %s\n",
 			    error->message);
 		return;
