@@ -127,7 +127,7 @@ def main(argv=None):
             "ERROR: no test files found in current working directory", file=sys.stderr
         )
         return -1
-    # sory by major, minor
+    # sort by major, minor
     tests.sort(key=lambda t: (t[0], t[1]))
 
     # Set begin value
@@ -155,8 +155,11 @@ def main(argv=None):
         testlog_name = f"test{test_id}.log"
         try:
             os.remove(testlog_name + ".failed")
-        except:
+        except Exception:
             pass
+
+        if os.path.exists(testlog_name):
+            os.remove(testlog_name)
         testlog = open(testlog_name, "w+")
 
         if options.time_individual:
@@ -202,9 +205,9 @@ def main(argv=None):
             test_output = testlog.read()
 
             sections = [s for s in test_output.split("=" * 78 + "\n")]
-            header = sections[1]
+            # header = sections[1]
             body = sections[2]
-            footer = "".join(sections[3:])
+            # footer = "".join(sections[3:])
 
             fatals = re.findall(
                 r"(?ms)\[[^\]]+\][ \[]+Fatal[ \]:]+(.*?) \(fail[^\)]+\)$", body
@@ -345,13 +348,13 @@ def test_parser(option, opt_str, value, parser):
         setattr(parser.values, option.dest, [])
 
     # Get a pointer to the option's destination array.
-    l = getattr(parser.values, option.dest)
+    dest = getattr(parser.values, option.dest)
 
     # Split the user's option string into a series of tuples that represent
     # each test, and add each tuple to the destination array.
     splitter = re.compile(r"[,\s]+")
     val = splitter.split(value)
-    test_re = re.compile(r"(test)?((\d+)|\*)\.((\d+)|\*)$")
+    test_re = re.compile(r"(test)?((\d+)|\*)[\._]((\d+)|\*)$")
     for v in val:
         m = test_re.match(v)
         if not m:
@@ -362,7 +365,7 @@ def test_parser(option, opt_str, value, parser):
         minor = m.group(4)
         if minor != "*":
             minor = int(minor)
-        l.append((major, minor))
+        dest.append((major, minor))
 
 
 if __name__ == "__main__":
